/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.basis;

import static de.bernhardjung.dsaprogramm.daten.xml.FernwaffeXmlTags.*;

import java.text.Collator;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;

import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.talente.Kampftechnik;
import de.bernhardjung.dsaprogramm.talente.Talent;

public class Fernkampfwaffe implements Comparable< Fernkampfwaffe > {

    /** Zum Sortieren der Waffen nach Name. */
    private static final Collator COLLATOR = Collator.getInstance();

    /** Die Id der Fernwaffe. */
    private final String id;

    /** Die Bezeichnung der Waffe. */
    private String name;

    /**
     * Die Reichweiten der Waffe. Sehr nah steht an Position 0, extrem weit an
     * Position 4.
     */
    private final List< Integer > reichweiten;

    /** Das Talent mit dem die Waffe gefuehrt wird. */
    private Kampftechnik talent;

    /** Die Trefferpunkte der Waffe. */
    private Tp tp;

    /**
     * Die Trefferpunkte durch Entfernung. Die TP fuer sehr nah stehen an
     * Position 0, die fuer extrem weit an Position 4.
     */
    private final List< Integer > tpPlus;

    public Fernkampfwaffe( final String dieId ) {

        id = dieId;

        if ( I18n.hatLokalisierungstext( id ) ) {

            name = I18n.i18n( id );

        } else {

            name = id;

        }

        reichweiten = new ArrayList< Integer >( 5 );
        Collections.fill( reichweiten, 0 );

        talent = Talent.BOGEN;

        tp = new Tp( "W+4" );

        tpPlus = new ArrayList< Integer >( 5 );
        Collections.fill( tpPlus, 0 );
    }

    /**
     * Sortiert die Waffen nach Name.
     */
    public int compareTo( final Fernkampfwaffe o ) {

        int ergebnis = COLLATOR.compare( getName(), o.getName() );

        if ( 0 == ergebnis ) {

            ergebnis = COLLATOR.compare( getId(), o.getId() );
        }

        return ergebnis;
    }

    @Override
    public boolean equals( final Object obj ) {

        if ( null == obj ) {

            return false;
        }

        if ( getClass().equals( obj.getClass() ) ) {

            return 0 == compareTo( (Fernkampfwaffe) obj );

        } else {

            return false;
        }
    }

    /** @return die Id der Waffe. */
    public String getId() {
        return id;
    }

    /**
     * @return den Namen der Waffe.
     */
    public String getName() {
        return name;
    }

    /**
     * @return die Reichweiten der Waffe. Sehr nah ist an Position 0, extrem
     *         weit an Position 4.
     */
    public List< Integer > getReichweiten() {
        return new ArrayList< Integer >( reichweiten );
    }

    /**
     * @return das Talent mit dem die Waffe gefuehrt wird.
     */
    public Kampftechnik getTalent() {
        return talent;
    }

    /** @return die TP der Waffe. */
    public Tp getTp() {
        return tp;
    }

    /**
     * @return die TP+ der Waffe. Sehr nah ist an Position 0, extrem
     *         weit an Position 4.
     */
    public List< Integer > getTpPlus() {
        return new ArrayList< Integer >( tpPlus );
    }

    /**
     * @param technik
     *            Die Kampftechnik mit der diese Waffe gefuehrt werden soll.
     * @return {@code true}, wenn man die Waffe mit dem Talent ohne Abzuege
     *         fuehren kann, sonst {@code false}. Bei anderen Talenten gibt
     *         es einen Abzug von 5 Punkten auf den Fernkampfwert.
     */
    public final boolean hatHaupttalent( final Kampftechnik technik ) {
        return talent.equals( technik );
    }

    /**
     * @param neuerName der Waffe.
     */
    public void setName( final String neuerName ) {
        this.name = neuerName;
    }

    /**
     * @param neueReichweiten fuer die Waffe. Sehr nah steht an Position 0,
     *          extrem weit an Position 4.
     */
    public void setReichweiten( final List< Integer > neueReichweiten ) {
        reichweiten.clear();

        reichweiten.addAll( neueReichweiten );
    }

    /**
     * @param neuesTalent mit dem die Waffe gefuehrt wird.
     */
    public void setTalent( final Kampftechnik neuesTalent ) {
        talent = neuesTalent;
    }

    /**
     * @param neueTp der Waffe.
     */
    public void setTp( final Tp neueTp ) {
        tp = neueTp;
    }

    /**
     * @param neueTpPlus fuer die Waffe. Sehr nah steht an Position 0,
     *          extrem weit an Position 4.
     */
    public void setTpPlus( final List< Integer > neueTpPlus ) {
        tpPlus.clear();

        tpPlus.addAll( neueTpPlus );
    }

    @Override
    public String toString() {
        return name;
    }

    /**
     * Wandelt eine Liste in einen String um, in dem die Listenelemente durch
     * Komma getrennt sind.
     */
    private String toString( final List< Integer > liste ) {

        final StringBuilder builder = new StringBuilder();

        builder.append( liste.get( 0 ) );

        for ( int i = 1, n = liste.size() ; i < n ; ++i ) {

            builder.append( "," );
            builder.append(  liste.get( i ) );
        }

        return builder.toString();
    }

    public IXMLElement toXml() {

        final IXMLElement element = new XMLElement( TAG_FERNWAFFE );

        element.setAttribute( ATTR_ID, id );

        if ( ! I18n.hatLokalisierungstext( id )
          || ! I18n.i18n( id ).equals( name ) ) {

            element.setAttribute( ATTR_NAME, name );
        }

        element.setAttribute( ATTR_REICHWEITEN, toString( reichweiten ) );

        element.setAttribute( ATTR_TALENT, talent.getId() );

        element.setAttribute( ATTR_TP, tp.toString() );

        element.setAttribute( ATTR_TP_PLUS, toString( tpPlus ) );

        return element;
    }
}
