/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.daten;

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Observable;

import de.bernhardjung.dsaprogramm.basis.Fernkampfwaffe;
import de.bernhardjung.dsaprogramm.basis.Nahkampfwaffe;
import de.bernhardjung.dsaprogramm.basis.Ruestung;
import de.bernhardjung.dsaprogramm.basis.Schild;
import de.bernhardjung.dsaprogramm.bausteine.Heldenbaustein;
import de.bernhardjung.dsaprogramm.bausteine.Kultur;
import de.bernhardjung.dsaprogramm.bausteine.Profession;
import de.bernhardjung.dsaprogramm.bausteine.Rasse;
import de.bernhardjung.dsaprogramm.gui.DialogFehlermeldung;

public class Datenspeicher extends Observable {

    private static Datenspeicher DATENSPEICHER;

    /**
     * Wird an die Observer gesendet, wenn sich die Liste der Kulturen aendert.
     */
    public static final Object KULTUREN = new Object();

    /**
     * Wird an die Observer gesendet, wenn sich die Liste der Professionen
     * aendern.
     */
    public static final Object PROFESSIONEN = new Object();

    /**
     * Wird an die Observer gesendet, wenn sich die Liste der Rassen aendern.
     */
    public static final Object RASSEN = new Object();

    public static final Datenspeicher getInstanz() {
        if ( null == DATENSPEICHER ) {
            try {

                DATENSPEICHER = new Datenspeicher();

            } catch ( final Exception e ) {

                DialogFehlermeldung.zeigeDialog(
                        "Der Datenspeicher konnte nicht initialisiert werden.",
                        e );
            }
        }

        return DATENSPEICHER;
    }

    /** Eine Liste aller Fernwaffen. */
    private final List< Fernkampfwaffe > fernkampfwaffen;

    /** Eine Map in der jede Fernwaffe unter ihrer Id gespeichert ist. */
    private final Map< String, Fernkampfwaffe > fernkampfwaffenMap;

    /** Eine Liste aller Kulturen. */
    private final List< Kultur > kulturen;

    /** Eine Map in der jede Kultur unter ihrer Id gespeichert ist. */
    private final Map< String, Kultur > kulturenMap;

    /** Eine Liste aller Nahkampfwaffen. */
    private final List< Nahkampfwaffe > nahkampfwaffen;

    /** Eine Map in der jede Nahkampfwaffe unter ihrer Id gespeichert ist. */
    private final Map< String, Nahkampfwaffe > nahkampfwaffenMap;

    /** Eine Liste aller initialisierten Rassen. */
    private final List< Rasse > rassen;

    /**
     * Eine Map in der jede Rasse unter ihrer ID gespeichert ist.
     * Jede Rasse, die erzeugt wird kommt in diese Map.
     */
    private final Map< String, Rasse > rassenMap;

    /** Eine Liste aller initalisierten Professionen. */
    private final List< Profession > professionen;

    /** Eine Map in der jede Profession unter ihrer Id gespeichert ist. */
    private final Map< String, Profession > professionenMap;

    /** Eine Liste aller initalisierten Ruestungen. */
    private final List< Ruestung > ruestungen;

    /** Eine Map in der jede Ruestung unter ihrer Id gespeichert ist. */
    private final Map< String, Ruestung > ruestungenMap;

    /** Eine Liste aller initialisierten Schilde. */
    private final List< Schild > schilde;

    /** Ein Map in der jeder Schild unter seiner Id gespeicher ist. */
    private final Map< String, Schild > schildeMap;

    private Datenspeicher() throws Exception {

        fernkampfwaffen = new ArrayList< Fernkampfwaffe >();

        fernkampfwaffenMap = new HashMap< String, Fernkampfwaffe >();

        kulturen = new ArrayList< Kultur >();

        kulturenMap = new HashMap< String, Kultur >();

        nahkampfwaffen = new ArrayList< Nahkampfwaffe >();

        nahkampfwaffenMap = new HashMap< String, Nahkampfwaffe >();

        rassen = new ArrayList< Rasse >( 6 );

        rassenMap = new HashMap< String, Rasse >();

        professionen = new ArrayList< Profession >();

        professionenMap = new HashMap< String, Profession >();

        ruestungen = new ArrayList< Ruestung >();

        ruestungenMap = new HashMap< String, Ruestung >();

        schilde = new ArrayList< Schild >();

        schildeMap = new HashMap< String, Schild >();
    }

    /**
     * Fuegt der Liste der Fernwaffen eine neue Waffe hinzu.
     *
     * @param waffe
     *            die der Liste der Fernwaffen hinzugefuegt werden soll.
     *            Es kann keine Waffe mehrfach zur Liste hinzugefuegt werden.
     */
    public final void einfuegen( final Fernkampfwaffe waffe ) {

        Collections.sort( fernkampfwaffen );

        int index = Collections.binarySearch( fernkampfwaffen, waffe );

        if ( index < 0 ) {

            index = -( index + 1 );

            fernkampfwaffen.add( index, waffe );
        }
    }

    /**
     * Fuegt der Liste der Kulturen eine neue Kultur hinzu.
     *
     * @param kultur
     *            die der Liste der Kulturen hinzugefuegt werden soll. Es kann
     *            keine Kultur mehrfach zur Liste hinzugefuegt werden.
     */
    public final void einfuegen( final Kultur kultur ) {

        Collections.sort( kulturen );

        int index = Collections.binarySearch( kulturen, kultur );

        if ( index < 0 ) {

            index = -( index + 1 );

            kulturen.add( index, kultur );

            setChanged();
            notifyObservers( KULTUREN );
        }
    }

    /**
     * Fuegt der Liste der Nahkampfwaffen eine neue Waffe hinzu.
     *
     * @param waffe
     *            die der Liste der Nahkampfwaffen hinzugefuegt werden soll.
     *            Es kann keine Waffe mehrfach zur Liste hinzugefuegt werden.
     */
    public final void einfuegen( final Nahkampfwaffe waffe ) {

        Collections.sort( nahkampfwaffen );

        int index = Collections.binarySearch( nahkampfwaffen, waffe );

        if ( index < 0 ) {

            index = -( index + 1 );

            nahkampfwaffen.add( index, waffe );
        }
    }

    /**
     * Fuegt der Liste der Professionen eine neue Profession hinzu.
     *
     * @param profession
     *            die der Liste der Professionen hinzugefuegt werden soll. Es kann
     *            keine Profession mehrfach zur Liste hinzugefuegt werden.
     */
    public final void einfuegen( final Profession profession ) {

        Collections.sort( professionen );

        int index = Collections.binarySearch( professionen, profession );

        if ( index < 0 ) {

            index = -( index + 1 );

            professionen.add( index, profession );

            setChanged();
            notifyObservers( PROFESSIONEN );
        }
    }

    /**
     * Fuegt der Liste der Rassen eine neue Rasse hinzu.
     *
     * @param rasse
     *            die der Liste der Rassen hinzugefuegt werden soll. Es kann
     *            keine Rasse mehrfach zur Liste hinzugefuegt werden.
     */
    public final void einfuegen( final Rasse rasse ) {

        Collections.sort( rassen );

        int index = Collections.binarySearch( rassen, rasse );

        if ( index < 0 ) {

            index = -( index + 1 );

            rassen.add( index, rasse );

            setChanged();
            notifyObservers( RASSEN );
        }
    }

    /**
     * Fuegt der Liste der Ruestungen eine neue Ruestung hinzu.
     *
     * @param ruestung
     *            die der Liste der Ruestungen hinzugefuegt werden soll. Es kann
     *            keine Ruestung mehrfach zur Liste hinzugefuegt werden.
     */
    public final void einfuegen( final Ruestung ruestung ) {

        Collections.sort( ruestungen );

        int index = Collections.binarySearch( ruestungen, ruestung );

        if ( index < 0 ) {

            index = -( index + 1 );

            ruestungen.add( index, ruestung );
        }
    }

    /**
     * Fuegt der Liste der Schilde einen neuen Schild hinzu.
     *
     * @param schild
     *            der der Liste der Schilde hinzugefuegt werden soll.
     *            Es kann kein Schild mehrfach zur Liste hinzugefuegt werden.
     */
    public final void einfuegen( final Schild schild ) {

        Collections.sort( schilde );

        int index = Collections.binarySearch( schilde, schild );

        if ( index < 0 ) {

            index = -( index + 1 );

            schilde.add( index, schild );
        }
    }

    /**
     * Entfernt eine Fernwaffe aus der Liste der Waffen.
     *
     * @param waffe die entfernt werden soll.
     */
    public final void entfernen( final Fernkampfwaffe waffe ) {

        fernkampfwaffen.remove( waffe );
    }

    /**
     * Entfernt eine Nahkampfwaffe aus der Liste der Waffen.
     *
     * @param waffe die entfernt werden soll.
     */
    public final void entfernen( final Nahkampfwaffe waffe ) {

        nahkampfwaffen.remove( waffe );
    }

    /**
     * Entfernt eine Ruestung aus der Liste der Ruestungen.
     *
     * @param ruestung die entfernt werden soll.
     */
    public final void entfernen( final Ruestung ruestung ) {

        ruestungen.remove( ruestung );
    }

    public final void entfernen( final Schild schild ) {

        schilde.remove( schild );
    }

    /**
     * Liefert eine bestimmte Fernwaffe.
     *
     * @param id
     *            der gesuchten Fernwaffe.
     * @return eine Fernwaffe.
     */
    public final Fernkampfwaffe getFernkampfwaffe( final String id ) {

        if ( ! fernkampfwaffenMap.containsKey( id ) ) {

            fernkampfwaffenMap.put( id, new Fernkampfwaffe( id ) );
        }

        return fernkampfwaffenMap.get( id );
    }

    /**
     * @return eine sortierte Liste aller Fernwaffen.
     */
    public final List< Fernkampfwaffe > getFernkampfwaffen() {
        return new ArrayList< Fernkampfwaffe >( fernkampfwaffen );
    }

    /**
     * @param id
     *            der gesuchte Kultur.
     * @return die Kultur mit der angegebenen id. Wenn die Kultur noch nicht
     *         existiert wird sie erzeugt.
     */
    public final Kultur getKultur( final String id ) {

        if ( ! kulturenMap.containsKey( id ) ) {

            kulturenMap.put( id, new Kultur( id ) );

        }

        return kulturenMap.get( id );
    }

    /**
     * @return eine sortierte Liste aller Kulturen.
     */
    public final List< Kultur > getKulturen() {
        return new ArrayList< Kultur >( kulturen );
    }

    /**
     * Liefert eine bestimmte Nahkampfwaffe.
     *
     * @param id
     *            der gesuchten Waffe.
     * @return eine Nahkampfwaffe.
     */
    public final Nahkampfwaffe getNahkampfwaffe( final String id ) {

        if ( ! nahkampfwaffenMap.containsKey( id ) ) {

            nahkampfwaffenMap.put( id, new Nahkampfwaffe( id ) );
        }

        return nahkampfwaffenMap.get( id );
    }

    /** @return eine sortierte Liste aller Nahkampfwaffen. */
    public final List< Nahkampfwaffe > getNahkampfwaffen() {
        return new ArrayList< Nahkampfwaffe >( nahkampfwaffen );
    }

    /**
     * @param id
     *            der gesuchte Profession.
     * @return die Profession mit der angegebenen id. Wenn die Profession noch
     *         nicht existiert wird sie erzeugt.
     */
    public final Profession getProfession( final String id ) {

        if ( ! professionenMap.containsKey( id ) ) {

            professionenMap.put( id, new Profession( id ) );

        }

        return professionenMap.get( id );
    }

    /**
     * @return eine sortierte Liste aller Professionen.
     */
    public final List< Profession > getProfessionen() {
        return new ArrayList< Profession >( professionen );
    }

    /**
     * Liefert eine Rasse mit einer bestimmten Id. Eine durch diese Methode
     * gelieferte Rasse steht im Programm noch nicht zur Verfuegung, d.h. sie
     * wird noch nicht von {@link #getRassen()} geliefert. Dazu muss sie erst
     * mit {@link #einfuegen(Rasse)} oder {@link #setRassen(Collection)} zur
     * Liste der Rassen hinzugefuegt werden.
     *
     * @param id
     *            der gesuchte Rasse.
     * @return die Rasse mit der angegebenen id. Wenn die Rasse noch nicht
     *         existiert wird sie erzeugt.
     */
    public final Rasse getRasse( final String id ) {

        if ( ! rassenMap.containsKey( id ) ) {

            rassenMap.put( id, new Rasse( id ) );

        }

        return rassenMap.get( id );
    }

    /**
     * @return eine sortierte Liste aller Rassen.
     */
    public final List< Rasse > getRassen() {
        return new ArrayList< Rasse >( rassen );
    }

    /**
     * Liefert eine bestimmte Ruestung.
     *
     * @param id
     *            der gesuchten Ruestung.
     * @return eine Ruestung.
     */
    public final Ruestung getRuestung( final String id ) {

        if ( ! ruestungenMap.containsKey( id ) ) {

            ruestungenMap.put( id, new Ruestung( id ) );
        }

        return ruestungenMap.get( id );
    }

    /** @return eine sortierte Liste aller Ruestungen. */
    public final List< Ruestung > getRuestungen() {
        return new ArrayList< Ruestung >( ruestungen );
    }

    /**
     * Liefert einen bestimmten Schild.
     *
     * @param id
     *            des gesucht Schildes.
     * @return einen Schild.
     */
    public final Schild getSchild( final String id ) {

        if ( ! schildeMap.containsKey( id ) ) {

            schildeMap.put( id, new Schild( id ) );
        }

        return schildeMap.get( id );
    }

    /** @return eine sortierte Liste aller Schilde. */
    public final List< Schild > getSchilde() {
        return new ArrayList< Schild >( schilde );
    }

    /**
     * @throws Exception
     *             wenn der XML-Parser zum Laden der Daten nicht initialisiert
     *             werden kann.
     */
    public final void ladeDaten() throws Exception {

        Datenlader.ladeDaten();
    }

    /**
     * Bestimmt die ueber den Datenspeicher verfuegbaren Fernwaffen.
     *
     * @param neueWaffen
     *           die im Datenspeicher gespeichert werden.
     */
    final void setFernwaffen( final Collection< Fernkampfwaffe > neueWaffen ) {

        fernkampfwaffen.clear();

        fernkampfwaffen.addAll( neueWaffen );

        Collections.sort( fernkampfwaffen );
    }

    /**
     * Bestimmt die ueber den Datenspeicher verfuegbaren Kulturen.
     *
     * @param neueKulturen
     *            Die Kulturen, die im Datenspeicher gespeichert werden.
     */
    final void setKulturen( final Collection< Kultur > neueKulturen ) {

        kulturen.clear();

        kulturen.addAll( neueKulturen );

        Collections.sort( kulturen, Heldenbaustein.NAME_COMPARATOR );

        setChanged();
        notifyObservers( KULTUREN );
    }

    /**
     * @param neueNahkampfwaffen
     *            die List der Nahkampfwaffen, die im Speicher gespeichert
     *            werden soll.
     */
    final void setNahkampfwaffen(
            final Collection< Nahkampfwaffe > neueNahkampfwaffen ) {

        nahkampfwaffen.clear();

        nahkampfwaffen.addAll( neueNahkampfwaffen );

        Collections.sort( nahkampfwaffen );
    }

    /**
     * Bestimmt die ueber den Datenspeicher verfuegbaren Rassen.
     *
     * @param neueProfessionen die im Datenspeicher gespeichert werden.
     */
    final void setProfessionen(
            final Collection< Profession > neueProfessionen ) {

        professionen.clear();

        professionen.addAll( neueProfessionen );

        Collections.sort( professionen, Heldenbaustein.NAME_COMPARATOR );

        setChanged();
        notifyObservers( PROFESSIONEN );
    }

    /**
     * Bestimmt die ueber den Datenspeicher verfuegbaren Professionen.
     *
     * @param neueRassen die im Datenspeicher gespeichert werden.
     */
    final void setRassen( final Collection< Rasse > neueRassen ) {

        rassen.clear();

        rassen.addAll( neueRassen );

        Collections.sort( rassen, Heldenbaustein.NAME_COMPARATOR );

        setChanged();
        notifyObservers( RASSEN );
    }

    /**
     * Bestimmt die ueber den Datenspeicher verfuegbaren Ruestungen.
     *
     * @param neueRuestungen die im Datenspeicher gespeichert werden.
     */
    final void setRuestungen( final Collection< Ruestung > neueRuestungen ) {
        ruestungen.clear();

        ruestungen.addAll( neueRuestungen );

        Collections.sort( ruestungen );
    }


    /**
     * Bestimmt die ueber den Datenspeicher verfuegbaren Schilde.
     *
     * @param neueSchilde die im Datenspeicher gespeichert werden.
     */
    final void setSchilde( final Collection< Schild > neueSchilde ) {

        schilde.clear();

        schilde.addAll( neueSchilde );

        Collections.sort( schilde );
    }

    public final void speichereDaten( final File verzeichnis ) {

        Datenspeicherer.speichereDaten( verzeichnis );
    }
}
