/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.daten;

import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.*;

import java.util.List;

import net.n3.nanoxml.IXMLElement;
import de.bernhardjung.dsaprogramm.bausteine.Heldenbaustein;
import de.bernhardjung.dsaprogramm.bausteine.bonus.SonderfertigkeitBonus;
import de.bernhardjung.dsaprogramm.bausteine.bonus.Talentbonus;
import de.bernhardjung.dsaprogramm.bausteine.bonus.VerbilligteSonderfertigkeit;
import de.bernhardjung.dsaprogramm.bausteine.bonus.VerbilligteVariante;
import de.bernhardjung.dsaprogramm.bausteine.bonus.VorNachteilBonus;
import de.bernhardjung.dsaprogramm.bausteine.bonus.Zauberbonus;
import de.bernhardjung.dsaprogramm.bausteine.ungeeignete.UngeeigneterVorNachteil;
import de.bernhardjung.dsaprogramm.bausteine.voraussetzungen.Voraussetzung;
import de.bernhardjung.dsaprogramm.daten.factories.bausteine.FactoryEmpfohleneVorNachteileIn;
import de.bernhardjung.dsaprogramm.daten.factories.bausteine.FactoryKostenIn;
import de.bernhardjung.dsaprogramm.daten.factories.bausteine.FactoryLeittalenteIn;
import de.bernhardjung.dsaprogramm.daten.factories.bausteine.FactoryModifikationenIn;
import de.bernhardjung.dsaprogramm.daten.factories.bausteine.FactorySonderfertigkeitBoniIn;
import de.bernhardjung.dsaprogramm.daten.factories.bausteine.FactoryTalentBoniIn;
import de.bernhardjung.dsaprogramm.daten.factories.bausteine.FactoryUngeeigneteVorNachteileIn;
import de.bernhardjung.dsaprogramm.daten.factories.bausteine.FactoryVerbilligteSonderfertigkeitenIn;
import de.bernhardjung.dsaprogramm.daten.factories.bausteine.FactoryVerbilligteVariantenIn;
import de.bernhardjung.dsaprogramm.daten.factories.bausteine.FactoryVorNachteileIn;
import de.bernhardjung.dsaprogramm.daten.factories.bausteine.FactoryVoraussetzungenIn;
import de.bernhardjung.dsaprogramm.daten.factories.bausteine.FactoryZauberBoniIn;
import de.bernhardjung.dsaprogramm.daten.factories.bausteine.FactoryModifikationenIn.Modifikationen;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;

/**
 * Erzeugt einen Heldenbaustein aus dem XML-Dokument.
 *
 * @author Bernhard Jung
 *
 * @param <T> Gibt an ob eine Rasse, Kultur oder Profession geladen wird.
 */
abstract class HeldenbausteinLader< T extends Heldenbaustein > {

    /** Erzeugt die Liste der empfohlenen Vor-/Nachteile. */
    private final FactoryEmpfohleneVorNachteileIn empfohlene;

    /** Erzeugt die Kosten des Bausteins aus dem XML-Dokument. */
    private final FactoryKostenIn kosten;

    /** Erzeugt die Liste der Leittalente aus dem XML-Dokument. */
    private final FactoryLeittalenteIn leittalente;

    /**
     * Erzeugt die Eigenschaft- und Basiswert-Modifikationen aus dem
     * XML-Dokument.
     */
    private final FactoryModifikationenIn modifikationen;

    /** Erzeugt die Nachteile aus dem XML-Dokument. */
    private final FactoryVorNachteileIn nachteile;

    /**
     * Erzeugt die Boni auf die Sonderfertigkeiten aus dem XML-Dokument.
     */
    private final FactorySonderfertigkeitBoniIn sonderfertigkeiten;

    /** Erzeugt die Talent-Boni aus dem XML-Dokument. */
    private final FactoryTalentBoniIn talentBoni;

    /** Erzeugt die Liste der ungeeigneten Vor-/Nachteile. */
    private final FactoryUngeeigneteVorNachteileIn ungeeignete;

    /** Erzeugt die verbilligten Sonderfertigkeiten aus dem XML-Dokument. */
    private final FactoryVerbilligteSonderfertigkeitenIn verbSonderfertigkeiten;

    private final FactoryVerbilligteVariantenIn verbilligteVarianten;

    /** Erzeugt die Voraussetzungen des Bauisteins aus dem XML-Dokument. */
    private final FactoryVoraussetzungenIn voraussetzung;

    /** Erzeugt die Vorteile aus dem XML-Dokument. */
    private final FactoryVorNachteileIn vorteile;

    /** Erzeugt die Zauber aus dem XML-Dokument. */
    private final FactoryZauberBoniIn zauber;

    HeldenbausteinLader() {

        empfohlene = new FactoryEmpfohleneVorNachteileIn();

        kosten = new FactoryKostenIn();

        leittalente = new FactoryLeittalenteIn();

        modifikationen = new FactoryModifikationenIn();

        nachteile = new FactoryVorNachteileIn( TAG_NACHTEIL_BONI );

        sonderfertigkeiten = new FactorySonderfertigkeitBoniIn();

        talentBoni = new FactoryTalentBoniIn();

        ungeeignete = new FactoryUngeeigneteVorNachteileIn();

        verbSonderfertigkeiten = new FactoryVerbilligteSonderfertigkeitenIn();

        verbilligteVarianten = new FactoryVerbilligteVariantenIn();

        voraussetzung = new FactoryVoraussetzungenIn();

        vorteile = new FactoryVorNachteileIn( TAG_VORTEIL_BONI );

        zauber = new FactoryZauberBoniIn();
    }

    /**
     * @param wurzelElement
     *            der XML-Datei.
     * @return den erzeugten Baustein.
     */
    protected T erzeugeBaustein(
            final IXMLElement wurzelElement ) {

        final String id =
            wurzelElement.getFirstChildNamed( TAG_ID ).getContent();

        final T baustein = getBaustein( id );

        // Kosten ermitteln.
        // *****************
        baustein.setKosten( kosten.erzeugen( wurzelElement ) );

        // Voraussetzungen ermitteln.
        // **************************
        final List< Voraussetzung > voraussetzungen;
        voraussetzungen = voraussetzung.erzeugen( wurzelElement );
        baustein.setVoraussetzungen( voraussetzungen );

        // Modifikationen ermitteln.
        // *************************
        final Modifikationen mods = modifikationen.erzeugen( wurzelElement );
        baustein.setBasiswertModifikationen( mods.basiswerte );
        baustein.setEigenschaftModifikationen( mods.eigenschaften );

        // Nachteile ermitteln.
        // ********************
        final List< VorNachteilBonus > nachteilBoni;
        nachteilBoni = nachteile.erzeugen( wurzelElement );
        baustein.setNachteile( nachteilBoni );

        // Vor-/Nachteile ermitteln.
        // *************************
        final List< VorNachteilBonus > vorteilBoni;
        vorteilBoni = vorteile.erzeugen( wurzelElement );
        baustein.setVorteile( vorteilBoni );

        // Die Punkte die evtl. auf Verbindungen verteilt werden duerfen.
        // **************************************************************
        final IXMLElement verbindungenElement =
            wurzelElement.getFirstChildNamed( TAG_VERBINDUNGEN );
        if ( null != verbindungenElement ) {

            final String verbindungenString = verbindungenElement.getContent();

            baustein.setVerbindungen( Integer.valueOf( verbindungenString ) );

        } else {

            baustein.setVerbindungen( 0 );
        }

        // Leittalente ermitteln.
        // **********************
        final List< Talent > talente;
        talente = leittalente.erzeugen( wurzelElement );
        baustein.setLeittalente( talente );

        // Talent-Boni ermitteln.
        // **********************
        final List< Talentbonus > boni = talentBoni.erzeugen( wurzelElement );
        baustein.setTalente( boni );

        // Sonderfertigkeiten ermitteln.
        // *****************************
        final List< SonderfertigkeitBonus > sfBoni;
        sfBoni = sonderfertigkeiten.erzeugen( wurzelElement );
        baustein.setSonderfertigkeiten( sfBoni );

        // verbilligte Sonderfertigkeiten ermitteln.
        // *****************************************
        final List< VerbilligteSonderfertigkeit > verbilligte;
        verbilligte = verbSonderfertigkeiten.erzeugen( wurzelElement );
        baustein.setVerbilligte( verbilligte );

        // verbilligte Sonderfertigkeit Varianten ermitteln.
        // *************************************************
        final List< VerbilligteVariante > varianten;
        varianten = verbilligteVarianten.erzeugen( wurzelElement );
        baustein.setVerbilligteVariante( varianten );

        // Zauber ermitteln.
        // *****************
        final List< Zauberbonus > zauberboni;
        zauberboni = zauber.erzeugen( wurzelElement );
        baustein.setZauberboni( zauberboni );

        // empfohlene Vor-/Nachteile ermitteln.
        // ************************************
        final List< VorNachteil > empfohleneVorNachteile;
        empfohleneVorNachteile = empfohlene.erzeugen( wurzelElement );
        baustein.setEmpfohlene( empfohleneVorNachteile );

        // ungeeignete Vor-/Nachteile ermitteln.
        // *************************************
        final List< UngeeigneterVorNachteil > ungeeigneteVorNachteile;
        ungeeigneteVorNachteile = ungeeignete.erzeugen( wurzelElement );
        baustein.setUngeeignete( ungeeigneteVorNachteile );

        return baustein;
    }

    /**
     * @param name
     *            der Datei aus der der Baustein geladen wurde ohne die
     *            Dateierweiterung '.xml'. Aus 'Mittelaender.xml' wird also
     *            'Mittellaender'.
     * @return ein Baustein.
     */
    protected abstract T getBaustein( final String name );
}
