/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JViewport;

import org.jdesktop.layout.GroupLayout;
import org.jdesktop.layout.GroupLayout.ParallelGroup;
import org.jdesktop.layout.GroupLayout.SequentialGroup;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.erschaffung.ErschaffungManager;
import de.bernhardjung.dsaprogramm.erschaffung.Zahlungsart;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.SonderfertigkeitWert;

/**
 * Ein Panel das eine Liste von Sonderfertigkeiten ({@link SonderfertigkeitWert})
 * anzeigt. Die Liste besteht aus einer Zeile fuer jede Sonderfertigkeit. Jede
 * Zeile hat 3 oder 4 Spalten.
 * <p>
 * Die erste Spalte enthaelt eine Checkbox mit dem Namen der Sonderfertigkeit.
 * Sie dient dazu Sonderfertigkeiten zu aktivieren und zu deaktiveren, d.h. fuer
 * den Helden zu waehlen oder abzuwaehlen.
 * <p>
 * Die zweite Spalte ist optional und wird nur angezeigt wenn die
 * Sonderfertigkeit Varianten besitzt und die Variante des Wertes aenderbar ist .
 * Sie enthaelt eine (editierbare) Combobox mit der man eine Variante waehlen
 * kann.
 * <p>
 * Die dritte und vierte Spalte enthalten je eine Radiobutton mit denen man
 * waehlen kann ob die Sonderfertigkeit mit AP oder GP bezahlt wird.
 * <p>
 * Die ganze Liste ist in eine Scrollpane eingefuegt.
 * <p>
 * Es gibt zwei Modi in denen die Liste betrieben werden kann. Einmal wird eine
 * feste Liste von Sonderfertigkeiten angezeigt. Dies dient dazu
 * Sonderfertigkeiten anzuzeigen aus denen ein Held waehlen kann. Der zweite
 * Modus stellt die Sonderfertigkeiten dar, die ein Held besitzt. Die beiden
 * Modi unterscheiden sich ausser in den dargestellten Sfen in der Art wie die
 * freien Varianten eine Sonderfertigkeit editiert werden. Wird im 1ten Modus
 * eine Variante veraenderts so steht die Sf mit der neuen Variante zur Auswahl.
 * Man kann also einen Haken setzen und sie aktivieren. Wird im 2ten Modus eine
 * Variante veraendert, so wird die Sonderfertigkeit die der Held besitzt
 * veraendert. Der Held erhaelt die neue Variante und verliert die bisherige.
 * Ist die Sonderfertigkeit eine durch Rasse, Kultur oder Profession vorgegebene
 * so wird auch die vorgegebene Sonderfertigkeit umgestellt.
 * <dl>
 * <dt>Beispiel</dt>
 * <dd> Nehmen wir an der Held besitzt Ruestungsgewoehnung Kettenhemd.
 * <p>
 * Stellt man im ersten Modus von Ruestungsgewoehnung Kettenhemd auf
 * Ruestungsgewoehnung Lederharnisch, so behaelt der Held Ruestungsgewoehnung
 * Kettenhemd und man kann jetzt noch Ruestungsgewoehnung Lederharnisch hinzu
 * waehlen.
 * <p>
 * Stellt man im zweiten Modus von Kettenhemd auf Lederharnisch, so verliert der
 * Held die Ruestungsgewoehnung Kettenhemd und erhaelt dafuer die andere
 * Ruestungsgewoehnung. War die Gewoehnung ans Kettenhemd durch die Kultur des
 * Helden vorgegeben so gilt im weiteren die Gewoehnung an den Lederharnisch als
 * vorgegeben. Die Vorgegebenen Sonderfertigkeiten sind natuerlich nicht in
 * jedem Fall so einfach zu aendern.
 * </dd>
 * </dl>
 *
 * @author Bernhard Jung
 */
public class PanelSonderfertigkeitenListe extends JPanel implements Observer {

    /** Ein Text um die AP-Kosten einer Sonderfertigkeit anzuzeigen. */
    private static final String AP_KOSTEN =
        I18n.i18n( "PanelSonderfertigkeitenListe.ApKosten" );

    /** Ein Text um die GP-Kosten einer Sonderfertigkeit anzuzeigen. */
    public static final String GP_KOSTEN =
        I18n.i18n( "PanelSonderfertigkeitenListe.GpKosten" );

    /** Das Icon das neben einer Leitsonderfertigkeit angezeigt wird. */
    public static final ImageIcon ICON_LEITSF = Util.ladeIcon( "leit" );

    /**
     * Das Icon dient als Platzhalter, wenn ein Zauber weder Haus- noch
     * Leitzauber ist.
     */
    public static final ImageIcon ICON_PLATZHALTER =
        Util.ladeIcon( "platzhalter" );

    /**
     * Die Ueberschrift fuer die letzten beiden Spalten mit den Radiobuttons
     * die die Kosten anzeigen.
     */
    private static final String KOSTEN =
        I18n.i18n( "PanelSonderfertigkeitenListe.Kosten" );

    /**
     * Erklaert dem Benutzer, dass eine Sonderfertigkeit nicht mit GP
     * bezahlt werden darf.
     */
    private static final String NICHT_MIT_GP =
        I18n.i18n( "PanelSonderfertigkeitenListe.nichtMitGp" );

    /**
     * Die Uebeschrift fuer die ersten 3 Spalten mit den Sonderfertigkeiten.
     */
    private static final String SONDERFERTIGKEITEN =
        I18n.i18n( "PanelSonderfertigkeitenListe.Sonderfertigkeiten" );

    /**
     * Speichert Regelelemente (Talente, Eigenschaften,...) als Schluessel.
     * Die Listen enthalten Indizes von Werten aus {@link #werte}.
     * Die in der Liste gespeicherten Werte sind von dem entsprechenden
     * Regelelement abhaengig.
     */
    private final Map< Object, List< Integer > > abhaengigkeiten;

    /**
     * Wird aufgerufen wenn fuer eine Sonderfertigkeit gewaehlt wird, sie mit
     * AP zu bezahlen.
     */
    private final ActionListener actionApZahlen;

    /**
     * Wird aufgerufen wenn eine Sonderfertigkeit vom Benutzer an-/abgewaehlt
     * wird, dh wenn auf eine der Checkboxen geklickt wird.
     */
    private final ActionListener actionDeAktivieren;

    /**
     * Wird aufgerufen wenn fuer eine Sonderfertigkeit gewaehlt wird, sie mit
     * GP zu bezahlen.
     */
    private final ActionListener actionGpZahlen;

    /**
     * Wird aufgerufen wenn die Variante eine Sonderfertigkeit veraendert wird,
     * dh wenn in eine Combobox ein Element ausgewaehlt wird.
     */
    private final ActionListener actionVariante;

    /**
     * Wenn im Panel die Werte eines Helden angezeigt werden wird in dieser
     * Variable gemerkt wie viele Sonderfertigkeiten das Panel zeigt, die der
     * Held auch besitzt. Wenn sich diese Anzahl aendert, muss das Panel neu
     * aufgebaut werden.
     */
    private int anzahlGewaehlte;

    /**
     * Die Radiobuttons welche bei der Erschaffung die AP-Kosten der
     * Sonderfertigkeit anzeigen. Der Button an Position i wird in der i-ten
     * Zeile angezeigt und zeigt die Kosten der Sonderfertigkeit die in
     * {@link #werte} an Position i steht.
     */
    private final List< JRadioButton > apKosten;

    /**
     * Die Label, die beim Steigern die AP-Kosten der Sonderfertigkeiten
     * anzeigen.
     */
    private final List< JLabel > apKostenLabel;

    /**
     * Eine fette (bold) Schrift zum Markieren von vorgegebenen
     * Sonderfertigkeiten.
     */
    private final Font fetteSchrift;

    /**
     * Die Radiobuttons welche, bei der Erschaffung, die GP-Kosten der
     * Sonderfertigkeit anzeigen. Der Button an Position i wird in der i-ten
     * Zeile angezeigt und zeigt die Kosten der Sonderfertigkeit die in
     * {@link #werte} an Position i steht.
     */
    private final List< JRadioButton > gpKosten;

    /** Der Held dessen Werte angezeigt und manipuliert werden. */
    private Held held;

    /** Dieses Panel enthaelt die Liste mit den Sonderfertigkeiten. */
    private final JPanel inhalt;

    /** Die Icons, welche die Leitsonderfertigkeiten markieren. */
    private final List< JLabel > leitsonderfertigkeiten;

    /**
     * Bestimmt beim Steigern die Lernmethode die zum Erlernen der
     * Sonderfertigkeiten verwendet wird.
     */
    private final Lernmethoden lernmethoden;

    /**
     * Der Manager fuer die aktuelle Helden-Erschaffung. Hier wird eingetragen
     * ob eine Sonderfertigkeit mit AP oder GP bezahlt wird.
     */
    private ErschaffungManager manager;

    /**
     * Die Checkboxen, die zum (de)aktivieren der Sonderfertigkeiten
     * dienen. Die Checkbox an Position i wird in der i-ten Zeile angezeigt
     * und steht fuer die Sonderfertigkeit die in {@link #werte} an Position
     * i steht.
     */
    private final List< JCheckBox > sonderfertigkeiten;

    /**
     * Gibt an ob das Panel bei der Heldenerschaffung ({@code false}) oder zum
     * Steigern ({@code true}) verwendet wird.
     */
    private final boolean steigern;

    /**
     * Enthaelt die Comboboxen zum auswaehlen der Varianten.
     * Jede Combobox ist unter der Zeile abgelegt in der sie angezeigt wird
     * bzw. unter dem Index des Wertes in {@link #werte} den sie editiert.
     */
    private final Map< Integer, JComboBox > varianten;

    /** Die Werte die im Panel angezeigt werden. */
    private final List< SonderfertigkeitWert > werte;

    /**
     * Gibt an ob das Panel die Werte eines Helden anzeigt oder eine Liste
     * von Sonderfertigkeiten aus der Werte fuer den Held gewaehlt werden
     * koennen.
     */
    private final boolean zeigtHeldenwerte;

    /**
     * @param zeigtHeldenwerte
     *            Gibt an ob im Panel die Werte angezeigt werden, die ein Held
     *            besitzt, oder eine Liste von Werten die mit
     *            {@link #setWerte(List)} bestimmt wird.
     * @param zumSteigern
     *            {@code true}, wenn das Panel zum Steigern des Helden verwendet
     *            wird, {@code false}, wenn das Panel bei der Heldenerschaffung
     *            verwendet wird.
     * @param dieLernmethoden
     *            Gibt beim Steigern die Lernmethode, an mit der die
     *            Sonderfertigkeiten erlernt werden. Wenn das Panel nicht zum
     *            Steigern verwendet wird kann das Argument {@code null} sein.
     */
    public PanelSonderfertigkeitenListe(
            final boolean zeigtHeldenwerte,
            final boolean zumSteigern,
            final Lernmethoden dieLernmethoden ) {

        abhaengigkeiten = new HashMap<Object, List<Integer>>();

        actionDeAktivieren = new ActionDeAktivieren();

        actionApZahlen = new ActionZahlungsart( Zahlungsart.AP );

        actionGpZahlen = new ActionZahlungsart( Zahlungsart.GP );

        if ( zeigtHeldenwerte ) {

            actionVariante = new ActionVarianteHeldenwerte();

        } else {

            actionVariante = new ActionVariante();

        }

        anzahlGewaehlte = 0;

        apKosten = new ArrayList< JRadioButton >();

        apKostenLabel = new ArrayList< JLabel >();

        fetteSchrift = getFont().deriveFont( Font.BOLD );

        gpKosten = new ArrayList< JRadioButton >();

        inhalt = new JPanel();

        leitsonderfertigkeiten = new ArrayList< JLabel >();

        lernmethoden = dieLernmethoden;
        if ( null != lernmethoden ) {
            lernmethoden.addObserver( this );
        }

        sonderfertigkeiten = new ArrayList< JCheckBox >();

        steigern = zumSteigern;

        varianten = new HashMap< Integer, JComboBox >();

        werte = new ArrayList< SonderfertigkeitWert > ();

        this.zeigtHeldenwerte = zeigtHeldenwerte;

        final JScrollPane scrollPane = new JScrollPane( inhalt );
        // Verhindert, dass unter GTK L&F der Hintergrund der ScrollPane
        // weiss wird. Es muss ein neues Color-Objekt erzeugt werden, wenn man
        // einfach das von getBackground() gelieferte Objekt uebernimmt, so
        // funktioniert es nicht.
        final Color color = new Color( this.getBackground().getRGB() );
        scrollPane.getViewport().setBackground( color );

        final JScrollBar scrollBar = scrollPane.getVerticalScrollBar();

        scrollBar.setUnitIncrement( Util.UNIT_INCREMENT );

        final JViewport viewport = scrollPane.getViewport();

        final Dimension groesse = viewport.getPreferredSize();

        groesse.height = 200;

        viewport.setPreferredSize( groesse );

        setLayout( new BorderLayout() );

        add( scrollPane );

        aufbauenBenutzeroberflaeche();
    }

    /**
     * Baut die Benutzeroberflaeche gemaess der darzustellenden {@link #werte}
     * auf.
     */
    private final void aufbauenBenutzeroberflaeche() {

        // falls die Benutzeroberflaeche neue aufgebaut wird muessen erst
        // alle alten Elemente der Oberflaeche entfernt werden.
        inhalt.removeAll();

        gpKosten.clear();
        apKosten.clear();
        apKostenLabel.clear();
        sonderfertigkeiten.clear();
        varianten.clear();

        // Das neue Layout festlegen.
        final GroupLayout layout = new GroupLayout( inhalt );

        layout.setAutocreateContainerGaps( true );

        layout.setAutocreateGaps( true );

        inhalt.setLayout( layout );

        final ParallelGroup hGroup = layout.createParallelGroup();

        layout.setHorizontalGroup( hGroup );

        final SequentialGroup vGroup = layout.createSequentialGroup();

        layout.setVerticalGroup( vGroup );

        SequentialGroup zeileH = layout.createSequentialGroup();
        ParallelGroup zeileV;
        zeileV = layout.createParallelGroup( GroupLayout.BASELINE );

        JLabel ueberschrift = Util.erzeugeUeberschrift( SONDERFERTIGKEITEN );
        zeileH.add( ueberschrift,
                    GroupLayout.PREFERRED_SIZE,
                    GroupLayout.PREFERRED_SIZE,
                    Short.MAX_VALUE );
        zeileV.add( ueberschrift );

        ueberschrift = Util.erzeugeUeberschrift( KOSTEN );
        final Component[] kosten = new Component[ 1 + werte.size() ];
        kosten[ 0 ] = ueberschrift;
        if ( steigern ) {
            zeileH.add( ueberschrift );
        } else {
            zeileH.add( ueberschrift,
                        125,
                        125,
                        GroupLayout.PREFERRED_SIZE );
        }
        zeileV.add( ueberschrift );

        hGroup.add( zeileH );
        vGroup.add( zeileV );

        // Alle 5 Zeilen wird ein Trennstrich eingefuegt, das erleichtert dem
        // Benutzer die Orientierung in den Zeilen der Liste.
        final int extraZeilen = 1 + ( werte.size() / 4 );

        // Jetzt die Liste Zeile fuer Zeile aufbauen.
        for ( int i = 0, n = werte.size() + extraZeilen, trennstriche = 0
                ; i < n
                ; ++i ) {

            zeileH = layout.createSequentialGroup();
            zeileV = layout.createParallelGroup( GroupLayout.CENTER, false );

            if ( ( i % 5 ) == 0 ) {
                // ein Trennstrich wird eingefuegt.

                final JSeparator separator = new JSeparator();

                zeileH.add( separator );
                zeileV.add( separator,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE );

                hGroup.add( zeileH );
                vGroup.add( zeileV );

                ++trennstriche;

                continue;
            }

            // Das ist der index des Wertes in werte, der in der aktuellen
            // Zeile dargestellt wird.
            final int index = i - trennstriche;

            final String indexString = String.valueOf( index );

            final SonderfertigkeitWert wert = werte.get( index );

            final Sonderfertigkeit sonderfertigkeit;
            sonderfertigkeit = wert.getSonderfertigkeit();

            // Die Markierung fuer die Leitsonderfertigkeiten
            final JLabel leitsf = new JLabel( ICON_PLATZHALTER );

            leitsonderfertigkeiten.add( leitsf );

            zeileH.add( leitsf );
            zeileV.add( leitsf );

            // Erste und evtl. Zweite Spalte aufbauen.
            if ( sonderfertigkeit.hatVarianten()
              && wert.istVarianteAenderbar() ) {

                // Hat der Wert eine Variante wird neben der Checkbox
                // eine Combobox benoetigt um die Variante waehlen zu koennen.

                final JCheckBox checkBox = new JCheckBox();

                checkBox.setText(  sonderfertigkeit.getName() );

                checkBox.setName( indexString );

                checkBox.addActionListener( actionDeAktivieren );

                sonderfertigkeiten.add( checkBox );

                zeileH.add( checkBox );
                zeileV.add( checkBox,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE );

                final VariantenListModel model = new VariantenListModel();

                model.setItems( sonderfertigkeit.getVarianten(held) );

                final JComboBox comboBox;
                comboBox = new JComboBox( model );

                comboBox.setEditable( sonderfertigkeit.hatFreieVarianten() );

                comboBox.setName( indexString );

                comboBox.addActionListener( actionVariante );

                varianten.put( index, comboBox );

                zeileH.add( comboBox, 100, 100, Short.MAX_VALUE );
                zeileV.add( comboBox,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE );

            } else {

                final JCheckBox checkBox = new JCheckBox( wert.getName() );

                checkBox.setName( indexString );

                checkBox.addActionListener( actionDeAktivieren );

                sonderfertigkeiten.add( checkBox );

                zeileH.add( checkBox,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE,
                        Short.MAX_VALUE );
                zeileV.add( checkBox,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE );

            }

            if ( steigern ) {

                final JLabel ap = new JLabel();

                apKostenLabel.add( ap );

                kosten[ 1 + index ] = ap;

                zeileH.add( ap );
                zeileV.add( ap,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE );

            } else {
                // Die dritte und vierte Spalte mit den Radiobuttons aufbauen.
                final ButtonGroup buttonGroup = new ButtonGroup();

                final JRadioButton ap = new JRadioButton();

                ap.setActionCommand( indexString );

                ap.setSelected( true );

                ap.addActionListener( actionApZahlen );

                buttonGroup.add( ap );

                apKosten.add( ap );

                zeileH.add( ap,
                        70,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE );
                zeileV.add( ap,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE );

                final JRadioButton gp = new JRadioButton();

                gp.setActionCommand( indexString );

                gp.addActionListener( actionGpZahlen );

                buttonGroup.add( gp );

                if ( ! sonderfertigkeit.hatGpKosten() ) {

                    gp.setForeground( Util.FEHLERFARBE );

                    gp.setToolTipText( NICHT_MIT_GP );

                }

                gpKosten.add( gp );

                zeileH.add( gp,
                        55,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE );
                zeileV.add( gp,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE );
            }

            hGroup.add( zeileH );
            vGroup.add( zeileV );
        }

        if ( 0 < werte.size() ) {

            if ( steigern ) {

                layout.linkSize( kosten, GroupLayout.HORIZONTAL );

            } else {

                layout.linkSize(
                        gpKosten.toArray( new Component[ gpKosten.size() ] ),
                        GroupLayout.HORIZONTAL );

                layout.linkSize(
                        apKosten.toArray( new Component[ apKosten.size() ] ),
                        GroupLayout.HORIZONTAL );
            }
        }
    }

    /**
     * Initialisiert {@link #abhaengigkeiten} neu.
     */
    private void initAbhaengigkeiten() {

        abhaengigkeiten.clear();

        for ( int i = 0, n = werte.size(); i < n ; ++i ) {

            final Sonderfertigkeit sf = werte.get( i ).getSonderfertigkeit();

            for ( final Object regelelement : sf.getAbhaengigkeiten() ) {

                setAbhaengigkeit( regelelement, i );
            }
        }
    }

    /**
     * Macht einen Wert in {@link #werte} von einem Regelelement abhaengig.
     *
     * @param regelelement
     *            von dem der Wert abhaengig ist.
     * @param index
     *            des Wertes in {@link #werte}.
     */
    private void setAbhaengigkeit(
            final Object regelelement,
            final int index ) {

        if ( ! abhaengigkeiten.containsKey( regelelement ) ) {

            abhaengigkeiten.put( regelelement, new ArrayList< Integer>() );
        }

        abhaengigkeiten.get( regelelement ).add( index );
    }

    /**
     * @param derHeld der ueber das Panel editiert wird.
     */
    public final void setHeld( final Held derHeld ) {

        held = derHeld;

        held.addObserver( this );

        anzahlGewaehlte = -1;

        update( null, null );

        for ( int i = werte.size()-1 ; i >= 0 ; --i ) {

            update( i );
        }
    }

    /**
     * Bestimmt den Manager und damit auch den Held dessen Werte ueber die
     * Benutzerschnittstelle editiert werden.
     *
     * @param derManager des aktuell erschaffenen Helden.
     */
    final void setManager( final ErschaffungManager derManager ) {

        manager = derManager;

        manager.addObserver( this );

        setHeld( manager.getHeld() );
    }

    /**
     * Aendert die Werte die in der Liste dargestellt werden.
     *
     * @param dieWerte die angezeigt werden.
     */
    final void setWerte( final Collection< SonderfertigkeitWert > dieWerte ) {

        werte.clear();

        werte.addAll( dieWerte );

        initAbhaengigkeiten();

        aufbauenBenutzeroberflaeche();
    }

    /**
     * Aktualisiert die Zeile eines dargestellten Werts.
     *
     * @param i
     *            Der Index des Wertes in {@link #werte}.
     */
    private void update( final int i ) {

        final SonderfertigkeitWert wert = werte.get( i );

        if ( held.hatLeitsonderfertigkeit( wert ) ) {

            leitsonderfertigkeiten.get( i ).setIcon( ICON_LEITSF );

        } else {

            leitsonderfertigkeiten.get( i ).setIcon( ICON_PLATZHALTER );

        }

        final JCheckBox checkBox = sonderfertigkeiten.get( i );

        checkBox.setSelected( held.hat( wert ) );

        final KonfliktBeschreibung beschreibung = wert.pruefen( held );

        if ( beschreibung.enthaeltKonflikt() ) {

            checkBox.setForeground( Util.FEHLERFARBE );
            checkBox.setToolTipText( beschreibung.toHtml() );

        } else {

            checkBox.setForeground( getForeground() );
            checkBox.setToolTipText( null );

        }

        if ( held.hatVorgegeben( wert ) ) {

            checkBox.setFont( fetteSchrift );

        } else {

            checkBox.setFont( getFont() );

        }

        final Sonderfertigkeit sonderfertigkeit;
        sonderfertigkeit = wert.getSonderfertigkeit();

        if ( sonderfertigkeit.hatVarianten()
          && wert.istVarianteAenderbar() ) {

            final JComboBox comboBox = varianten.get( i );

            comboBox.removeActionListener( actionVariante );

            final VariantenListModel model;
            model = (VariantenListModel) comboBox.getModel();
            model.setItems( sonderfertigkeit.getVarianten(held) );

            final Object variante = wert.getVariante();

            model.setSelectedItem( variante );

            comboBox.addActionListener( actionVariante );

            checkBox.setEnabled( ! variante.equals( "" ) );

        }

        if ( steigern ) {

            final JLabel ap = apKostenLabel.get( i );

            final int apKosten;

            if ( held.hat( wert ) ) {

                apKosten = held.getWert( wert ).getLernkosten();

            } else {
                apKosten =
                    wert.getApKosten( held, lernmethoden.getLernmethode() );
            }

            ap.setText( String.format( AP_KOSTEN, apKosten ) );

        } else {

            final JRadioButton ap = apKosten.get( i );

            ap.setText( String.format( AP_KOSTEN, wert.getApKosten( held ) ) );

            ap.setSelected( ! manager.wirdMitGpBezahlt( wert ) );

            final JRadioButton gp = gpKosten.get( i );

            gp.setText( String.format( GP_KOSTEN, wert.getGpKosten( held ) ) );

            gp.setSelected( manager.wirdMitGpBezahlt( wert ) );
        }
    }

    /**
     * Aktualisiert die Anzeige wenn sich die Werte des Helden geaendert haben.
     *
     * @param o ungenutzt.
     * @param arg Das Regelelement des Helden, das veraendert wurde.
     */
    public void update( final Observable o, final Object arg ) {

        if ( zeigtHeldenwerte ) {

            final List< SonderfertigkeitWert > gewaehlte;
            gewaehlte = held.getSonderfertigkeiten();

            if ( gewaehlte.size() != anzahlGewaehlte ) {

                // Die Liste der Werte die der Held besitzt hat sich gaendert.
                // Die ganze Liste muss neu aufgebaut werden.

                anzahlGewaehlte = gewaehlte.size();

                final List< SonderfertigkeitWert > werte;
                werte = new ArrayList< SonderfertigkeitWert >();

                werte.addAll( held.getFehlendeSonderfertigkeiten() );

                werte.addAll( gewaehlte );

                setWerte( werte );

                for ( int i = werte.size()-1 ; i >= 0 ; --i ) {

                    update( i );
                }
            }

        }

        if ( Lernmethoden.LERNMETHODE.equals( arg ) ) {

            for ( int i = 0, n = werte.size() ; i < n ; ++i ) {
                update( i );
            }

        } else if ( abhaengigkeiten.containsKey( arg ) ) {

            for ( final int i : abhaengigkeiten.get( arg ) ) {

                update( i );
            }
        }

    }

    /**
     * Wird aufgerufen wenn eine Sonderfertigkeit (ab)gewaehlt wird.
     *
     * @author Bernhard Jung
     */
    private class ActionDeAktivieren implements ActionListener {

        public final void actionPerformed( final ActionEvent e ) {

            final JCheckBox checkBox = (JCheckBox) e.getSource();

            final int index = Integer.valueOf( checkBox.getName() );

            final SonderfertigkeitWert wert = werte.get( index );

            if ( checkBox.isSelected() ) {

                if ( steigern ) {
                    final int lernkosten =
                        wert.getApKosten( held, lernmethoden.getLernmethode() );

                    wert.setLernkosten( lernkosten );
                }

                held.einfuegen( wert );

            } else {

                held.entfernen( wert );

            }
        }
    }

    /**
     * Wird aufgerufen wenn die Variante einer Sonderfertigkeit geaendert wird.
     * Dies Klasse ist fuer Modus 1 gedacht. Siehe Dokumentation von
     * {@link PanelSonderfertigkeitenListe}.
     *
     * @author Bernhard Jung
     */
    private class ActionVariante implements ActionListener {

        public void actionPerformed( final ActionEvent e ) {

            final JComboBox comboBox = (JComboBox) e.getSource();

            Object variante = comboBox.getSelectedItem();

            if ( null == variante ) {
                // keine gueltige Variante gewaehlt. Abbruch.
                return;
            }

            if ( variante instanceof String ) {
                variante = ((String)variante).trim();
            }

            final int index = Integer.valueOf( comboBox.getName() );

            final SonderfertigkeitWert wert = werte.get( index );

            // nur wenn eine neue Variante gewaehlt wurde und ...
            if ( ! wert.getVariante().equals( variante )
                    // ... es eine sinnvolle Variante ist.
                    && ( ! variante.equals( "" ) ) ) {

                if ( variante instanceof String ) {
                    final Sonderfertigkeit sonderfertigkeit;
                    sonderfertigkeit = wert.getSonderfertigkeit();

                    sonderfertigkeit.einfuegenVariante( (String) variante );
                }

                if ( held.getVerbilligteVariante( wert ) == wert
                        && held.hat( wert ) ) {
                    // Hier wird gerade auf der Registerkarte 'verbilligte'
                    // eine verbilligte Sonderfertigkeit geaendert.
                    // Wenn der der Held die bisher verbilligte Sf besitzt,
                    // so soll diese Sonderfertigkeit, wenn moeglich, mit
                    // geaendert werden.
                    final SonderfertigkeitWert heldenwert;
                    heldenwert = held.getWert( wert );

                    if ( heldenwert.istVarianteAenderbar() ) {

                        heldenwert.setVariante( variante );
                    }

                }

                if ( held.getLeitsonderfertigkeit( wert ) == wert
                        && held.hat( wert ) ) {
                    // Wie oben bei verbilligte Sonderfertigkeit nur fuer die
                    // Leitsonderfertigkeiten.
                    final SonderfertigkeitWert heldenwert;
                    heldenwert = held.getWert( wert );

                    if ( heldenwert.istVarianteAenderbar() ) {

                        heldenwert.setVariante( variante );
                    }

                }

                wert.setVariante( variante );

                // Es koennte eine neue verbilligte Variante gewaehlt worden
                // sein und eine gewaehlte Variante koennte dadurch verbilligt
                // werden, dann muss die Anzeige der Kosten aktualisiert werden.
                // Beispiel: Der Held besitzt Ortskenntnis (Andergast) dies wird
                // in der Liste der gewaehlten Sf mit den Kosten 150 AP
                // angezeigt.
                // Der Benutzer waehlt jetzt Ortskenntnis (Andergast) als
                // verbilligt. Damit muss die Anzeige der Kosten in der der Liste
                // auf 75 AP aktualisiert werden.
                // Das wird durch den folgenden Aufruf erreicht.
                held.notifyObservers( wert.getSonderfertigkeit() );
            }

            update( index );
        }
    }

    /**
     * Wird aufgerufen wenn die Variante einer Sonderfertigkeit geaendert wird.
     * Diese Klasse ist fuer Modus 2 gedacht, in dem die Benutzeroberflaeche
     * die Sonderfertigkeiten eines Helden anzeigt. Siehe Dokumentation von
     * {@link PanelSonderfertigkeitenListe}.
     *
     * @author Bernhard Jung
     */
    private class ActionVarianteHeldenwerte implements ActionListener {

        public void actionPerformed( final ActionEvent e ) {

            final JComboBox comboBox = (JComboBox) e.getSource();

            Object variante = comboBox.getSelectedItem();

            if ( null == variante ) {
                // keine gueltige Variante gewaehlt. Abbruch.
                return;
            }

            if ( variante instanceof String ) {
                variante = ( (String)variante ).trim();
            }

            final int index = Integer.valueOf( comboBox.getName() );

            final SonderfertigkeitWert wert = werte.get( index );

            // Der testwert ist da um mit  held.hat( testwert ) zu pruefen ob
            // der Held den Wert in dieser Variante schon besitzt.
            final SonderfertigkeitWert testwert = wert.clone();

            testwert.setHeld( null );

            testwert.setVariante( variante );

            // nur wenn eine neue Variante gewaehlt wurde, ...
            if ( ! wert.getVariante().equals( variante )
                    // ... es eine sinnvolle Variante ist, ...
                    && ( ! variante.equals( "" ) )
                    // ... der Held die Variante noch nicht besitzt und ...
                    && ( ! held.hat( testwert ) )
                    // ... die Variante auch nicht vorgegeben ist.
                    // Warum dies letzte?
                    // Wenn dieser Ausdruck hier ausgewertet wird beutet dies
                    // der Held besitzt den Wert nicht, sonst haetten wir in der
                    // vorhergehenden Abfrage ein false erhalten und die &&
                    // Auswertung waere abgebrochen worden (&& ist lazy).
                    // Der Held besitzt den Wert also nicht. Ist der Wert
                    // vorgegeben so bedeutet dies, dass der Wert aktuell in der
                    // Liste der fehlenden, vorgegebenen Werte gespeichert ist.
                    // Werte die da gespeichert sind darf der Held nur erhalten,
                    // indem ein Wert mit held.einfuegen() hinzugefuegt wird und
                    // nicht indem man die Variante eines Wertes aendert.
                    // So wie es aktuell programmiert ist ist das halt so.
                    && ( ! held.hatVorgegeben( testwert ) ) ) {

                final Sonderfertigkeit sonderfertigkeit;
                sonderfertigkeit = wert.getSonderfertigkeit();

                if ( ( variante instanceof String )
                  && sonderfertigkeit.hatFreieVarianten() ) {

                    sonderfertigkeit.einfuegenVariante( (String) variante );
                }

                // Hier ist der Unterschied zur Klasse ActionVariante zu
                // finden. Wenn die Variante einer fuer den Helden
                // vorgegebenen Sonderfertigkeit geaendert wird, so bedeutet
                // das in diesem Fall, dass die geaenderte Variante als
                // vorgegeben gilt.
                // Das ist fuer den Fall geadacht der bei der Profession
                // Einbrecher auftaucht. Der Einbrecher erhaelt die Sf
                // Ortskenntnis (bevorzugtes Gebiet fuer Fischzuege), dabei
                // muss der Text "bevorzugtes Gebiet fuer Fischzuege"
                // natuerlich durch eine echte Oertlichkeit ersetzt werden.
                // Aehnlich verhaelt es sich mit den verbilligten
                // Sonderfertigkeiten.
                if ( held.hatVorgegeben( wert ) ) {

                    held.getVorgegeben( wert ).setVariante( variante );

                }

                if ( held.hatVerbilligtVariante( wert ) ) {

                    held.getVerbilligteVariante( wert ).setVariante( variante );

                }

                if ( held.hatLeitsonderfertigkeit( wert ) ) {

                    held.getLeitsonderfertigkeit( wert ).setVariante( variante );

                }

                wert.setVariante( variante );

            } else {

                // Wenn der Benutzer einen leeren String eingegeben und Return
                // gedrueckt hat, soll der urspruengliche Eintrag wieder im
                // Textfeld erscheinen um dem Benutzer zu zeigen, dass der leere
                // String nicht uebernommen wird.
                update( index );
            }
        }
    }

    /**
     * Wird aufgerufen wenn vom Benutzer eingestellt wird wie eine
     * Sonderfertigkeit bezahlt wird.
     *
     * @author Bernhard Jung
     */
    private class ActionZahlungsart implements ActionListener {

        private final Zahlungsart zahlungsart;

        ActionZahlungsart( final Zahlungsart dieZahlungsart ) {
            zahlungsart = dieZahlungsart;
        }

        public void actionPerformed( final ActionEvent e ) {

            final int index = Integer.valueOf( e.getActionCommand() );

            final SonderfertigkeitWert wert = werte.get( index );

            manager.setZahlungsart( wert, zahlungsart );
        }

    }
}
