/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;

import javax.swing.ImageIcon;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JSpinner;
import javax.swing.JViewport;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.jdesktop.layout.GroupLayout;
import org.jdesktop.layout.GroupLayout.ParallelGroup;
import org.jdesktop.layout.GroupLayout.SequentialGroup;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.basis.SKT;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.zauber.Zauber;
import de.bernhardjung.dsaprogramm.zauber.ZauberWert;

class PanelZauberListe extends JPanel implements Observer {

    /** Ein String um die Aktivierungs-/Steigerungskosten anzuzeigen. */
    public static final String AP = I18n.i18n( "PanelZauberListe.AP" );

    /** Die Ueberschrift fuer die Spalte mit den Steigerungskosten. */
    public static final String KOSTEN = I18n.i18n( "PanelZauberListe.Kosten" );

    /** Die Ueberschrift fuer die Spalte mit den Komplexitaeten der Zauber. */
    public static final String FAKTOR =
        I18n.i18n( "PanelZauberListe.Steigerungsfaktor" );

    /** Das Icon das neben einem Hauszauber angezeigt wird. */
    public static final ImageIcon ICON_HAUSZAUBER =
        Util.ladeIcon( "hauszauber" );

    /** Das Icon das neben einem Leitzauber angezeigt wird. */
    public static final ImageIcon ICON_LEITZAUBER = Util.ladeIcon( "leit" );

    /**
     * Das Icon dient als Platzhalter, wenn ein Zauber weder Haus- noch
     * Leitzauber ist.
     */
    public static final ImageIcon ICON_PLATZHALTER =
        Util.ladeIcon( "platzhalter" );

    /** Die Ueberschrift fuer die Spalte mit den Proben. */
    private static final String PROBE = I18n.i18n( "PanelZauberListe.Probe" );

    /** Die Ueberschrift fuer die Spalte mit den Speziellen Erfahrungen. */
    private static final String SPEZIELLE_ERFAHRUNG =
        I18n.i18n( "PanelZauberListe.SpezielleErfahrung" );

    /** Der Tooltip-Text fuer das Icon, das einen Hauszauber anzeigt. */
    private static final String TEXT_HAUSZAUBER =
        I18n.i18n( "PanelZauberListe.Hauszauber" );

    /** Der Tooltip-Text fuer das Icon, das einen Leitzauber anzeigt. */
    private static final String TEXT_LEITZAUBER =
        I18n.i18n( "PanelZauberListe.Leitzauber" );

    /** Die Ueberschrift fuer die Spalte mit den Zaubern. */
    public static final String ZAUBER = I18n.i18n( "PanelZauberListe.Zauber" );

    /** Die Ueberschrift fuer die Spalte mit den Zauberstufen. */
    public static final String ZFW = I18n.i18n( "PanelZauberListe.ZfW" );

    /**
     * Speichert Regelemente, wie zB eine Eigenschaft als Schluessel und
     * Indizes fuer Zauber aus {@link #werte} als Wert.
     * Die Indizes geben die Zauber an die von dem Regelelement abhaengig sind.
     * Wenn ein Schluessel, z.B. eine Eigenschaft, geaendert wird, muss
     * die Anzeige fuer alle davon abhaengigen Zauber aktualisiert werden.
     */
    private final Map< Object, List< Integer > > abhaengigkeiten;

    /** Wird aufgerufen, wenn ein Wert an-/abgewaehlt wird. */
    private final ActionListener actionDeAktivieren;

    /** Wird aufgerufen, wenn die Variante eines Zaubers geaendert wird. */
    private final ActionListener actionVariante;

    /** Wird aufgerufen, wenn eine Spezielle Erfahrung vergeben wird. */
    private final ActionListener actionSpezielleErfahrung;

    /** Wird aufgerufen, wenn die Stufe eines Wertes geaendert wird. */
    private final ChangeListener actionStufe;

    /**
     * Die JCheckBoxen die in der ersten Spalte stehen und mit denen angezeigt
     * wird ob das Zauber aktiviert ist.
     * Die i-te Checkbox in der Oberflaeche (von oben gezaehlt) steht an
     * Listen-Position i.
     */
    private final List< JCheckBox > aktiviert;

    private int anzahlGewaehlte;

    /** Eine fette (bold) Schrift. */
    private final Font fetteSchrift;

    /** Der Held dessen Wert angezeigt und manipuliert werden. */
    private Held held;

    /**
     * Das Panel, das die ganzen Elemente der Benutzeroberflaeche enthaelt.
     */
    private final JPanel inhalt;

    /**
     * Die JLabel mit denen die Steigerungskosten angezeigt werden.
     * Das i-te Label in der Oberflaeche (von oben gezaehlt) steht an
     * Listen-Position i.
     */
    private final List< JLabel > kosten;

    /**
     * Liefert beim Steigern die Lernmethode die verwendet wird.
     */
    private final Lernmethoden lernmethoden;

    /**
     * Die Icons, die anzeigen wenn ein Zauber Hauszauber oder Leitzauber ist.
     */
    private final List< JLabel > markierungen;

    /**
     * Enthaelt die Checkboxen mit denen eine spezielle Erfahrung angegeben
     * werden kann.
     */
    private final List< JCheckBox > spezielleErfahrung;

    /**
     * Gibt an ob der Panel beim Steigern ({@code true}) oder bei der
     * Heldenerschaffung ({@code false}) verwendet wird.
     */
    private final boolean steigern;

    /**
     * Die JLabel die die Steigerungsfaktoren der Zauber anzeigen.
     * Das i-te Label in der Oberflaeche (von oben gezaehlt) steht an
     * Listen-Position i.
     */
    private final List< JLabel > steigerungsfaktoren;

    /**
     * Die JSpinner mit denen die Stufen der Zauber angezeigt und manipuliert
     * werden.
     * Der Spinner an Listen-Position i steht in der Oberflaeche an i-ter
     * Stelle (von oben gezaehlt).
     */
    private final List< JSpinner > stufen;

    /**
     * Eine Map in der fuer eine Zeile, die zugehoerige Combobox gespeichert
     * ist um die Variante zu waehlen.
     */
    private final Map< Integer, JComboBox > varianten;

    /**
     * Die angezeigten Zauber.
     * Der Zauber an Position i wird in der Benutzeroberflaech in Zeile i
     * angezeigt. Die Zeilen mit den Trennstrichen nicht mitgerechnet.
     */
    private final List< ZauberWert > werte;

    /** Gibt an ob die Werte eines Helden angezeigt werden. */
    private final boolean zeigtHeldenwerte;

    /**
     * Erzeugt ein neues Panel.
     *
     * @param zeigtHeldenwerte
     *            {@code true}, wenn das Panel nur die Zauber anzeigen soll, die
     *            der Held besitzt, {@code false}, wenn das Panel eine Liste von
     *            Zaubern anzeigen soll, die mit {@link #setWerte(List)}
     *            festgelegt werden.
     * @param zumSteigern
     *            {@code true}, wenn das Panel beim Steigern eines Helden
     *            eingesetzt wird, {@code false}, wenn das Panel bei der
     *            Heldenerschaffung eingesetzt wird.
     * @param dieLernmethoden
     *            Legt beim Steigern die Lernmethode fest mit der die Zauber
     *            gesteigert werden.
     */
    PanelZauberListe(
            final boolean zeigtHeldenwerte,
            final boolean zumSteigern,
            final Lernmethoden dieLernmethoden ) {

        super( new BorderLayout() );

        werte = new ArrayList< ZauberWert >();

        abhaengigkeiten = new HashMap< Object, List< Integer > >();

        actionDeAktivieren = new ActionDeAktivieren();

        actionSpezielleErfahrung = new ActionSpezielleErfahrung();

        actionStufe = new ActionStufe();

        if ( zeigtHeldenwerte ) {

            actionVariante = new ActionVarianteHeldenwerte();

        } else {

            actionVariante = new ActionVariante();
        }

        aktiviert = new ArrayList< JCheckBox >();

        anzahlGewaehlte = -1;

        fetteSchrift = getFont().deriveFont( Font.BOLD );

        inhalt = new JPanel();

        markierungen = new ArrayList< JLabel >();

        kosten = new ArrayList< JLabel >();

        lernmethoden = dieLernmethoden;
        if ( null != lernmethoden ) {
            lernmethoden.addObserver( this );
        }

        spezielleErfahrung = new ArrayList< JCheckBox >();

        steigern = zumSteigern;

        steigerungsfaktoren = new ArrayList< JLabel >();

        stufen = new ArrayList< JSpinner >();

        varianten = new HashMap< Integer, JComboBox >();

        this.zeigtHeldenwerte = zeigtHeldenwerte;

        final JScrollPane scrollPane = new JScrollPane( inhalt );
        // Verhindert, dass unter GTK L&F der Hintergrund der ScrollPane
        // weiss wird. Es muss ein neues Color-Objekt erzeugt werden, wenn man
        // einfach das von getBackground() gelieferte Objekt uebernimmt, so
        // funktioniert es nicht.
        final Color color = new Color( this.getBackground().getRGB() );
        scrollPane.getViewport().setBackground( color );

        final JScrollBar scrollBar = scrollPane.getVerticalScrollBar();

        scrollBar.setUnitIncrement( Util.UNIT_INCREMENT );

        final JViewport viewport = scrollPane.getViewport();

        final Dimension groesse = viewport.getPreferredSize();

        groesse.height = 200;

        viewport.setPreferredSize( groesse );

        setLayout( new BorderLayout() );

        add( scrollPane );

        aufbauenBenutzeroberflaeche();
    }

    /**
     * Baut die Benutzeroberflaeche auf.
     */
    private final void aufbauenBenutzeroberflaeche() {

        inhalt.removeAll();

        initElemente();

        // Das neue Layout festlegen.
        final GroupLayout layout = new GroupLayout( inhalt );

        layout.setAutocreateContainerGaps( true );

        layout.setAutocreateGaps( true );

        inhalt.setLayout( layout );

        final ParallelGroup hGroup = layout.createParallelGroup();

        layout.setHorizontalGroup( hGroup );

        final SequentialGroup vGroup = layout.createSequentialGroup();

        layout.setVerticalGroup( vGroup );

        SequentialGroup zeileH = layout.createSequentialGroup();
        ParallelGroup zeileV;
        zeileV = layout.createParallelGroup( GroupLayout.BASELINE );

        final Component[] faktorArray = new Component[ 1 + werte.size() ];
        final Component[] probenArray = new Component[ 1 + werte.size() ];
        final Component[] stufenArray = new Component[ 1 + werte.size() ];
        final Component[] kostenArray = new Component[ 1 + werte.size() ];
        final Component[] spzErfArray = new Component[ 1 + werte.size() ];

        JLabel ueberschrift = Util.erzeugeUeberschrift( ZAUBER );
        zeileH.add( ueberschrift,
                    GroupLayout.PREFERRED_SIZE,
                    GroupLayout.PREFERRED_SIZE,
                    Short.MAX_VALUE );
        zeileV.add( ueberschrift );

        ueberschrift = Util.erzeugeUeberschrift( FAKTOR );
        faktorArray[ 0 ] = ueberschrift;
        zeileH.add( ueberschrift,
                    GroupLayout.PREFERRED_SIZE,
                    GroupLayout.PREFERRED_SIZE,
                    GroupLayout.PREFERRED_SIZE );
        zeileV.add( ueberschrift );

        ueberschrift = Util.erzeugeUeberschrift( PROBE );
        probenArray[ 0 ] = ueberschrift;
        zeileH.add( ueberschrift,
                    GroupLayout.PREFERRED_SIZE,
                    GroupLayout.PREFERRED_SIZE,
                    GroupLayout.PREFERRED_SIZE );
        zeileV.add( ueberschrift );

        ueberschrift = Util.erzeugeUeberschrift( ZFW );
        stufenArray[ 0 ] = ueberschrift;
        zeileH.add( ueberschrift,
                    GroupLayout.PREFERRED_SIZE,
                    GroupLayout.PREFERRED_SIZE,
                    GroupLayout.PREFERRED_SIZE );
        zeileV.add( ueberschrift );

        ueberschrift = Util.erzeugeUeberschrift( KOSTEN );
        kostenArray[ 0 ] = ueberschrift;
        zeileH.add( ueberschrift,
                    GroupLayout.PREFERRED_SIZE,
                    GroupLayout.PREFERRED_SIZE,
                    GroupLayout.PREFERRED_SIZE );
        zeileV.add( ueberschrift );

        if ( steigern ) {
            ueberschrift = Util.erzeugeUeberschrift( SPEZIELLE_ERFAHRUNG );
            spzErfArray[ 0 ] = ueberschrift;
            zeileH.add( ueberschrift,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE );
            zeileV.add( ueberschrift );
        }

        hGroup.add( zeileH );
        vGroup.add( zeileV );

        // In der 2. und ab da in jeder 5. Zeile wird ein Trennstrich
        // eingefuegt.
        final int extraZeilen = 1 + ( werte.size() / 4 );

        // trennstriche gibt an wie viele Trennstriche schon eingefuegt wurden.
        for ( int i = 0, n = werte.size() + extraZeilen, trennstriche = 0
                ; i < n
                ; ++i ) {


            zeileH = layout.createSequentialGroup();
            zeileV = layout.createParallelGroup( GroupLayout.CENTER );

            if ( ( i % 5 ) == 0 ) {
                // ein Trennstrich wird eingefuegt.

                final JSeparator separator = new JSeparator();

                zeileH.add( separator );
                zeileV.add( separator,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE );

                hGroup.add( zeileH );
                vGroup.add( zeileV );

                ++trennstriche;

                continue;
            }

            final int index = i - trennstriche;

            // Die Markierung fuer Haus- und Leitzauber.
            // *****************************************
            final JLabel markierung = markierungen.get( index );

            zeileH.add( markierung );
            zeileV.add( markierung,
                    GroupLayout.PREFERRED_SIZE,
                    GroupLayout.PREFERRED_SIZE,
                    GroupLayout.PREFERRED_SIZE );

            // Die Checkbox zum Aktivieren und Combobox fuer die Variante.
            // ***********************************************************
            final Zauber z = werte.get( index ).getZauber();

            if ( z.hatVarianten() ) {

                final JCheckBox checkBox = aktiviert.get( index  );

                checkBox.setText( z.getName() );

                zeileH.add( checkBox );
                zeileV.add( checkBox,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE );

                final JComboBox comboBox;

                if ( varianten.containsKey( index ) ) {

                    comboBox = varianten.get( index );

                } else {

                    comboBox = new JComboBox( new VariantenListModel() );
                    comboBox.setName( String.valueOf( index ) );
                    comboBox.addActionListener( actionVariante );
                    varianten.put( index, comboBox );

                }

                comboBox.setEditable( z.hatFreieVarianten() );

                zeileH.add( comboBox,
                            GroupLayout.PREFERRED_SIZE,
                            GroupLayout.PREFERRED_SIZE,
                            Integer.MAX_VALUE );
                zeileV.add( comboBox,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE );

            } else {

                final JCheckBox checkBox = aktiviert.get( index  );

                checkBox.setText( werte.get( index ).getName() );

                zeileH.add( checkBox,
                            GroupLayout.PREFERRED_SIZE,
                            GroupLayout.PREFERRED_SIZE,
                            Integer.MAX_VALUE );
                zeileV.add( checkBox,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE );
            }

            // Der Steigerungsfaktor
            // *********************
            final JLabel faktorLabel = steigerungsfaktoren.get( index );

            faktorArray[ 1 + index ] = faktorLabel;

            zeileH.add( faktorLabel );
            zeileV.add( faktorLabel,
                    GroupLayout.PREFERRED_SIZE,
                    GroupLayout.PREFERRED_SIZE,
                    GroupLayout.PREFERRED_SIZE );

            // Die Probe
            // *********
            final String probe = z.getProbe().toString();

            final JLabel probeLabel = new JLabel( probe );

            probenArray[ 1 + index ] = probeLabel;

            zeileH.add( probeLabel );
            zeileV.add( probeLabel,
                    GroupLayout.PREFERRED_SIZE,
                    GroupLayout.PREFERRED_SIZE,
                    GroupLayout.PREFERRED_SIZE );

            // Die Stufe
            // *********
            final JSpinner spinner = stufen.get( index );

            stufenArray[ 1 + index ] = spinner;

            zeileH.add( spinner,
                    GroupLayout.PREFERRED_SIZE,
                    GroupLayout.PREFERRED_SIZE,
                    GroupLayout.PREFERRED_SIZE );
            zeileV.add( spinner,
                    GroupLayout.PREFERRED_SIZE,
                    GroupLayout.PREFERRED_SIZE,
                    GroupLayout.PREFERRED_SIZE );

            // Die AP-Kosten
            // *************
            final JLabel kostenLabel = kosten.get( index );

            kostenArray[ 1 + index ] = kostenLabel;

            zeileH.add( kostenLabel );
            zeileV.add( kostenLabel,
                    GroupLayout.PREFERRED_SIZE,
                    GroupLayout.PREFERRED_SIZE,
                    GroupLayout.PREFERRED_SIZE );

            // Die Spezielle Erfahrung
            // ***********************
            if ( steigern ) {

                final JCheckBox se = spezielleErfahrung.get( index );

                spzErfArray[ 1 + index ] = se;

                zeileH.add( se );
                zeileV.add( se,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE );
            }

            hGroup.add( zeileH );
            vGroup.add( zeileV );
        }

        if ( 0 < werte.size() ) {

            layout.linkSize( kostenArray, GroupLayout.HORIZONTAL );

            layout.linkSize( faktorArray, GroupLayout.HORIZONTAL );

            layout.linkSize( probenArray, GroupLayout.HORIZONTAL );

            layout.linkSize( stufenArray, GroupLayout.HORIZONTAL );

            if ( steigern ) {
                layout.linkSize( spzErfArray, GroupLayout.HORIZONTAL );
            }
        }

    }

    /**
     * Baut {@link #abhaengigkeiten} auf.
     */
    private void initAbhaengigkeiten() {

        abhaengigkeiten.clear();

        for ( int i = 0, n = werte.size(); i < n ; ++i ) {

            final Zauber z = werte.get( i ).getZauber();

            for( final Object regelelement : z.getAbhaengigkeiten() ) {

                setAbhaengigkeit( regelelement, i );
            }
        }
    }

    /**
     * Erzeugt die fuer die Benutzeroberflaeche benoetigten Elemente.
     */
    private void initElemente() {
        for ( int i = aktiviert.size(), n = werte.size()
            ; i < n
            ; ++i ) {


            final JCheckBox checkBox = new JCheckBox();
            checkBox.addActionListener( actionDeAktivieren );
            checkBox.setName( String.valueOf( i ) );
            aktiviert.add( checkBox );

            kosten.add( new JLabel() );
            markierungen.add( new JLabel() );

            final JCheckBox se = new JCheckBox();
            se.addActionListener( actionSpezielleErfahrung );
            se.setName( String.valueOf( i ) );
            spezielleErfahrung.add( se );

            steigerungsfaktoren.add( new JLabel() );

            final JSpinner spinner = Util.erzeugeSpinner( 0, 0, 99, 1 );
            spinner.addChangeListener( actionStufe );
            spinner.setName( String.valueOf( i ) );
            stufen.add( spinner );
        }
    }

    /**
     * Legt fest, dass ein Zauber von einem bestimmten Regelelement abhaengig
     * ist.
     *
     * @param regelelement
     *            von dem der Zauber abhaengig ist.
     * @param index
     *            des Zaubers in {@link #werte}.
     */
    private void setAbhaengigkeit( final Object regelelement, final int index ) {

        if ( ! abhaengigkeiten.containsKey( regelelement ) ) {

            abhaengigkeiten.put( regelelement, new ArrayList< Integer >() );
        }

        abhaengigkeiten.get( regelelement ).add( index );
    }

    /**
     * Bestimmt den Helden dessen Werte im Panel angezeigt und editiert werden.
     *
     * @param derHeld der vom Panle bearbeitet wird.
     */
    public final void setHeld( final Held derHeld ) {

        held = derHeld;

        held.addObserver( this );

        anzahlGewaehlte = -1;

        // evtl. Heldenwerte anzeigen.
        update( null, null );

        for ( int i = werte.size()-1 ; i >= 0 ; --i ) {

            update( i );
        }
    }

    /**
     * Bestimmt die Zauber die im Panel angezeigt werden.
     *
     * @param neueWerte
     *            die angezeigt werden sollen.
     */
    final void setWerte( final List< ZauberWert > neueWerte ) {

        werte.clear();

        werte.addAll( neueWerte );

        initAbhaengigkeiten();

        aufbauenBenutzeroberflaeche();
    }

    /**
     * Aktualisiert eine Zeile mit einem Zauber.
     *
     * @param index des Zaubers, dessen Zeile aktualisiert werden soll.
     */
    private final void update( final int index ) {

        final ZauberWert wert = werte.get( index );

        final Zauber zauber = wert.getZauber();

        if ( held.hatHauszauber( wert.getZauber() ) ) {

            markierungen.get( index ).setIcon( ICON_HAUSZAUBER );
            markierungen.get( index ).setToolTipText( TEXT_HAUSZAUBER );

        } else if ( held.hatLeitzauber( wert.getZauber() ) ) {

            markierungen.get( index ).setIcon( ICON_LEITZAUBER );
            markierungen.get( index ).setToolTipText( TEXT_LEITZAUBER );

        } else {

            markierungen.get( index ).setIcon( ICON_PLATZHALTER );
            markierungen.get( index ).setToolTipText( null );

        }

        final boolean hatZauber = held.hat( wert );

        final JCheckBox checkBox = aktiviert.get( index );

        checkBox.setSelected( hatZauber );


        if ( zauber.hatVarianten() ) {

            final JComboBox comboBox = varianten.get( index );

            final VariantenListModel model =
                (VariantenListModel) comboBox.getModel();

            model.setItems( zauber.getVarianten( held ) );

            final Object variante = wert.getVariante();

            comboBox.setSelectedItem( variante );

            checkBox.setEnabled( ! variante.equals( "" ) );
        }

        final int stufe;
        if ( hatZauber ) {

            stufe = held.getWert( wert ).getStufe();

        } else {

            if ( held.hatVorgegeben( wert ) ) {

                stufe = held.getMindeststufe( wert );

            } else {

                stufe = 0;

            }
        }

        wert.setStufe( stufe );

        final KonfliktBeschreibung beschreibung = wert.pruefen( held );

        if ( beschreibung.enthaeltKonflikt() ) {

            checkBox.setForeground( Util.FEHLERFARBE );
            checkBox.setToolTipText( beschreibung.toHtml() );

        } else {

            checkBox.setForeground( getForeground() );
            checkBox.setToolTipText( null );

        }

        if ( held.hatVorgegeben( wert ) ) {

            checkBox.setFont( fetteSchrift );

        } else {

            checkBox.setFont( getFont() );

        }

        final JSpinner spinner = stufen.get( index );

        spinner.setValue( stufe );

        spinner.setEnabled( hatZauber );


        final int faktor;

        if ( steigern ) {

            faktor =
                wert.getSteigerungsfaktor( held, lernmethoden.getLernmethode());

        } else {

            faktor = wert.getSteigerungsfaktor( held );
        }

        steigerungsfaktoren.get( index ).setText( SKT.toString( faktor ) );


        final int apKosten;

        if ( hatZauber ) {

            if ( steigern ) {

                apKosten = wert.getSteigerungskosten(
                        held, lernmethoden.getLernmethode() );

            } else {

                apKosten = wert.getSteigerungskosten( held );
            }

        } else {

            if ( steigern ) {

                apKosten = wert.getAktivierungskosten(
                        held, lernmethoden.getLernmethode() );
            } else {

                apKosten = wert.getAktivierungskosten( held );
            }

        }

        kosten.get( index ).setText( String.format( AP, apKosten ) );

    }

    /**
     * Aktualisert die Anzeige wenn eine Veraenderung am Helden durchgefuehrt
     * wurde.
     *
     * @param o der ueberwachte Held.
     * @param arg Das Regelelement des Helden, das veraendert wurde.
     */
    public final void update( final Observable o, final Object arg ) {

        if ( zeigtHeldenwerte ) {

            final List< ZauberWert > gewaehlte;
            gewaehlte = held.getZauber();

            if ( gewaehlte.size() != anzahlGewaehlte ) {

                // Die Liste der Werte die der Held besitzt hat sich gaendert.
                // Die ganze Liste muss neu aufgebaut werden.

                anzahlGewaehlte = gewaehlte.size();

                final List< ZauberWert > werte;
                werte = new ArrayList< ZauberWert >();

                werte.addAll( held.getFehlendeZauber() );

                werte.addAll( gewaehlte );

                setWerte( werte );

                for ( int i = werte.size()-1 ; i >= 0 ; --i ) {

                    update( i );
                }
            }

        }

        if ( Lernmethoden.LERNMETHODE.equals( arg ) )  {

            for ( int i = 0, n = werte.size(); i < n ; ++i ) {

                update( i );
            }

        } else if ( abhaengigkeiten.containsKey( arg ) ) {

            for ( final int i : abhaengigkeiten.get( arg ) ) {

                update( i );
            }
        }
    }

    /**
     * Wird aufgerufen wenn ein Zauber (ab)gewaehlt wird.
     *
     * @author Bernhard Jung
     */
    private class ActionDeAktivieren implements ActionListener {

        public final void actionPerformed( final ActionEvent e ) {

            final JCheckBox checkBox = (JCheckBox) e.getSource();

            final int index = Integer.valueOf( checkBox.getName() );

            final ZauberWert wert = werte.get( index );

            if ( checkBox.isSelected() ) {

                if ( steigern ) {

                    held.aktivieren( wert, lernmethoden.getLernmethode() );

                } else {

                    held.einfuegen( wert );
                }

            } else {

                if ( steigern ) {

                    held.deaktivieren( wert );

                } else {

                    held.entfernen( wert );
                }

            }
        }
    }

    /**
     * Wird aufgerufen wenn die Variante eines Zaubers geaendert wird.
     *
     * @author Bernhard Jung
     */
    private class ActionVariante implements ActionListener {

        public void actionPerformed( final ActionEvent e ) {

            final JComboBox comboBox = (JComboBox) e.getSource();

            Object variante = comboBox.getSelectedItem();

            if ( null == variante ) {
                // keine gueltige Variante gewaehlt. Abbruch.
                return;
            }

            if ( variante instanceof String ) {
                variante = ((String)variante).trim();
            }

            final int index = Integer.valueOf( comboBox.getName() );

            final ZauberWert wert = werte.get( index );

            // nur wenn eine neue Variante gewaehlt wurde und ...
            if ( ! wert.getVariante().equals( variante )
                    // ... es eine sinnvolle Variante ist.
                    && ( ! variante.equals( "" ) ) ) {

                final Zauber zauber = wert.getZauber();

                if ( zauber.hatFreieVarianten() ) {

                    zauber.einfuegenVariante( (String) variante );
                }

                wert.setVariante( variante );
            }

            update( index );
        }
    }

    /**
     * Wird aufgerufen wenn die Variante eines Zaubers geaendert wird.
     * Diese Klasse wird benoetigt, wenn das Panel die Werte eines Helden
     * anzeigt.
     *
     * @author Bernhard Jung
     */
    private class ActionVarianteHeldenwerte implements ActionListener {

        public void actionPerformed( final ActionEvent e ) {

            final JComboBox comboBox = (JComboBox) e.getSource();

            Object variante = comboBox.getSelectedItem();

            if ( null == variante ) {
                // keine gueltige Variante gewaehlt. Abbruch.
                return;
            }

            if ( variante instanceof String ) {
                variante = ( (String)variante ).trim();
            }

            final int index = Integer.valueOf( comboBox.getName() );

            final ZauberWert wert = werte.get( index );

            // Der testwert ist da um mit  held.hat( testwert ) zu pruefen ob
            // der Held den Wert in dieser Variante schon besitzt.
            final ZauberWert testwert = wert.clone();

            testwert.setHeld( null );

            testwert.setVariante( variante );

            // nur wenn eine neue Variante gewaehlt wurde, ...
            if ( ! wert.getVariante().equals( variante )
                    // ... es eine sinnvolle Variante ist, ...
                    && ( ! variante.equals( "" ) )
                    // ... der Held die Variante noch nicht besitzt und ...
                    && ( ! held.hat( testwert ) )
                    // ... die Variante auch nicht vorgegeben ist.
                    // Warum dies letzte?
                    // Wenn dieser Ausdruck hier ausgewertet wird beutet dies
                    // der Held besitzt den Wert nicht, sonst haetten wir in der
                    // vorhergehenden Abfrage ein false erhalten und die &&
                    // Auswertung waere abgebrochen worden (&& ist lazy).
                    // Der Held besitzt den Wert also nicht. Ist der Wert
                    // vorgegeben so bedeutet dies, dass der Wert aktuell in der
                    // Liste der fehlenden, vorgegebenen Werte gespeichert ist.
                    // Werte die da gespeichert sind darf der Held nur erhalten,
                    // indem ein Wert mit held.einfuegen() hinzugefuegt wird und
                    // nicht indem man die Variante eines Wertes aendert den der
                    // Held schon besitzt.
                    // So wie es aktuell programmiert ist ist das halt so.
                    && ( ! held.hatVorgegeben( testwert ) ) ) {

                final Zauber zauber = wert.getZauber();

                if ( zauber.hatFreieVarianten() ) {

                    zauber.einfuegenVariante( (String) variante );
                }

                if ( held.hatVorgegeben( wert ) ) {
                    // Hier ist der Unterschied zur Klasse ActionVariante zu
                    // finden. Wenn die Variante eines fuer den Helden
                    // vorgegebenen Zaubers geaendert wird, so bedeutet
                    // das in diesem Fall, dass die geaenderte Variante als
                    // vorgegeben gilt.

                    held.getVorgegeben( wert ).setVariante( variante );

                }

                wert.setVariante( variante );

            } else {

                // Wenn der Benutzer einen leeren String eingegeben und Return
                // gedrueckt hat, soll der urspruengliche Eintrag wieder im
                // Textfeld erscheinen um dem Benutzer zu zeigen, dass der leere
                // String nicht uebernommen wird.
                update( index );
            }
        }
    }

    /**
     * Wird aufgerufen, wenn auf eine der Checkboxen, fuer die Spezielle
     * Erfahrung, geklickt wird.
     *
     * @author Bernhard Jung
     *
     */
    public final class ActionSpezielleErfahrung implements ActionListener {

        public void actionPerformed( final ActionEvent e ) {

            final JCheckBox se = (JCheckBox) e.getSource();

            final int index = Integer.valueOf( se.getName() );

            final ZauberWert wert = werte.get( index );

            wert.setSpezielleErfahrung( se.isSelected() );
        }
    }

    /**
     * Wird aufgerufen wenn der Wert eines Spinners geaendert wird.
     *
     * @author Bernhard Jung
     *
     */
    private final class ActionStufe implements ChangeListener {
        /**
         * Wird aufgerufen wenn der Wert in einem der Spinner veraendert wird.
         */
        public void stateChanged( final ChangeEvent e ) {

            final JSpinner spinner = (JSpinner) e.getSource();

            final int i = Integer.valueOf( spinner.getName() );

            final ZauberWert wert = werte.get( i );

            final int stufe = (Integer) spinner.getValue();

            if ( held.hat( wert ) ) {

                if ( steigern ) {

                    held.setStufe( wert, stufe, lernmethoden.getLernmethode() );

                } else {

                    held.getWert( wert ).setStufe( stufe );
                }
            }
        }
    }

}
