/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui.bonus;

import java.awt.BorderLayout;
import java.awt.Color;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.bausteine.bonus.TalentbonusFrei;
import de.bernhardjung.dsaprogramm.gui.Util;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.talente.TalentWert;

/**
 * Eine Benutzerschnittstelle die es erlaubt eine Anzahl an Talentpunkten auf
 * mehrere Talente zu verteilen.
 * Es wird die Liste der Talente angezeigt und fuer jedes Talent ein JSpinner
 * mit dem die Talentpunkte festgelegt werden koennen, die auf das Talent
 * verteilt werden.
 * Die aktuelle Stufe des Talents wird ebenfalls angezeigt.
 * Der Inhalt des Panels ist in eine JScrollPane eingefasst.
 *
 * @author Bernhard Jung
 */
public class PanelTalentBonusFrei extends PanelTalentBonus implements ChangeListener {

    /**
     * Die Ueberschrift fuer die Spalte mit der aktuellen Stufe des Helden in
     * den einzelnen Talenten.
     */
    private static final String AKTUELL =
        I18n.i18n( "PanelTalentBonusFrei.aktuell" );

    /**
     * Die Erklaerung die dem Benuter mitteilt wie viele Talentpunkte er
     * verteilen darf.
     */
    private static final String VERTEILEN =
        I18n.i18n( "PanelTalentBonusFrei.verteilen" );

    /**
     * Die Liste mit den Spinnern die zum waehlen der Talentstufe verwendet
     * werden.
     */
    private final List< JSpinner > spinner = new ArrayList< JSpinner >();

    /** Die Punkte die man auf die Talente verteilen darf. */
    private int stufenBonus;

    /**
     * Die Talente auf die es den Stufen-Bonus zu verteilen gilt.
     */
    private List< Talent > talente;

    /** Erzeugt ein neues, leeres Panel. */
    public PanelTalentBonusFrei() {
        setLayout( new BorderLayout() );
    }

    /**
     * Baut die Benutzeroberflaeche auf.
     * Ordnet die Label und Spinner an.
     *
     * @param held der den Bonus erhalten soll, der angezeigt wird.
     */
    private final void aufbauenBenutzeroberflaeche( final Held held ) {

        removeAll();

        // Pruefen ob genuegend Spinner da sind.
        initialisiereSpinner();

        final JPanel panel = new JPanel();

        // Ein auf die Anzahl der Talent angepasstes Layout setzen.
        panel.setLayout( erzeugeLayout() );

        final CellConstraints cc = new CellConstraints();

        // Der Hinweis fuer den Benutzer wie viele Talentpunkte verteilt
        // werden sollen.
        final String bitteVerteilen = String.format( VERTEILEN, stufenBonus );
        panel.add( new JLabel( bitteVerteilen ), cc.xyw( 2, 2, 6 ) );

        // Die Ueberschrift fuer die Spalte mit den aktuellen Talent-Stufen.
        panel.add( Util.erzeugeUeberschrift( AKTUELL ), cc.xy( 4, 4 ) );

        // Die Benutzeroberflaeche Zeilenweise aufbauen.
        for ( int i = 0, n = talente.size() ; i < n ; ++i ) {

            final int y = ( 2 * i ) + 6;

            final Talent talent = talente.get( i );

            // Die Spalte mit den Namen der Talente.
            panel.add( new JLabel( talent.getName() ), cc.xy( 2, y ) );

            // Die Spalte mit den aktuellen Talentwerten.
            final String aktuelleStufe;
            if ( held.hat( talent ) ) {

                aktuelleStufe = String.valueOf( held.getStufe( talent ) );

            } else {

                aktuelleStufe = "-";

            }
            panel.add( new JLabel( aktuelleStufe ), cc.xy( 4, y ) );

            // Der Spinner zum einsellen des Wertes.
            panel.add( spinner.get( i ), cc.xy( 6, y ) );
        }

        final JScrollPane scrollPane = new JScrollPane( panel );
        // HACK: Verhindert, dass unter GTK L&F der Hintergrund der ScrollPane
        // weiss wird. Es muss ein neues Color-Objekt erzeugt werden, wenn man
        // einfach das von getBackground() gelieferte Objekt uebernimmt, so
        // funktioniert es nicht.
        final Color color = new Color( this.getBackground().getRGB() );
        scrollPane.getViewport().setBackground( color );

        final JScrollBar scrollBar = scrollPane.getVerticalScrollBar();

        scrollBar.setUnitIncrement( Util.UNIT_INCREMENT );

        add( scrollPane, BorderLayout.CENTER );
    }

    /** Erzeugt das Layout fuer die Anordnung der Label und Spinner. */
    private final FormLayout erzeugeLayout() {

        final StringBuilder zeilenLayout = new StringBuilder();

        zeilenLayout.append( "7dlu, P, 7dlu, P," );
        for ( int i = talente.size(); i > 0 ; --i ) {
            zeilenLayout.append( " 4dlu, P," );
        }
        zeilenLayout.append( " 7dlu" );

        return new FormLayout(
                "7dlu, R:P:G, 4dlu, C:P, 4dlu, P, P:G, 7dlu",
                zeilenLayout.toString() );
    }

    /**
     * Sorgt dafuer das genuegend Spinner vorhanden sind und, dass alle Spinner
     * auf 0 stehen.
     */
    private final void initialisiereSpinner() {

        final int anzahl = talente.size();

        if ( spinner.size() < anzahl ) {
            // Es sind zu wenig Spinner da.
            for ( int i = spinner.size() ; i < anzahl ; ++i ) {

                final JSpinner s = Util.erzeugeSpinner( 0, 0, 99, 1);

                s.addChangeListener( this );

                spinner.add( s );
            }
        }

        // alle Spinner auf 0 setzen.
        for ( final JSpinner s : spinner ) {

            s.setValue( 0 );

            setMaximum( s, stufenBonus );
        }
    }

    /** {@inheritDoc} */
    @Override
    public final List< TalentWert > getTalentBoni() {

        final List< TalentWert > werte = new ArrayList< TalentWert >( 2 );

        for ( int i = 0, n = talente.size() ; i < n ; ++i ) {

            final int stufe = (Integer) spinner.get( i ).getValue();

            if ( 0 < stufe ) {

                final Talent talent = talente.get( i );

                werte.add( new TalentWert( talent, stufe ) );

            }
        }

        return werte;
    }

    /**
     * Bestimmt Bonus und Held fuer die man eine Auswahl treffen soll.
     *
     * @param held der den Bonus erhalten soll.
     * @param bonus den es zu verteilen gilt.
     */
    public final void setBonus(
            final Held held,
            final TalentbonusFrei bonus ) {

        stufenBonus = bonus.getStufenBonus();

        talente = bonus.getTalente();

        aufbauenBenutzeroberflaeche( held );

        validate();
        repaint();
    }

    /**
     * Bestimmt den Maximal einstellbaren Wert in einem Spinner.
     * @param s dessen Maximum gesetzt werden soll.
     * @param maximum fuer den Spinner.
     */
    private final void setMaximum( final JSpinner s, final int maximum ) {

        final SpinnerNumberModel model;
        model = (SpinnerNumberModel) s.getModel();

        model.setMaximum( maximum );
    }

    /**
     * Aktualisiert den Maximal waehlbaren Wert in den Spinnern, wenn der
     * Wert in einem Spinner geaendert wird.
     */
    public final void stateChanged( final ChangeEvent e ) {

        // Wie viele Punkte sind schon verteilt worden?
        int verteilt = 0;
        for ( final JSpinner s : spinner ) {

            verteilt += (Integer) s.getValue();
        }

        // Wie viele Punkte duerfen noch verteilt werden?
        final int rest = stufenBonus - verteilt;

        // Maximum der Spinner aktualisieren.
        for ( final JSpinner s : spinner ) {

            final int wert = (Integer) s.getValue();

            setMaximum( s, wert + rest );
        }
    }
}
