/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.talente;

import static de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft.CH;
import static de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft.FF;
import static de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft.GE;
import static de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft.IN;
import static de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft.KK;
import static de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft.KL;
import static de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft.KO;
import static de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft.MU;

import java.text.Collator;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;

import de.bernhardjung.dsaprogramm.basis.Eigenschaftsprobe;
import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.basis.Lernmethode;
import de.bernhardjung.dsaprogramm.basis.Steigerbar;
import de.bernhardjung.dsaprogramm.basis.SKT;
import de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.talente.Kampftechnik.Art;
import de.bernhardjung.dsaprogramm.talente.kosten.Kosten;
import de.bernhardjung.dsaprogramm.talente.stufen.Stufen;
import de.bernhardjung.dsaprogramm.talente.voraussetzungen.Voraussetzung;

/**
 * Repraesentiert ein Talent.
 * Ein Objekt dieser Klasse liefert alle zu einem Talent benoetigten
 * Informationen.
 * Fuer jedes Talent existiert in dieser Klasse eine statische Konstante.
 * Bevor die Talente im Programm verwendet werden koenne muss die
 * Methode {@link #initialisieren()} einmal aufgerufen werden.
 *
 * @author Bernhard Jung
 */
public class Talent implements Comparable<Talent>, Steigerbar {

    /** Collator zum Sortieren der Talente nach Name. */
    private static final Collator COLLATOR = Collator.getInstance();

    /**
     * Sortiert die Talente 1. nach Talentgruppe, 2. alphabetisch.
     */
    public static final Comparator< Talent > TALENTGRUPPEN_COMPARATOR =
        new Comparator< Talent >() {
            public int compare( Talent t1, Talent t2 ) {

                int ergebnis = t1.getTalentgruppe().compareTo( t2.getTalentgruppe() );

                if ( ergebnis == 0 ) {

                    ergebnis = t1.compareTo( t2 );
                }

                return ergebnis;
            }
    };

    static enum Typ {
        /** Der Typ fuer die Basis-Talente. */
        BASIS,
        /** Der Typ fuer die Spezial-Talente. */
        SPEZIAL,
        /** Der Typ fuer die Gaben. */
        GABE,
        /** Ein extra Typ fuer die Ritualkenntnis. */
        RITUALKENNTNIS
    };

    /** Eine Liste aller Basis-Talente. */
    private static final List< Talent > BASIS_TALENTE =
        new ArrayList< Talent >();

    /** Eine Lister aller Fernkampf-Techniken. */
    private static final List< Kampftechnik > FERNKAMPF_TALENTE =
        new ArrayList< Kampftechnik >();

    /** Eine Liste aller Talente. */
    private static final List< Talent > TALENTE =
        new ArrayList< Talent >();

    /** Eine Map in der jedes Talent unter seiner Id gespeichert ist. */
    private static final Map< String, Talent > TALENTE_MAP =
        new HashMap< String, Talent >();

    /*
     * KAMPFTECHNIK-TALENTE
     */
    public static final Kampftechnik ANDERTHALBHAENDER =
        new Kampftechnik( "Talent.Anderthalbhaender",
                Typ.SPEZIAL,
                Art.NAHKAMPF );

    public static final Kampftechnik ARMBRUST =
        new Kampftechnik( "Talent.Armbrust",
                Typ.SPEZIAL,
                Art.FERNKAMPF );

    public static final Kampftechnik BOGEN =
        new Kampftechnik( "Talent.Bogen",
                Typ.SPEZIAL,
                Art.FERNKAMPF );

    public static final Kampftechnik DOLCHE =
        new Kampftechnik( "Talent.Dolche",
                Typ.BASIS,
                Art.NAHKAMPF );

    public static final Kampftechnik FECHTWAFFEN =
        new Kampftechnik( "Talent.Fechtwaffen",
                Typ.SPEZIAL,
                Art.NAHKAMPF );

    public static final Kampftechnik HIEBWAFFEN =
        new Kampftechnik( "Talent.Hiebwaffen",
                Typ.BASIS,
                Art.NAHKAMPF );

    public static final Kampftechnik INFANTERIEWAFFEN =
        new Kampftechnik( "Talent.Infanteriewaffen",
                Typ.SPEZIAL,
                Art.NAHKAMPF );

    public static final Kampftechnik KETTENWAFFEN =
        new Kampftechnik( "Talent.Kettenwaffen",
                Typ.SPEZIAL,
                Art.NAHKAMPF );

    public static final Kampftechnik LANZENREITEN =
        new Kampftechnik( "Talent.Lanzenreiten",
                Typ.SPEZIAL,
                Art.NAHKAMPF );

    public static final Kampftechnik RAUFEN =
        new Kampftechnik( "Talent.Raufen",
                Typ.BASIS,
                Art.NAHKAMPF );

    public static final Kampftechnik RINGEN =
        new Kampftechnik( "Talent.Ringen",
                Typ.BASIS,
                Art.NAHKAMPF );

    public static final Kampftechnik SAEBEL =
        new Kampftechnik( "Talent.Saebel",
                Typ.BASIS,
                Art.NAHKAMPF );

    public static final Kampftechnik SCHWERTER =
        new Kampftechnik( "Talent.Schwerter",
                Typ.SPEZIAL,
                Art.NAHKAMPF );

    public static final Kampftechnik SPEERE =
        new Kampftechnik( "Talent.Speere",
                Typ.SPEZIAL,
                Art.NAHKAMPF );

    public static final Kampftechnik STAEBE =
        new Kampftechnik( "Talent.Staebe",
                Typ.SPEZIAL,
                Art.NAHKAMPF );

    public static final Kampftechnik WURFBEILE =
        new Kampftechnik( "Talent.Wurfbeile",
                Typ.SPEZIAL,
                Art.FERNKAMPF );

    public static final Kampftechnik WURFMESSER =
        new Kampftechnik( "Talent.Wurfmesser",
                Typ.BASIS,
                Art.FERNKAMPF );

    public static final Kampftechnik WURFSPEERE =
        new Kampftechnik( "Talent.Wurfspeere",
                Typ.SPEZIAL,
                Art.FERNKAMPF );

    public static final Kampftechnik ZWEIHAND_HIEBWAFFEN =
        new Kampftechnik( "Talent.ZweihandHiebwaffen",
                Typ.SPEZIAL,
                Art.NAHKAMPF );

    public static final Kampftechnik ZWEIHANDSCHWERTER =
        new Kampftechnik( "Talent.Zweihandschwerter",
                Typ.SPEZIAL,
                Art.NAHKAMPF );

    /*
     * KOERPERLICHE TALENTE
     */
    public static final Talent AKROBATIK = new Talent(
            "Talent.Akrobatik",
            Talentgruppe.KOERPER,
            Typ.SPEZIAL );

    public static final Talent ATHLETIK = new Talent(
            "Talent.Athletik",
            Talentgruppe.KOERPER,
            Typ.BASIS );

    public static final Talent GAUKELEIEN = new Talent(
            "Talent.Gaukeleien",
            Talentgruppe.KOERPER,
            Typ.SPEZIAL );

    public static final Talent KLETTERN = new Talent(
            "Talent.Klettern",
            Talentgruppe.KOERPER,
            Typ.BASIS );

    public static final Talent KOERPERBEHERRSCHUNG = new Talent(
            "Talent.Koerperbeherrschung",
            Talentgruppe.KOERPER,
            Typ.BASIS );

    public static final Talent REITEN = new Talent(
            "Talent.Reiten",
            Talentgruppe.KOERPER,
            Typ.SPEZIAL );

    public static final Talent SCHLEICHEN = new Talent(
            "Talent.Schleichen",
            Talentgruppe.KOERPER,
            Typ.BASIS);

    public static final Talent SCHWIMMEN = new Talent(
            "Talent.Schwimmen",
            Talentgruppe.KOERPER,
            Typ.BASIS );

    public static final Talent SELBSTBEHERRSCHUNG = new Talent(
            "Talent.Selbstbeherrschung",
            Talentgruppe.KOERPER,
            Typ.BASIS );

    public static final Talent SICH_VERSTECKEN = new Talent(
            "Talent.SichVerstecken",
            Talentgruppe.KOERPER,
            Typ.BASIS );

    public static final Talent SINGEN = new Talent(
            "Talent.Singen",
            Talentgruppe.KOERPER,
            Typ.BASIS );

    public static final Talent SINNENSCHAERFE = new Talent(
            "Talent.Sinnenschaerfe",
            Talentgruppe.KOERPER,
            Typ.BASIS );

    public static final Talent STIMMEN_IMITIEREN = new Talent(
            "Talent.StimmenImitieren",
            Talentgruppe.KOERPER,
            Typ.SPEZIAL );

    public static final Talent TANZEN = new Talent(
            "Talent.Tanzen",
            Talentgruppe.KOERPER,
            Typ.BASIS );

    public static final Talent TASCHENDIEBSTAHL = new Talent(
            "Talent.Taschendiebstahl",
            Talentgruppe.KOERPER,
            Typ.SPEZIAL );

    public static final Talent ZECHEN = new Talent(
            "Talent.Zechen",
            Talentgruppe.KOERPER,
            Typ.BASIS );

    /*
     * GESELLSCHAFTLICHE TALENTE
     */
    public static final Talent BETOEREN = new Talent(
            "Talent.Betoeren",
            Talentgruppe.GESELLSCHAFT,
            Typ.SPEZIAL );

    public static final Talent ETIKETTE = new Talent(
            "Talent.Etikette",
            Talentgruppe.GESELLSCHAFT,
            Typ.SPEZIAL );

    public static final Talent GASSENWISSEN = new Talent(
            "Talent.Gassenwissen",
            Talentgruppe.GESELLSCHAFT,
            Typ.SPEZIAL );

    public static final Talent LEHREN = new Talent(
            "Talent.Lehren",
            Talentgruppe.GESELLSCHAFT,
            Typ.SPEZIAL );

    public static final Talent MENSCHENKENNTNIS = new Talent(
            "Talent.Menschenkenntnis",
            Talentgruppe.GESELLSCHAFT,
            Typ.BASIS );

    public static final Talent SICH_VERKLEIDEN = new Talent(
            "Talent.SichVerkleiden",
            Talentgruppe.GESELLSCHAFT,
            Typ.SPEZIAL );

    public static final Talent UEBERREDEN = new Talent(
            "Talent.Ueberreden",
            Talentgruppe.GESELLSCHAFT,
            Typ.BASIS );

    public static final Talent UEBERZEUGEN = new Talent(
            "Talent.Ueberzeugen",
            Talentgruppe.GESELLSCHAFT,
            Typ.SPEZIAL );

    /*
     * NATUR-TALENTE
     */
    public static final Talent FAEHRTENSUCHEN = new Talent(
            "Talent.Faehrtensuchen",
            Talentgruppe.NATUR,
            Typ.BASIS );

    public static final Talent FALLENSTELLEN = new Talent(
            "Talent.Fallenstellen",
            Talentgruppe.NATUR,
            Typ.SPEZIAL );

    public static final Talent FESSELN_ENTFESSELN = new Talent(
            "Talent.FesselnEntfesseln",
            Talentgruppe.NATUR,
            Typ.SPEZIAL );

    public static final Talent FISCHEN_ANGELN = new Talent(
            "Talent.FischenAngeln",
            Talentgruppe.NATUR,
            Typ.SPEZIAL );

    public static final Talent ORIENTIERUNG = new Talent(
            "Talent.Orientierung",
            Talentgruppe.NATUR,
            Typ.BASIS );

    public static final Talent WETTERVORHERSAGE = new Talent(
            "Talent.Wettervorhersage",
            Talentgruppe.NATUR,
            Typ.SPEZIAL );

    public static final Talent WILDNISLEBEN = new Talent(
            "Talent.Wildnisleben",
            Talentgruppe.NATUR,
            Typ.BASIS );

    /*
     * WISSENS-TALENTE
     */
    public static final Talent ANATOMIE = new Talent(
            "Talent.Anatomie",
            Talentgruppe.WISSEN,
            Typ.SPEZIAL );

    public static final Talent BRETT_WUERFELSPIELE = new Talent(
            "Talent.BrettWuerfelspiele",
            Talentgruppe.WISSEN,
            Typ.SPEZIAL );

    public static final Talent GEOGRAPHIE = new Talent(
            "Talent.Geographie",
            Talentgruppe.WISSEN,
            Typ.SPEZIAL );

    public static final Talent GESCHICHTSWISSEN = new Talent(
            "Talent.Geschichtswissen",
            Talentgruppe.WISSEN,
            Typ.SPEZIAL );

    public static final Talent GESTEINSKUNDE = new Talent(
            "Talent.Gesteinskunde",
            Talentgruppe.WISSEN,
            Typ.SPEZIAL );

    public static final Talent GOETTER_KULTE = new Talent(
            "Talent.GoetterKulte",
            Talentgruppe.WISSEN,
            Typ.BASIS );

    public static final Talent HERALDIK = new Talent(
            "Talent.Heraldik",
            Talentgruppe.WISSEN,
            Typ.SPEZIAL );

    public static final Talent KRIEGSKUNST = new Talent(
            "Talent.Kriegskunst",
            Talentgruppe.WISSEN,
            Typ.SPEZIAL );

    public static final Talent MAGIEKUNDE = new Talent(
            "Talent.Magiekunde",
            Talentgruppe.WISSEN,
            Typ.SPEZIAL );

    public static final Talent MECHANIK = new Talent(
            "Talent.Mechanik",
            Talentgruppe.WISSEN,
            Typ.SPEZIAL );

    public static final Talent PFLANZENKUNDE = new Talent(
            "Talent.Pflanzenkunde",
            Talentgruppe.WISSEN,
            Typ.SPEZIAL );

    public static final Talent RECHNEN = new Talent(
            "Talent.Rechnen",
            Talentgruppe.WISSEN,
            Typ.BASIS );

    public static final Talent RECHTSKUNDE = new Talent(
            "Talent.Rechtskunde",
            Talentgruppe.WISSEN,
            Typ.SPEZIAL );

    public static final Talent SAGEN_LEGENDEN = new Talent(
            "Talent.SagenLegenden",
            Talentgruppe.WISSEN,
            Typ.BASIS );

    public static final Talent SCHAETZEN = new Talent(
            "Talent.Schaetzen",
            Talentgruppe.WISSEN,
            Typ.SPEZIAL );

    public static final Talent SPRACHENKUNDE = new Talent(
            "Talent.Sprachenkunde",
            Talentgruppe.WISSEN,
            Typ.SPEZIAL );

    public static final Talent STAATSKUNST = new Talent(
            "Talent.Staatskunst",
            Talentgruppe.WISSEN,
            Typ.SPEZIAL );

    public static final Talent STERNKUNDE = new Talent(
            "Talent.Sternkunde",
            Talentgruppe.WISSEN,
            Typ.SPEZIAL );

    public static final Talent TIERKUNDE = new Talent(
            "Talent.Tierkunde",
            Talentgruppe.WISSEN,
            Typ.SPEZIAL );

    /*
     * SPRACHEN
     */
    public static final Sprache ALAANI =
        new Sprache( "Talent.Alaani" );

    public static final Sprache ASDHARIA =
        new Sprache( "Talent.Asdharia" );

    public static final Sprache ATAK =
        new Sprache( "Talent.Atak" );

    public static final Sprache BOSPARANO =
        new Sprache( "Talent.Bosparano" );

    public static final Sprache FUECHSISCH =
        new Sprache( "Talent.Fuechsisch" );

    public static final Sprache GARETHI =
        new Sprache( "Talent.Garethi" );

    public static final Sprache GOBLINISCH =
        new Sprache( "Talent.Goblinisch" );

    public static final Sprache ISDIRA =
        new Sprache( "Talent.Isdira" );

    public static final Sprache MOHISCH =
        new Sprache( "Talent.Mohisch" );

    public static final Sprache NUJUKA =
        new Sprache( "Talent.Nujuka" );

    public static final Sprache OLOARKH =
        new Sprache( "Talent.Oloarkh" );

    public static final Sprache OLOGHAIJAN =
        new Sprache( "Talent.Ologhaijan" );

    public static final Sprache ROGOLAN =
        new Sprache( "Talent.Rogolan" );

    public static final Sprache RSSAHH =
        new Sprache( "Talent.Rssahh" );

    public static final Sprache THORWALSCH =
        new Sprache( "Talent.Thorwalsch" );

    public static final Sprache TULAMIDYA =
        new Sprache( "Talent.Tulamidya" );

    public static final Sprache UR_TULAMIDYA =
        new Sprache( "Talent.UrTulamidya" );

    public static final Sprache ZHAYAD =
        new Sprache( "Talent.Zhayad" );

    /*
     *  SCHRIFTEN
     */
    public static final Schrift ASDHARIA_SCHRIFT =
        new Schrift( "Talent.AsdhariaSchrift" );

    public static final Schrift CHRMK =
        new Schrift( "Talent.Chrmk" );

    public static final Schrift GLYPHEN_VON_UNAU =
        new Schrift( "Talent.GlyphenVonUnau" );

    public static final Schrift HJALDINGSCHE_RUNEN =
        new Schrift( "Talent.HjaldingscheRunen" );

    public static final Schrift ISDIRA_SCHRIFT =
        new Schrift( "Talent.IsdiraSchrift" );

    public static final Schrift KUSLIKER_ZEICHEN =
        new Schrift( "Talent.KuslikerZeichen" );

    public static final Schrift NANDURIA =
        new Schrift( "Talent.Nanduria" );

    public static final Schrift ROGOLAN_SCHRIFT =
        new Schrift( "Talent.RogolanSchrift" );

    public static final Schrift TULAMIDYA_SCHRIFT =
        new Schrift( "Talent.TulamidyaSchrift" );

    public static final Schrift UR_TULAMIDYA_SCHRIFT =
        new Schrift( "Talent.UrTulamidyaSchrift" );

    public static final Schrift ZHAYAD_SCHRIFT =
        new Schrift( "Talent.ZhayadSchrift" );

    /*
     * HANDWERKS-TALENTE
     */
    public static final Talent ABRICHTEN = new Talent(
            "Talent.Abrichten",
            Talentgruppe.HANDWERK,
            Typ.SPEZIAL );

    public static final Talent ACKERBAU = new Talent(
            "Talent.Ackerbau",
            Talentgruppe.HANDWERK,
            Typ.SPEZIAL );

    public static final Talent ALCHIMIE = new Talent(
            "Talent.Alchimie",
            Talentgruppe.HANDWERK,
            Typ.SPEZIAL );

    public static final Talent BERGBAU = new Talent(
            "Talent.Bergbau",
            Talentgruppe.HANDWERK,
            Typ.SPEZIAL );

    public static final Talent BOGENBAU = new Talent(
            "Talent.Bogenbau",
            Talentgruppe.HANDWERK,
            Typ.SPEZIAL );

    public static final Talent BOOTE_FAHREN = new Talent(
            "Talent.BooteFahren",
            Talentgruppe.HANDWERK,
            Typ.SPEZIAL );

    public static final Talent FAHRZEUG_LENKEN = new Talent(
            "Talent.FahrzeugLenken",
            Talentgruppe.HANDWERK,
            Typ.SPEZIAL );

    public static final Talent FALSCHSPIEL = new Talent(
            "Talent.Falschspiel",
            Talentgruppe.HANDWERK,
            Typ.SPEZIAL );

    public static final Talent FEINMECHANIK = new Talent(
            "Talent.Feinmechanik",
            Talentgruppe.HANDWERK,
            Typ.SPEZIAL );

    public static final Talent FLEISCHER = new Talent(
            "Talent.Fleischer",
            Talentgruppe.HANDWERK,
            Typ.SPEZIAL );

    public static final Talent GERBER_KUERSCHNER = new Talent(
            "Talent.GerberKuerschner",
            Talentgruppe.HANDWERK,
            Typ.SPEZIAL );

    public static final Talent GROBSCHMIED = new Talent(
            "Talent.Grobschmied",
            Talentgruppe.HANDWERK,
            Typ.SPEZIAL );

    public static final Talent HEILKUNDE_GIFT = new Talent(
            "Talent.HeilkundeGift",
            Talentgruppe.HANDWERK,
            Typ.SPEZIAL );

    public static final Talent HEILKUNDE_KRANKHEITEN = new Talent(
            "Talent.HeilkundeKrankheiten",
            Talentgruppe.HANDWERK,
            Typ.SPEZIAL );

    public static final Talent HEILKUNDE_SEELE = new Talent(
            "Talent.HeilkundeSeele",
            Talentgruppe.HANDWERK,
            Typ.SPEZIAL );

    public static final Talent HEILKUNDE_WUNDEN = new Talent(
            "Talent.HeilkundeWunden",
            Talentgruppe.HANDWERK,
            Typ.BASIS );

    public static final Talent HOLZBEARBEITUNG = new Talent(
            "Talent.Holzbearbeitung",
            Talentgruppe.HANDWERK,
            Typ.BASIS );

    public static final Talent KARTOGRAPHIE = new Talent(
            "Talent.Kartographie",
            Talentgruppe.HANDWERK,
            Typ.SPEZIAL );

    public static final Talent KOCHEN = new Talent(
            "Talent.Kochen",
            Talentgruppe.HANDWERK,
            Typ.BASIS );

    public static final Talent LEDERARBEITEN = new Talent(
            "Talent.Lederarbeiten",
            Talentgruppe.HANDWERK,
            Typ.BASIS );

    public static final Talent MALEN_ZEICHNEN = new Talent(
            "Talent.MalenZeichnen",
            Talentgruppe.HANDWERK,
            Typ.BASIS );

    public static final Talent MUSIZIEREN = new Talent(
            "Talent.Musizieren",
            Talentgruppe.HANDWERK,
            Typ.SPEZIAL );

    public static final Talent SCHLOESSER_KNACKEN = new Talent(
            "Talent.SchloesserKnacken",
            Talentgruppe.HANDWERK,
            Typ.SPEZIAL );

    public static final Talent SCHNEIDERN = new Talent(
            "Talent.Schneidern",
            Talentgruppe.HANDWERK,
            Typ.BASIS );

    public static final Talent SEEFAHRT = new Talent(
            "Talent.Seefahrt",
            Talentgruppe.HANDWERK,
            Typ.SPEZIAL );

    public static final Talent STEINMETZ = new Talent(
            "Talent.Steinmetz",
            Talentgruppe.HANDWERK,
            Typ.SPEZIAL );

    public static final Talent STEINSCHNEIDER = new Talent(
            "Talent.Steinschneider",
            Talentgruppe.HANDWERK,
            Typ.SPEZIAL );

    public static final Talent TAETOWIEREN = new Talent(
            "Talent.Taetowieren",
            Talentgruppe.HANDWERK,
            Typ.SPEZIAL );

    public static final Talent ZIMMERMANN = new Talent(
            "Talent.Zimmermann",
            Talentgruppe.HANDWERK,
            Typ.SPEZIAL );

    /*
     * GABEN
     */
    public static final Talent GEFAHRENINSTINKT = new Talent(
            "Talent.Gefahreninstinkt",
            Talentgruppe.GABEN,
            Typ.GABE );

    public static final Talent MAGIEGESPUER = new Talent(
            "Talent.Magiegespuer",
            Talentgruppe.GABEN,
            Typ.GABE );

    public static final Talent PROPHEZEIEN = new Talent(
            "Talent.Prophezeien",
            Talentgruppe.GABEN,
            Typ.GABE );

    public static final Talent ZWERGENNASE = new Talent(
            "Talent.Zwergennase",
            Talentgruppe.GABEN,
            Typ.GABE );

    /*
     * RITUALKENNTNIS
     */
    public static final Talent RITUALKENNTNIS = new Talent(
            "Talent.Ritualkenntnis",
            Talentgruppe.RITUALKENNTNIS,
            Typ.RITUALKENNTNIS );

    /**
     * @param id des gesuchten Talents.
     * @return das Talent mit der uebergebenen Id.
     * @throws IllegalArgumentException wenn kein Talent mit der uebergebenen
     *      Id existiert.
     */
    public static final Talent getTalent( final String id ) {

        if ( TALENTE_MAP.containsKey( id ) ) {

            return TALENTE_MAP.get( id );

        } else {

            throw new IllegalArgumentException(
                    "Es existiert kein Talent mit der Id " + id );

        }
    }
    /**
     * Initialisiert alle Talente.
     * Dies Methode muss einmal aufgerufen werden bevor man die Talente im
     * Programm verwenden kann.
     */
    public static final void initialisieren() {

        /*
         * KAMPFTECHNIK-TALENTE
         */
        initialisiereKampftechnik( ANDERTHALBHAENDER, SKT.E, 2 );
        ANDERTHALBHAENDER.setSpezialisierungen(
                I18n.i18n( "Waffe.Anderthalbhaender" ),
                I18n.i18n( "Waffe.Rondrakamm" ),
                I18n.i18n( "Waffe.Tuzakmesser" ) );
        ANDERTHALBHAENDER.setVerwandteFertigkeiten(
                SCHWERTER, ZWEIHANDSCHWERTER );

        initialisiereKampftechnik( ARMBRUST, SKT.C, 5 );
        ARMBRUST.setSpezialisierungen(
                I18n.i18n( "Waffe.Ballaester" ),
                I18n.i18n( "Waffe.Eisenwalder" ),
                I18n.i18n( "Waffe.LeichteArmbrust" ),
                I18n.i18n( "Waffe.Windenarmbrust" ) );
        ARMBRUST.setVerwandteFertigkeiten( BOGEN );

        initialisiereKampftechnik( BOGEN, SKT.E, 3 );
        BOGEN.einfuegen( Voraussetzung.NICHT_EINARMIG );
        BOGEN.setSpezialisierungen(
                I18n.i18n( "Waffe.Elfenbogen" ),
                I18n.i18n( "Waffe.Kompositbogen" ),
                I18n.i18n( "Waffe.Kriegsbogen" ),
                I18n.i18n( "Waffe.Kurzbogen" ),
                I18n.i18n( "Waffe.Langbogen" ) );

        initialisiereKampftechnik( DOLCHE, SKT.D, 1 );
        DOLCHE.setSpezialisierungen(
                I18n.i18n( "Waffe.Dolch" ),
                I18n.i18n( "Waffe.Drachenzahn" ),
                I18n.i18n( "Waffe.Jagdmesser" ),
                I18n.i18n( "Waffe.Langdolch" ),
                I18n.i18n( "Waffe.Messer" ),
                I18n.i18n( "Waffe.SchwererDolch" ),
                I18n.i18n( "Waffe.Wurfdolch" ),
                I18n.i18n( "Waffe.Wurfmesser" ) );
        DOLCHE.setVerwandteFertigkeiten( FECHTWAFFEN );

        initialisiereKampftechnik( FECHTWAFFEN, SKT.E, 1 );
        FECHTWAFFEN.setSpezialisierungen(
                I18n.i18n( "Waffe.Degen" ),
                I18n.i18n( "Waffe.Florett" ),
                I18n.i18n( "Waffe.Rapier" ) );
        FECHTWAFFEN.setVerwandteFertigkeiten( DOLCHE, SCHWERTER );

        initialisiereKampftechnik( HIEBWAFFEN, SKT.D, 4 );
        HIEBWAFFEN.setSpezialisierungen(
                I18n.i18n( "Waffe.Beil" ),
                I18n.i18n( "Waffe.Brabakbengel" ),
                I18n.i18n( "Waffe.Fackel" ),
                I18n.i18n( "Waffe.Keule" ),
                I18n.i18n( "Waffe.Knueppel" ),
                I18n.i18n( "Waffe.Lindwurmschlaeger" ),
                I18n.i18n( "Waffe.Rabenschnabel" ),
                I18n.i18n( "Waffe.Sichel" ),
                I18n.i18n( "Waffe.Skraja" ),
                I18n.i18n( "Waffe.Streitaxt" ),
                I18n.i18n( "Waffe.Streitkolben" ),
                I18n.i18n( "Waffe.Stuhlbein" ),
                I18n.i18n( "Waffe.Wurfbeil" ) );
        HIEBWAFFEN.setVerwandteFertigkeiten( SAEBEL );

        initialisiereKampftechnik( INFANTERIEWAFFEN, SKT.D, 3 );
        INFANTERIEWAFFEN.setSpezialisierungen(
                I18n.i18n( "Waffe.Glefe" ),
                I18n.i18n( "Waffe.Hakenspiess" ),
                I18n.i18n( "Waffe.Hellebarde" ),
                I18n.i18n( "Waffe.Partisane" ),
                I18n.i18n( "Waffe.Sense" ),
                I18n.i18n( "Waffe.Sturmsense" ),
                I18n.i18n( "Waffe.Wurmspiess" ) );
        INFANTERIEWAFFEN.setVerwandteFertigkeiten( SPEERE, ZWEIHAND_HIEBWAFFEN);

        initialisiereKampftechnik( KETTENWAFFEN, SKT.D, 3 );
        KETTENWAFFEN.setSpezialisierungen(
                I18n.i18n( "Waffe.Morgenstern" ),
                I18n.i18n( "Waffe.Ochsenherde" ),
                I18n.i18n( "Waffe.Ogerschelle" ) );

        initialisiereKampftechnik( LANZENREITEN, SKT.E, 0 );

        initialisiereKampftechnik( RAUFEN, SKT.C, 0 );

        initialisiereKampftechnik( RINGEN, SKT.D, 0 );

        initialisiereKampftechnik( SAEBEL, SKT.D, 2 );
        SAEBEL.setSpezialisierungen(
                I18n.i18n( "Waffe.Entermesser" ),
                I18n.i18n( "Waffe.Haumesser" ),
                I18n.i18n( "Waffe.Khunchomer" ),
                I18n.i18n( "Waffe.Saebel" ),
                I18n.i18n( "Waffe.Waqqif" ) );
        SAEBEL.setVerwandteFertigkeiten( SCHWERTER, HIEBWAFFEN );

        initialisiereKampftechnik( SCHWERTER, SKT.E, 2 );
        SCHWERTER.setSpezialisierungen(
                I18n.i18n( "Waffe.Kurzschwert" ),
                I18n.i18n( "Waffe.KuslikerSaebel" ),
                I18n.i18n( "Waffe.Schwert" ) );
        SCHWERTER.setVerwandteFertigkeiten( SAEBEL, FECHTWAFFEN,
                ANDERTHALBHAENDER );

        initialisiereKampftechnik( SPEERE, SKT.D, 3 );
        SPEERE.setSpezialisierungen(
                I18n.i18n( "Waffe.Dreizack" ),
                I18n.i18n( "Waffe.Dschadra" ),
                I18n.i18n( "Waffe.Holzspeer" ),
                I18n.i18n( "Waffe.Jagdspiess" ),
                I18n.i18n( "Waffe.Speer" ),
                I18n.i18n( "Waffe.Stossspeer" ),
                I18n.i18n( "Waffe.Wurfspeer" ) );
        SPEERE.setVerwandteFertigkeiten( INFANTERIEWAFFEN );

        initialisiereKampftechnik( STAEBE, SKT.D, 2 );
        STAEBE.setSpezialisierungen(
                I18n.i18n( "Waffe.Kampfstab" ),
                I18n.i18n( "Waffe.Magierstab" ),
                I18n.i18n( "Waffe.Zweililien" ) );
        STAEBE.setVerwandteFertigkeiten( SPEERE );

        initialisiereKampftechnik( WURFBEILE, SKT.C, 2 );
        WURFBEILE.setSpezialisierungen(
                I18n.i18n( "Waffe.Schneidzahn" ),
                I18n.i18n( "Waffe.Wurfbeil" ) );
        WURFBEILE.setVerwandteFertigkeiten( WURFSPEERE, WURFMESSER );

        initialisiereKampftechnik( WURFMESSER, SKT.C, 3 );
        WURFMESSER.setSpezialisierungen(
                I18n.i18n( "Waffe.Borndorn" ),
                I18n.i18n( "Waffe.Dolch" ),
                I18n.i18n( "Waffe.Wurfdolch" ),
                I18n.i18n( "Waffe.Wurfmesser" ),
                I18n.i18n( "Waffe.WurfscheibeRing" ) );
        WURFMESSER.setVerwandteFertigkeiten( WURFBEILE, WURFSPEERE );

        initialisiereKampftechnik( WURFSPEERE, SKT.C, 2 );
        WURFSPEERE.setSpezialisierungen(
                I18n.i18n( "Waffe.Holzspeer" ),
                I18n.i18n( "Waffe.Speer" ),
                I18n.i18n( "Waffe.Wurfspeer" ),
                I18n.i18n( "Waffe.Dschadra" ) );
        WURFSPEERE.setVerwandteFertigkeiten( WURFBEILE );

        initialisiereKampftechnik( ZWEIHAND_HIEBWAFFEN, SKT.D, 3 );
        ZWEIHAND_HIEBWAFFEN.einfuegen( Voraussetzung.NICHT_EINARMIG );
        ZWEIHAND_HIEBWAFFEN.setSpezialisierungen(
                I18n.i18n( "Waffe.Barbarenstreitaxt" ),
                I18n.i18n( "Waffe.Felsspalter" ),
                I18n.i18n( "Waffe.Holzfaelleraxt" ),
                I18n.i18n( "Waffe.Kriegshammer" ),
                I18n.i18n( "Waffe.Orknase" ),
                I18n.i18n( "Waffe.Spitzhacke" ),
                I18n.i18n( "Waffe.Vorschlaghammer" ),
                I18n.i18n( "Waffe.WarunkerHammer" ),
                I18n.i18n( "Waffe.Zwergenschlaegel" ) );
        ZWEIHAND_HIEBWAFFEN.setVerwandteFertigkeiten( HIEBWAFFEN,
                INFANTERIEWAFFEN );

        initialisiereKampftechnik( ZWEIHANDSCHWERTER, SKT.E, 2 );
        ZWEIHANDSCHWERTER.einfuegen( Voraussetzung.NICHT_EINARMIG );
        ZWEIHANDSCHWERTER.setSpezialisierungen(
                I18n.i18n( "Waffe.Andergaster" ),
                I18n.i18n( "Waffe.Boronssichel" ),
                I18n.i18n( "Waffe.Doppelkhunchomer" ),
                I18n.i18n( "Waffe.Rondrakamm" ),
                I18n.i18n( "Waffe.Zweihaender" ) );
        ZWEIHANDSCHWERTER.setVerwandteFertigkeiten( ANDERTHALBHAENDER,
                SCHWERTER, SAEBEL );

        /*
         * KOERPERLICHE TALENTE
         */
        initialisieren( AKROBATIK,
                        Kosten.SCHLANGENMENSCH,
                        SKT.D,
                        Stufen.TALENT,
                        MU, GE, KK );
        AKROBATIK.einfuegen( Voraussetzung.KOERPERBEHERSCHUNG_4 );
        AKROBATIK.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.Balancieren" ),
                I18n.i18n( "Talent.Spezialisierung.Bodenakrobatik" ),
                I18n.i18n( "Talent.Spezialisierung.Schwingen" ),
                I18n.i18n( "Talent.Spezialisierung.Spruenge" ),
                I18n.i18n( "Talent.Spezialisierung.Winden" ) );

        initialisieren( ATHLETIK, SKT.D, GE, KO, KK );
        ATHLETIK.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.Hochsprung" ),
                I18n.i18n( "Talent.Spezialisierung.Kraftakte" ),
                I18n.i18n( "Talent.Spezialisierung.Langlauf" ),
                I18n.i18n( "Talent.Spezialisierung.Sprint" ),
                I18n.i18n( "Talent.Spezialisierung.Weitsprung" ) );

        initialisieren( GAUKELEIEN,
                        Kosten.SCHLANGENMENSCH,
                        SKT.D,
                        Stufen.TALENT,
                        MU, CH, FF );
        GAUKELEIEN.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.Bauchreden" ),
                I18n.i18n( "Talent.Spezialisierung.Feuerkunst" ),
                I18n.i18n( "Talent.Spezialisierung.Jonglieren" ),
                I18n.i18n( "Talent.Spezialisierung.Possenreissen" ),
                I18n.i18n( "Talent.Spezialisierung.Taschenspielereien" ) );

        initialisieren( KLETTERN, SKT.D, MU, GE, KK );
        KLETTERN.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.Bergsteigen" ),
                I18n.i18n( "Talent.Spezialisierung.Eisklettern" ),
                I18n.i18n( "Talent.Spezialisierung.Freiklettern" ),
                I18n.i18n( "Talent.Spezialisierung.Seilklettern" ) );

        initialisieren( KOERPERBEHERRSCHUNG,
                        Kosten.SCHLANGENMENSCH,
                        SKT.D,
                        Stufen.TALENT,
                        MU, IN, GE );
        KOERPERBEHERRSCHUNG.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.Fallen" ),
                I18n.i18n( "Talent.Spezialisierung.Spruenge" ),
                I18n.i18n( "Talent.Spezialisierung.Standfestigkeit" ) );

        initialisieren( REITEN, SKT.D, CH, GE, KK );
        REITEN.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.PferdEselMuli" ),
                I18n.i18n( "Talent.Spezialisierung.Kamel" ),
                I18n.i18n( "Talent.Spezialisierung.Strauss" ),
                I18n.i18n( "Talent.Spezialisierung.Reitechse" ),
                I18n.i18n( "Talent.Spezialisierung.Flugechse" ),
                I18n.i18n( "Talent.Spezialisierung.Hippogriff" ),
                I18n.i18n( "Talent.Spezialisierung.GoblinischesReitWildschwein" ) );

        initialisieren( SCHLEICHEN,
                        Kosten.SCHLANGENMENSCH,
                        SKT.D,
                        Stufen.TALENT,
                        MU, IN, GE );
        SCHLEICHEN.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.EisSchnee" ),
                I18n.i18n( "Talent.Spezialisierung.Gebaeude" ),
                I18n.i18n( "Talent.Spezialisierung.Geroell" ),
                I18n.i18n( "Talent.Spezialisierung.Strassen" ),
                I18n.i18n( "Talent.Spezialisierung.WaldDschungel" ) );

        initialisieren( SCHWIMMEN, SKT.D, GE, KO, KK );
        SCHWIMMEN.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.Langstreckenschwimmen" ),
                I18n.i18n( "Talent.Spezialisierung.Schnellschwimmen" ),
                I18n.i18n( "Talent.Spezialisierung.Tauchen" ),
                I18n.i18n( "Talent.Spezialisierung.Unterwasserkampf" ) );

        initialisieren( SELBSTBEHERRSCHUNG, SKT.D, MU, KO, KK );
        SELBSTBEHERRSCHUNG.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.ErschoepfungIgnorieren" ),
                I18n.i18n( "Talent.Spezialisierung.SchmerzUnterdruecken" ),
                I18n.i18n( "Talent.Spezialisierung.StoerungenIgnorieren" ),
                I18n.i18n( "Talent.Spezialisierung.WundenIgnorieren" ) );

        initialisieren( SICH_VERSTECKEN,
                        Kosten.SCHLANGENMENSCH,
                        SKT.D,
                        Stufen.TALENT,
                        MU, IN, GE );
        SICH_VERSTECKEN.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.Gebaeude" ),
                I18n.i18n( "Talent.Spezialisierung.Gebirge" ),
                I18n.i18n( "Talent.Spezialisierung.Grasland" ),
                I18n.i18n( "Talent.Spezialisierung.EisSchnee" ),
                I18n.i18n( "Talent.Spezialisierung.Stadt" ),
                I18n.i18n( "Talent.Spezialisierung.WaldDschungel" ),
                I18n.i18n( "Talent.Spezialisierung.Wueste" ) );

        initialisieren( SINGEN, SKT.D, IN, CH, KO );
        SINGEN.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.Balladenvortrag" ),
                I18n.i18n( "Talent.Spezialisierung.Chorgesang" ),
                I18n.i18n( "Talent.Spezialisierung.Mehrstimmigkeit" ),
                I18n.i18n( "Talent.Spezialisierung.Operngesang" ) );

        initialisieren( SINNENSCHAERFE, SKT.D, KL, IN, IN, FF );
        SINNENSCHAERFE.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.Hoeren" ),
                I18n.i18n( "Talent.Spezialisierung.RiechenSchmecken" ),
                I18n.i18n( "Talent.Spezialisierung.Sehen" ),
                I18n.i18n( "Talent.Spezialisierung.Tasten" ) );

        initialisieren( STIMMEN_IMITIEREN, SKT.D,  KL, IN, CH );
        STIMMEN_IMITIEREN.einfuegen( Voraussetzung.SINNENSCHAERFE_4 );
        STIMMEN_IMITIEREN.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.Haustiere" ),
                I18n.i18n( "Talent.Spezialisierung.Jagdwild" ),
                I18n.i18n( "Talent.Spezialisierung.MenschlicheStimmen" ),
                I18n.i18n( "Talent.Spezialisierung.Raubtiere" ),
                I18n.i18n( "Talent.Spezialisierung.Voegel" ) );

        initialisieren( TANZEN,
                        Kosten.SCHLANGENMENSCH,
                        SKT.D,
                        Stufen.TALENT,
                        CH, GE , GE );
        TANZEN.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.Ausdruckstaenze" ),
                I18n.i18n( "Talent.Spezialisierung.Formationstaenze" ),
                I18n.i18n( "Talent.Spezialisierung.HoefischeTaenze" ),
                I18n.i18n( "Talent.Spezialisierung.Kulttaenze" ),
                I18n.i18n( "Talent.Spezialisierung.Meditationstaenze" ) );

        initialisieren( TASCHENDIEBSTAHL, SKT.D, MU, IN, FF );
        TASCHENDIEBSTAHL.einfuegen(
                Voraussetzung.MENSCHENKENNTNIS_4_STEIGERN );

        initialisieren( ZECHEN, SKT.D, IN, KO, KK );

        /*
         * GESELLSCHAFTLICHE TALENTE
         */
        initialisieren( BETOEREN, SKT.B, IN, CH, CH );
        BETOEREN.einfuegen( Voraussetzung.MENSCHENKENNTNIS_4 );

        initialisieren( ETIKETTE, SKT.B, KL, IN, CH );
        ETIKETTE.setSpezialisierungen(
                I18n.i18n( "Kulturkunde.Almada" ),
                I18n.i18n( "Kulturkunde.Amazonen" ),
                I18n.i18n( "Kulturkunde.Ambosszwerge" ),
                I18n.i18n( "Kulturkunde.AndergastNostria" ),
                I18n.i18n( "Kulturkunde.Aranien" ),
                I18n.i18n( "Kulturkunde.ArchaischeAchaz" ),
                I18n.i18n( "Kulturkunde.Auelfen" ),
                I18n.i18n( "Kulturkunde.Bornland" ),
                I18n.i18n( "Kulturkunde.Brilliantzwerge" ),
                I18n.i18n( "Kulturkunde.Bukanier" ),
                I18n.i18n( "Kulturkunde.Erzzwerge" ),
                I18n.i18n( "Kulturkunde.Ferkina" ),
                I18n.i18n( "Kulturkunde.Firnelfen" ),
                I18n.i18n( "Kulturkunde.Fjarninger" ),
                I18n.i18n( "Kulturkunde.Gjalskerlaender" ),
                I18n.i18n( "Kulturkunde.Goblins" ),
                I18n.i18n( "Kulturkunde.Grolme" ),
                I18n.i18n( "Kulturkunde.Horasreich" ),
                I18n.i18n( "Kulturkunde.Huegelzwerge" ),
                I18n.i18n( "Kulturkunde.Nivesen" ),
                I18n.i18n( "Kulturkunde.Norbarden" ),
                I18n.i18n( "Kulturkunde.NordaventurischeStaedte" ),
                I18n.i18n( "Kulturkunde.Maraskan" ),
                I18n.i18n( "Kulturkunde.Mohas" ),
                I18n.i18n( "Kulturkunde.Mittelreich" ),
                I18n.i18n( "Kulturkunde.Novadi" ),
                I18n.i18n( "Kulturkunde.Orks" ),
                I18n.i18n( "Kulturkunde.SchwarzeLande" ),
                I18n.i18n( "Kulturkunde.StammesAchaz" ),
                I18n.i18n( "Kulturkunde.Steppenelfen" ),
                I18n.i18n( "Kulturkunde.Suedaventurien" ),
                I18n.i18n( "Kulturkunde.Svellttal" ),
                I18n.i18n( "Kulturkunde.Thorwal" ),
                I18n.i18n( "Kulturkunde.Tocamuyac" ),
                I18n.i18n( "Kulturkunde.Trolle" ),
                I18n.i18n( "Kulturkunde.Trollzacker" ),
                I18n.i18n( "Kulturkunde.Tulamidenlande" ),
                I18n.i18n( "Kulturkunde.Waldelfen" ),
                I18n.i18n( "Kulturkunde.Zahori" ),
                I18n.i18n( "Kulturkunde.Zyklopeninseln" ) );

        initialisieren( GASSENWISSEN, SKT.B, KL, IN, CH );
        GASSENWISSEN.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.Beschatten" ),
                I18n.i18n( "Talent.Spezialisierung.Hehlerei" ),
                I18n.i18n( "Talent.Spezialisierung.Kontakte" ),
                I18n.i18n( "Talent.Spezialisierung.Ortseinschaetzung" ) );

        initialisieren( LEHREN, SKT.B, KL, IN, CH );
        LEHREN.einfuegen( Voraussetzung.MENSCHENKENNTNIS_4_STEIGERN );
        LEHREN.setSpezialisierungen(
                I18n.i18n( "Talentgruppe.Kampf.Name" ),
                I18n.i18n( "Talentgruppe.Koerper.Name" ),
                I18n.i18n( "Talentgruppe.Gesellschaft.Name" ),
                I18n.i18n( "Talentgruppe.Natur.Name" ),
                I18n.i18n( "Talentgruppe.Wissen.Name" ),
                I18n.i18n( "Talentgruppe.Handwerk.Name" ),
                I18n.i18n( "Talent.Spezialisierung.SprachenSchriften" ) );

        initialisieren( MENSCHENKENNTNIS, SKT.B, KL, IN, CH );
        MENSCHENKENNTNIS.setSpezialisierungen(
                I18n.i18n( "Kulturkunde.Almada" ),
                I18n.i18n( "Kulturkunde.Amazonen" ),
                I18n.i18n( "Kulturkunde.Ambosszwerge" ),
                I18n.i18n( "Kulturkunde.AndergastNostria" ),
                I18n.i18n( "Kulturkunde.Aranien" ),
                I18n.i18n( "Kulturkunde.ArchaischeAchaz" ),
                I18n.i18n( "Kulturkunde.Auelfen" ),
                I18n.i18n( "Kulturkunde.Bornland" ),
                I18n.i18n( "Kulturkunde.Brilliantzwerge" ),
                I18n.i18n( "Kulturkunde.Bukanier" ),
                I18n.i18n( "Kulturkunde.Erzzwerge" ),
                I18n.i18n( "Kulturkunde.Ferkina" ),
                I18n.i18n( "Kulturkunde.Firnelfen" ),
                I18n.i18n( "Kulturkunde.Fjarninger" ),
                I18n.i18n( "Kulturkunde.Gjalskerlaender" ),
                I18n.i18n( "Kulturkunde.Goblins" ),
                I18n.i18n( "Kulturkunde.Grolme" ),
                I18n.i18n( "Kulturkunde.Horasreich" ),
                I18n.i18n( "Kulturkunde.Huegelzwerge" ),
                I18n.i18n( "Kulturkunde.Nivesen" ),
                I18n.i18n( "Kulturkunde.Norbarden" ),
                I18n.i18n( "Kulturkunde.NordaventurischeStaedte" ),
                I18n.i18n( "Kulturkunde.Maraskan" ),
                I18n.i18n( "Kulturkunde.Mohas" ),
                I18n.i18n( "Kulturkunde.Mittelreich" ),
                I18n.i18n( "Kulturkunde.Novadi" ),
                I18n.i18n( "Kulturkunde.Orks" ),
                I18n.i18n( "Kulturkunde.SchwarzeLande" ),
                I18n.i18n( "Kulturkunde.StammesAchaz" ),
                I18n.i18n( "Kulturkunde.Steppenelfen" ),
                I18n.i18n( "Kulturkunde.Suedaventurien" ),
                I18n.i18n( "Kulturkunde.Svellttal" ),
                I18n.i18n( "Kulturkunde.Thorwal" ),
                I18n.i18n( "Kulturkunde.Tocamuyac" ),
                I18n.i18n( "Kulturkunde.Trolle" ),
                I18n.i18n( "Kulturkunde.Trollzacker" ),
                I18n.i18n( "Kulturkunde.Tulamidenlande" ),
                I18n.i18n( "Kulturkunde.Waldelfen" ),
                I18n.i18n( "Kulturkunde.Zahori" ),
                I18n.i18n( "Kulturkunde.Zyklopeninseln" ) );

        initialisieren( SICH_VERKLEIDEN, SKT.B, MU, CH, GE );
        SICH_VERKLEIDEN.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.AndererStand" ),
                I18n.i18n( "Talent.Spezialisierung.AnderesGeschlecht" ),
                I18n.i18n( "Talent.Spezialisierung.AndereRasse" ) );

        initialisieren( UEBERREDEN, SKT.B, MU, IN, CH );
        UEBERREDEN.einfuegen( Voraussetzung.MENSCHENKENNTNIS_4_STEIGERN );
        UEBERREDEN.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.Aufwiegeln" ),
                I18n.i18n( "Talent.Spezialisierung.Betteln" ),
                I18n.i18n( "Talent.Spezialisierung.Einschuechtern" ),
                I18n.i18n( "Talent.Spezialisierung.Feilschen" ),
                I18n.i18n( "Talent.Spezialisierung.Luegen" ) );

        initialisieren( UEBERZEUGEN, SKT.B, KL, IN, CH );
        UEBERZEUGEN.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.DemagogiePlaedoyerPredigt" ),
                I18n.i18n( "Talent.Spezialisierung.EinzelgespraechVerhoer" ),
                I18n.i18n( "Talent.Spezialisierung.DiskussionsRethorik" ),
                I18n.i18n( "Talent.Spezialisierung.SchriftlicheRethorik" ) );

        /*
         * NATUR-TALENTE
         */
        initialisieren( FAEHRTENSUCHEN, SKT.B, KL, IN, KO, IN );
        FAEHRTENSUCHEN.einfuegen( Voraussetzung.SINNENSCHAERFE_4_STEIGERN );
        FAEHRTENSUCHEN.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.EisSchnee" ),
                I18n.i18n( "Talent.Spezialisierung.Gebirge" ),
                I18n.i18n( "Talent.Spezialisierung.Grasland" ),
                I18n.i18n( "Talent.Spezialisierung.Stadt" ),
                I18n.i18n( "Talent.Spezialisierung.Sumpf" ),
                I18n.i18n( "Talent.Spezialisierung.Unterirdisch" ),
                I18n.i18n( "Talent.Spezialisierung.WaldDschungel" ),
                I18n.i18n( "Talent.Spezialisierung.Wueste" ) );

        initialisieren( FALLENSTELLEN, SKT.B, KL, FF, KK );
        FALLENSTELLEN.einfuegen( Voraussetzung.WILDNISLEBEN_4_STEIGERN );
        FALLENSTELLEN.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.Gruben" ),
                I18n.i18n( "Talent.Spezialisierung.Schlingen" ),
                I18n.i18n( "Talent.Spezialisierung.Speere" ) );

        initialisieren( FESSELN_ENTFESSELN,
                        Kosten.SCHLANGENMENSCH,
                        SKT.B,
                        Stufen.TALENT,
                        FF, GE, KK );
        FESSELN_ENTFESSELN.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.Entfesseln" ),
                I18n.i18n( "Talent.Spezialisierung.Fesseln" ),
                I18n.i18n( "Talent.Spezialisierung.Knotenkunde" ),
                I18n.i18n( "Talent.Spezialisierung.TaueSpleissen" ),
                I18n.i18n( "Talent.Spezialisierung.NetzeKnuepfen" ) );

        initialisieren( FISCHEN_ANGELN, SKT.B, IN, FF, KK );
        FISCHEN_ANGELN.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.FluesseBaeche" ),
                I18n.i18n( "Talent.Spezialisierung.Hochseeangeln" ),
                I18n.i18n( "Talent.Spezialisierung.SeenSumpfBrackwasser" ),
                I18n.i18n( "Talent.Spezialisierung.StrandRiff" ) );

        initialisieren( ORIENTIERUNG, SKT.B, KL, IN, IN );
        ORIENTIERUNG.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.EisSchnee" ),
                I18n.i18n( "Talent.Spezialisierung.Dschungel" ),
                I18n.i18n( "Talent.Spezialisierung.Gebirge" ),
                I18n.i18n( "Talent.Spezialisierung.Grasland" ),
                I18n.i18n( "Talent.Spezialisierung.Meer" ),
                I18n.i18n( "Talent.Spezialisierung.Stadt" ),
                I18n.i18n( "Talent.Spezialisierung.Unterirdisch" ),
                I18n.i18n( "Talent.Spezialisierung.Wald" ),
                I18n.i18n( "Talent.Spezialisierung.Wueste" ) );

        initialisieren( WETTERVORHERSAGE, SKT.B, KL, IN, IN );

        initialisieren( WILDNISLEBEN, SKT.B, IN, GE, KO );
        WILDNISLEBEN.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.Dschungel" ),
                I18n.i18n( "Talent.Spezialisierung.EisSchnee" ),
                I18n.i18n( "Talent.Spezialisierung.Gebirge" ),
                I18n.i18n( "Talent.Spezialisierung.Meer" ),
                I18n.i18n( "Talent.Spezialisierung.Steppe" ),
                I18n.i18n( "Talent.Spezialisierung.Sumpf" ),
                I18n.i18n( "Talent.Spezialisierung.Wald" ),
                I18n.i18n( "Talent.Spezialisierung.Wueste" ) );
        /*
         * WISSENS-TALENTE
         */
        initialisierenWissen( ANATOMIE, MU, KL, FF );
        ANATOMIE.einfuegen( Voraussetzung.NICHT_TOTENANGST );

        initialisierenWissen( BRETT_WUERFELSPIELE, KL, KL, IN );

        initialisierenWissen( GEOGRAPHIE, KL, KL, IN );

        initialisierenWissen( GESCHICHTSWISSEN, KL, KL, IN );
        GESCHICHTSWISSEN.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.BauKunstgeschichte" ),
                I18n.i18n( "Talent.Spezialisierung.Militaergeschichte" ),
                I18n.i18n( "Talent.Spezialisierung.Religionsgeschichte" ),
                I18n.i18n( "Talent.Spezialisierung.Wissenschaftsgeschichte" ) );

        initialisierenWissen( GESTEINSKUNDE, KL, IN, FF );
        GESTEINSKUNDE.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.Baugestein" ),
                I18n.i18n( "Talent.Spezialisierung.Edelmetalle" ),
                I18n.i18n( "Talent.Spezialisierung.Edelsteine" ) );

        initialisierenWissen( GOETTER_KULTE, KL, KL, IN );

        initialisierenWissen( HERALDIK, KL, KL, FF );

        initialisierenWissen( KRIEGSKUNST, MU, KL, CH );
        KRIEGSKUNST.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.Logistik" ),
                I18n.i18n( "Talent.Spezialisierung.Militaergeschichte" ),
                I18n.i18n( "Talent.Spezialisierung.Monstren" ),
                I18n.i18n( "Talent.Spezialisierung.Seegefechte" ),
                I18n.i18n( "Talent.Spezialisierung.Strategie" ),
                I18n.i18n( "Talent.Spezialisierung.Taktik" ),
                I18n.i18n( "Talent.Spezialisierung.DrachenBekaempfung" ) );

        initialisierenWissen( MAGIEKUNDE, KL, KL, IN );
        MAGIEKUNDE.einfuegen( Voraussetzung.KULTURSPRACHE_6_STEIGERN );
        MAGIEKUNDE.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.Artefaktherstellung" ),
                I18n.i18n( "Talent.Spezialisierung.Daemonologie" ),
                I18n.i18n( "Talent.Spezialisierung.Elementarismus" ),
                I18n.i18n( "Talent.Spezialisierung.Magiehistorie" ),
                I18n.i18n( "Talent.Spezialisierung.Magietheorie" ),
                I18n.i18n( "Talent.Spezialisierung.MagischeAnalyse" ),
                I18n.i18n( "Talent.Spezialisierung.Sphaerologie" ),
                I18n.i18n( "Talent.Spezialisierung.Zauberpraxis" ),
                I18n.i18n( "Talent.Spezialisierung.Zauberwerkstatt" ) );

        initialisierenWissen( MECHANIK, KL, KL, FF );
        MECHANIK.einfuegen( Voraussetzung.SCHRIFT_6_STEIGERN );
        MECHANIK.einfuegen( Voraussetzung.RECHNEN_6_STEIGERN );
        MECHANIK.einfuegen( Voraussetzung.MALEN_ZEICHNEN_6_STEIGERN );
        MECHANIK.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.Belagerungswaffen" ),
                I18n.i18n( "Talent.Spezialisierung.KraeneHebewerkeMahlwerke" ),
                I18n.i18n( "Talent.Spezialisierung.Materialkunde" ),
                I18n.i18n( "Talent.Spezialisierung.PumpenWasserkraft" ),
                I18n.i18n( "Talent.Spezialisierung.Windkraft" ) );

        initialisierenWissen( PFLANZENKUNDE, KL, IN, FF );
        PFLANZENKUNDE.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.Dschungel" ),
                I18n.i18n( "Talent.Spezialisierung.Gebirge" ),
                I18n.i18n( "Talent.Spezialisierung.Grasland" ),
                I18n.i18n( "Talent.Spezialisierung.Meer" ),
                I18n.i18n( "Talent.Spezialisierung.Steppe" ),
                I18n.i18n( "Talent.Spezialisierung.Sumpf" ),
                I18n.i18n( "Talent.Spezialisierung.Tundra" ),
                I18n.i18n( "Talent.Spezialisierung.Wald" ),
                I18n.i18n( "Talent.Spezialisierung.Wueste" ) );

        initialisierenWissen( RECHNEN, KL, KL, IN );
        RECHNEN.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.Arithmetik" ),
                I18n.i18n( "Talent.Spezialisierung.Buchfuehrung" ),
                I18n.i18n( "Talent.Spezialisierung.Geometrie" ) );

        initialisierenWissen( RECHTSKUNDE, KL, KL, IN );
        RECHTSKUNDE.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.Gildenrecht" ),
                I18n.i18n( "Talent.Spezialisierung.Kirchenrecht" ),
                I18n.i18n( "Talent.Spezialisierung.Strafrecht" ),
                I18n.i18n( "Talent.Spezialisierung.Staatsrecht" ) );

        initialisierenWissen( SAGEN_LEGENDEN, KL, IN, CH );
        SAGEN_LEGENDEN.setSpezialisierungen(
                I18n.i18n( "Kulturkunde.Almada" ),
                I18n.i18n( "Kulturkunde.Amazonen" ),
                I18n.i18n( "Kulturkunde.Ambosszwerge" ),
                I18n.i18n( "Kulturkunde.AndergastNostria" ),
                I18n.i18n( "Kulturkunde.Aranien" ),
                I18n.i18n( "Kulturkunde.ArchaischeAchaz" ),
                I18n.i18n( "Kulturkunde.Auelfen" ),
                I18n.i18n( "Kulturkunde.Bornland" ),
                I18n.i18n( "Kulturkunde.Brilliantzwerge" ),
                I18n.i18n( "Kulturkunde.Bukanier" ),
                I18n.i18n( "Kulturkunde.Erzzwerge" ),
                I18n.i18n( "Kulturkunde.Ferkina" ),
                I18n.i18n( "Kulturkunde.Firnelfen" ),
                I18n.i18n( "Kulturkunde.Fjarninger" ),
                I18n.i18n( "Kulturkunde.Gjalskerlaender" ),
                I18n.i18n( "Kulturkunde.Goblins" ),
                I18n.i18n( "Kulturkunde.Grolme" ),
                I18n.i18n( "Kulturkunde.Horasreich" ),
                I18n.i18n( "Kulturkunde.Huegelzwerge" ),
                I18n.i18n( "Kulturkunde.Nivesen" ),
                I18n.i18n( "Kulturkunde.Norbarden" ),
                I18n.i18n( "Kulturkunde.NordaventurischeStaedte" ),
                I18n.i18n( "Kulturkunde.Maraskan" ),
                I18n.i18n( "Kulturkunde.Mohas" ),
                I18n.i18n( "Kulturkunde.Mittelreich" ),
                I18n.i18n( "Kulturkunde.Novadi" ),
                I18n.i18n( "Kulturkunde.Orks" ),
                I18n.i18n( "Kulturkunde.SchwarzeLande" ),
                I18n.i18n( "Kulturkunde.StammesAchaz" ),
                I18n.i18n( "Kulturkunde.Steppenelfen" ),
                I18n.i18n( "Kulturkunde.Suedaventurien" ),
                I18n.i18n( "Kulturkunde.Svellttal" ),
                I18n.i18n( "Kulturkunde.Thorwal" ),
                I18n.i18n( "Kulturkunde.Tocamuyac" ),
                I18n.i18n( "Kulturkunde.Trolle" ),
                I18n.i18n( "Kulturkunde.Trollzacker" ),
                I18n.i18n( "Kulturkunde.Tulamidenlande" ),
                I18n.i18n( "Kulturkunde.Waldelfen" ),
                I18n.i18n( "Kulturkunde.Zahori" ),
                I18n.i18n( "Kulturkunde.Zyklopeninseln" ) );


        initialisierenWissen( SCHAETZEN, KL, IN, IN );
        SCHAETZEN.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.Antiquitaeten" ),
                I18n.i18n( "Talent.Spezialisierung.Materialwert" ),
                I18n.i18n( "Talent.Spezialisierung.Schmuck" ) );

        initialisierenWissen( SPRACHENKUNDE, KL, KL, IN );

        initialisierenWissen( STERNKUNDE, KL, KL, IN );
        STERNKUNDE.einfuegen( Voraussetzung.GARETHI_TULAMIDYA_6_STEIGERN );
        STERNKUNDE.einfuegen( Voraussetzung.RECHNEN_6_STEIGERN );
        STERNKUNDE.einfuegen( Voraussetzung.SINNENSCHAERFE_6_STEIGERN );
        STERNKUNDE.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.Himmelskartographie" ),
                I18n.i18n( "Talent.Spezialisierung.Horoskope" ),
                I18n.i18n( "Talent.Spezialisierung.Navigation" ),
                I18n.i18n( "Talent.Spezialisierung.Zeitbestimmung" ) );

        initialisierenWissen( STAATSKUNST, KL, IN, CH );
        STAATSKUNST.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.Diplomatie" ),
                I18n.i18n( "Talent.Spezialisierung.Intrige" ),
                I18n.i18n( "Talent.Spezialisierung.Verwaltung" ) );

        initialisierenWissen( TIERKUNDE, MU, KL, IN );
        TIERKUNDE.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.Drachen" ),
                I18n.i18n( "Talent.Spezialisierung.Jagdwild" ),
                I18n.i18n( "Talent.Spezialisierung.Meeresgetier" ),
                I18n.i18n( "Talent.Spezialisierung.Monstren" ),
                I18n.i18n( "Talent.Spezialisierung.Nutztiere" ),
                I18n.i18n( "Talent.Spezialisierung.Raubtiere" ),
                I18n.i18n( "Talent.Spezialisierung.Schaedlinge" ),
                I18n.i18n( "Talent.Spezialisierung.Vielbeiner" ),
                I18n.i18n( "Talent.Spezialisierung.Voegel" ) );

        /*
         * SPRACHEN
         */

        initialisiereSprache( ALAANI, SKT.A, 21, Sprachfamilie.TULAMIDYA,
                KUSLIKER_ZEICHEN );

        initialisiereSprache( ASDHARIA, SKT.B, 24, Sprachfamilie.ASDHARIA,
                ASDHARIA_SCHRIFT );

        initialisiereSprache( ATAK, SKT.A, 12, Sprachfamilie.ATAK );

        initialisiereSprache( BOSPARANO, SKT.A, 21, Sprachfamilie.GARETHI,
                KUSLIKER_ZEICHEN );

        initialisiereSprache(
                FUECHSISCH, SKT.A, 12, Sprachfamilie.FUECHSISCH );

        initialisiereSprache( GARETHI, SKT.A, 18, Sprachfamilie.GARETHI,
                KUSLIKER_ZEICHEN );

        initialisiereSprache(
                GOBLINISCH, SKT.A, 12, Sprachfamilie.GOBLINISCH );

        initialisiereSprache( ISDIRA, SKT.A, 21, Sprachfamilie.ISDIRA,
                ISDIRA_SCHRIFT );

        initialisiereSprache( MOHISCH, SKT.A, 15, Sprachfamilie.MOHISCH );

        initialisiereSprache( NUJUKA, SKT.A, 15, Sprachfamilie.NUJUKA );

        initialisiereSprache( OLOARKH, SKT.A, 10, Sprachfamilie.OLOARKH );

        initialisiereSprache(
                OLOGHAIJAN, SKT.A, 15, Sprachfamilie.OLOGHAIJAN );

        initialisiereSprache( ROGOLAN, SKT.A, 21, Sprachfamilie.ROGOLAN,
                ROGOLAN_SCHRIFT );

        initialisiereSprache( RSSAHH, SKT.A, 18, Sprachfamilie.RSSAHH,
                CHRMK );

        initialisiereSprache(
                THORWALSCH, SKT.A, 18, Sprachfamilie.THORWALSCH,
                KUSLIKER_ZEICHEN );

        initialisiereSprache( TULAMIDYA, SKT.A, 18, Sprachfamilie.TULAMIDYA,
                GLYPHEN_VON_UNAU, TULAMIDYA_SCHRIFT );

        initialisiereSprache(
                UR_TULAMIDYA, SKT.A, 21, Sprachfamilie.TULAMIDYA,
                UR_TULAMIDYA_SCHRIFT );

        initialisiereSprache( ZHAYAD, SKT.A, 15, Sprachfamilie.ZAHYAD,
                ZHAYAD_SCHRIFT );

        /*
         *  SCHRIFTEN
         */

        initialisiereSchrift( ASDHARIA_SCHRIFT, SKT.A, 18 );

        initialisiereSchrift( CHRMK, SKT.B, 18 );

        initialisiereSchrift( GLYPHEN_VON_UNAU, SKT.A, 13 );

        initialisiereSchrift( HJALDINGSCHE_RUNEN, SKT.A, 10 );

        initialisiereSchrift( ISDIRA_SCHRIFT, SKT.A, 15 );

        initialisiereSchrift( KUSLIKER_ZEICHEN, SKT.A, 10 );

        initialisiereSchrift( NANDURIA, SKT.A, 10 );

        initialisiereSchrift( ROGOLAN_SCHRIFT, SKT.A, 11 );

        initialisiereSchrift( TULAMIDYA_SCHRIFT, SKT.A, 14 );

        initialisiereSchrift( UR_TULAMIDYA_SCHRIFT, SKT.A, 16 );

        initialisiereSchrift( ZHAYAD_SCHRIFT, SKT.A, 18 );

        /*
         * HANDWERKS-TALENTE
         */
        initialisieren( ABRICHTEN, SKT.B, MU, IN, CH );
        ABRICHTEN.einfuegen( Voraussetzung.TIERKUNDE_4_STEIGERN );
        ABRICHTEN.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.Dompteur" ),
                I18n.i18n( "Talent.Spezialisierung.Falkner" ),
                I18n.i18n( "Talent.Spezialisierung.Hundefuehrer" ),
                I18n.i18n( "Talent.Spezialisierung.Zureiter" ) );

        initialisieren( ACKERBAU, SKT.B, IN, FF, KO );

        initialisieren( ALCHIMIE, SKT.B, MU, KL, FF );
        ALCHIMIE.einfuegen( Voraussetzung.SCHRIFT_4 );
        ALCHIMIE.einfuegen( Voraussetzung.RECHNEN_4 );
        ALCHIMIE.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.Brandmittel" ),
                I18n.i18n( "Talent.Spezialisierung.Gifte" ),
                I18n.i18n( "Talent.Spezialisierung.Laborpraxis" ),
                I18n.i18n( "Talent.Spezialisierung.ZauberElixiere" ),
                I18n.i18n( "Talent.Spezialisierung.FarbenLacke" ),
                I18n.i18n( "Talent.Spezialisierung.Materialveredelung" ) );

        initialisieren( BERGBAU, SKT.B, IN, KO, KK );
        BERGBAU.einfuegen( Voraussetzung.GESTEINSKUNDE_4 );
        BERGBAU.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.Bergwerke" ),
                I18n.i18n( "Talent.Spezialisierung.Kanalisation" ),
                I18n.i18n( "Talent.Spezialisierung.NatuerlicheHoehlen" ),
                I18n.i18n( "Talent.Spezialisierung.Sappeur" ),
                I18n.i18n( "Talent.Spezialisierung.UnterirdischeStaedte" ) );

        initialisieren( BOGENBAU, SKT.B, KL, IN, FF );
        BOGENBAU.einfuegen( Voraussetzung.HOLZBEARBEITUNG_4 );
        BOGENBAU.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.Armbrust" ),
                I18n.i18n( "Talent.Spezialisierung.Bogen" ),
                I18n.i18n( "Talent.Spezialisierung.BolzenPfeile" ),
                I18n.i18n( "Talent.Spezialisierung.Torsionswaffen" ) );

        initialisieren( BOOTE_FAHREN, SKT.B, GE, KO, KK );
        BOOTE_FAHREN.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.Einmaster" ),
                I18n.i18n( "Talent.Spezialisierung.Floesse" ),
                I18n.i18n( "Talent.Spezialisierung.KanusKajaks" ),
                I18n.i18n( "Talent.Spezialisierung.Ruderboote" ) );

        initialisieren( FAHRZEUG_LENKEN, SKT.B, IN, CH, FF );
        FAHRZEUG_LENKEN.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.Lastkarren" ),
                I18n.i18n( "Talent.Spezialisierung.Streitwagen" ),
                I18n.i18n( "Talent.Spezialisierung.Zuggespanne" ) );

        initialisieren( FALSCHSPIEL, SKT.B, MU, CH, FF );
        FALSCHSPIEL.einfuegen( Voraussetzung.MENSCHENKENNTNIS_4 );
        FALSCHSPIEL.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.Wuerfel" ),
                I18n.i18n( "Talent.Spezialisierung.Karten" ) );

        initialisieren( FEINMECHANIK, SKT.B, KL, FF, FF );
        FEINMECHANIK.einfuegen( Voraussetzung.MALEN_ZEICHNEN_4 );
        FEINMECHANIK.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.GoldSilberschmied" ),
                I18n.i18n( "Talent.Spezialisierung.Gravieren" ),
                I18n.i18n( "Talent.Spezialisierung.Schloesser" ),
                I18n.i18n( "Talent.Spezialisierung.Siegelstoecke" ),
                I18n.i18n( "Talent.Spezialisierung.TrickwaffenFallen" ),
                I18n.i18n( "Talent.Spezialisierung.Uhrwerke" ) );

        initialisieren( FLEISCHER, SKT.B, KL, FF, KK );
        FLEISCHER.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.Fische" ),
                I18n.i18n( "Talent.Spezialisierung.Gefluegel" ),
                I18n.i18n( "Talent.Spezialisierung.Reptilien" ),
                I18n.i18n( "Talent.Spezialisierung.Saeugetiere" ) );

        initialisieren( GERBER_KUERSCHNER, SKT.B, KL, FF, KO );
        GERBER_KUERSCHNER.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.Gerber" ),
                I18n.i18n( "Talent.Spezialisierung.Kuerschner" ),
                I18n.i18n( "Talent.Spezialisierung.Trophaeen" ) );

        initialisieren( GROBSCHMIED, SKT.B, FF, KO, KK );
        GROBSCHMIED.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.Drahtzieher" ),
                I18n.i18n( "Talent.Spezialisierung.Hufschmied" ),
                I18n.i18n( "Talent.Spezialisierung.Plaettner" ),
                I18n.i18n( "Talent.Spezialisierung.Schwarzschmied" ),
                I18n.i18n( "Talent.Spezialisierung.Spengler" ),
                I18n.i18n( "Talent.Spezialisierung.Waffenschmied" ) );

        initialisieren( HEILKUNDE_GIFT, SKT.B, MU, KL, IN );
        HEILKUNDE_GIFT.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.AlchimistischeZaubergifte" ),
                I18n.i18n( "Talent.Spezialisierung.MineralischeGifte" ),
                I18n.i18n( "Talent.Spezialisierung.PflanzlicheGifte" ),
                I18n.i18n( "Talent.Spezialisierung.TierischeGifte" ),
                I18n.i18n( "Talent.Spezialisierung.Atemgifte" ),
                I18n.i18n( "Talent.Spezialisierung.Einnahmegifte" ),
                I18n.i18n( "Talent.Spezialisierung.Kontaktgifte" ),
                I18n.i18n( "Talent.Spezialisierung.Waffengifte" ) );

        initialisieren( HEILKUNDE_KRANKHEITEN, SKT.B, MU, KL, CH );

        initialisieren( HEILKUNDE_SEELE, SKT.B, IN, CH, CH );

        initialisieren( HEILKUNDE_WUNDEN, SKT.B, KL, CH, FF );
        HEILKUNDE_WUNDEN.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.BruecheQuetschungen" ),
                I18n.i18n( "Talent.Spezialisierung.Schnitte" ),
                I18n.i18n( "Talent.Spezialisierung.Verbrennungen" ),
                I18n.i18n( "Talent.Spezialisierung.Zahnleiden" ) );

        initialisieren( HOLZBEARBEITUNG, SKT.B, KL, FF, KK );
        HOLZBEARBEITUNG.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.Beinschnitzerei" ),
                I18n.i18n( "Talent.Spezialisierung.Holzkunde" ),
                I18n.i18n( "Talent.Spezialisierung.Holzwaffen" ),
                I18n.i18n( "Talent.Spezialisierung.KueferWagner" ),
                I18n.i18n( "Talent.Spezialisierung.MoebelschreinereiTischlerei" ),
                I18n.i18n( "Talent.Spezialisierung.Schnitzen" ) );

        initialisieren( KARTOGRAPHIE, SKT.B, KL, KL, FF );
        KARTOGRAPHIE.einfuegen( Voraussetzung.MALEN_ZEICHNEN_4 );
        KARTOGRAPHIE.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.Gebaeude" ),
                I18n.i18n( "Talent.Spezialisierung.HoehlenTunnel" ),
                I18n.i18n( "Talent.Spezialisierung.KuestenMeere" ),
                I18n.i18n( "Talent.Spezialisierung.Landschaften" ),
                I18n.i18n( "Talent.Spezialisierung.Stadt" ) );

        initialisieren( KOCHEN, SKT.B, KL, IN, FF );
        KOCHEN.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.BackenBraten" ),
                I18n.i18n( "Talent.Spezialisierung.Festmahle" ),
                I18n.i18n( "Talent.Spezialisierung.Haltbarmachen" ),
                I18n.i18n( "Talent.Spezialisierung.Marschversorgung" ),
                I18n.i18n( "Talent.Spezialisierung.Traenke" ),
                I18n.i18n( "Talent.Spezialisierung.Vorkoster" ) );

        initialisieren( LEDERARBEITEN, SKT.B, KL, FF, FF );
        LEDERARBEITEN.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.GurteRiemen" ),
                I18n.i18n( "Talent.Spezialisierung.Lederkleidung" ),
                I18n.i18n( "Talent.Spezialisierung.Lederruestungen" ),
                I18n.i18n( "Talent.Spezialisierung.Saettel" ),
                I18n.i18n( "Talent.Spezialisierung.Schuhwerk" ) );

        initialisieren( MALEN_ZEICHNEN, SKT.B, KL, IN, FF );
        MALEN_ZEICHNEN.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.Architektur" ),
                I18n.i18n( "Talent.Spezialisierung.Landschaften" ),
                I18n.i18n( "Talent.Spezialisierung.Portraet" ),
                I18n.i18n( "Talent.Spezialisierung.TechnischeSkizze" ) );

        initialisieren( MUSIZIEREN, SKT.B, IN, CH, FF );
        MUSIZIEREN.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.Blechblasinstrumente" ),
                I18n.i18n( "Talent.Spezialisierung.Floeten" ),
                I18n.i18n( "Talent.Spezialisierung.Harfen" ),
                I18n.i18n( "Talent.Spezialisierung.Lauten" ),
                I18n.i18n( "Talent.Spezialisierung.Rhythmusinstrumente" ),
                I18n.i18n( "Talent.Spezialisierung.Streichinstrumente" ),
                I18n.i18n( "Talent.Spezialisierung.Tasteninstrumente" ) );

        initialisieren( SCHLOESSER_KNACKEN, SKT.B, IN, FF, FF );
        SCHLOESSER_KNACKEN.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.Fallen" ),
                I18n.i18n( "Talent.Spezialisierung.Schloesser" ) );

        initialisieren( SCHNEIDERN, SKT.B, KL, FF, FF );
        SCHNEIDERN.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.EntwurfFestgewaender" ),
                I18n.i18n( "Talent.Spezialisierung.FlickGebrauchsschneiderei" ),
                I18n.i18n( "Talent.Spezialisierung.Tuchruestungen" ) );

        initialisieren( SEEFAHRT, SKT.B, FF, GE, KK );
        SEEFAHRT.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.Ruderschiff" ),
                I18n.i18n( "Talent.Spezialisierung.Segelschiff" ) );

        initialisieren( STEINMETZ, SKT.B, FF, FF, KK );
        STEINMETZ.einfuegen( Voraussetzung.GESTEINSKUNDE_4 );
        STEINMETZ.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.Baugestein" ),
                I18n.i18n( "Talent.Spezialisierung.Reliefs" ),
                I18n.i18n( "Talent.Spezialisierung.Statuen" ) );

        initialisieren( STEINSCHNEIDER, SKT.B, IN, FF, FF );
        STEINSCHNEIDER.einfuegen( Voraussetzung.GESTEINSKUNDE_4_STEIGERN );

        initialisieren( TAETOWIEREN, SKT.B, IN, FF, FF );
        TAETOWIEREN.einfuegen( Voraussetzung.MALEN_ZEICHNEN_4 );
        TAETOWIEREN.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.NaturalistischeDarstellung"),
                I18n.i18n( "Talent.Spezialisierung.Ornamente" ),
                I18n.i18n( "Talent.Spezialisierung.Zaubertaetowierungen" ) );

        initialisieren( ZIMMERMANN, SKT.B, KL, FF, KK );
        ZIMMERMANN.einfuegen( Voraussetzung.HOLZBEARBEITUNG_4_STEIGERN );
        ZIMMERMANN.setSpezialisierungen(
                I18n.i18n( "Talent.Spezialisierung.Dachdecker" ),
                I18n.i18n( "Talent.Spezialisierung.Holzkonstruktionen" ),
                I18n.i18n( "Talent.Spezialisierung.Schiffszimmermann" ) );

        /*
         * GABEN
         */
        initialisieren( GEFAHRENINSTINKT, SKT.F, KL, IN, IN );
        GEFAHRENINSTINKT.einfuegen( Voraussetzung.GEFAHRENINSTINKT );

        initialisieren( MAGIEGESPUER, SKT.F, MU, IN, IN );
        MAGIEGESPUER.einfuegen( Voraussetzung.MAGIEGESPUER );

        initialisieren( PROPHEZEIEN, SKT.F, IN, IN, CH );
        PROPHEZEIEN.einfuegen( Voraussetzung.PROPHEZEIEN );

        initialisieren( ZWERGENNASE, SKT.F, FF, IN, IN );
        ZWERGENNASE.einfuegen( Voraussetzung.ZWERGENNASE );

        /*
         * RITUALKENNTNIS
         */
        initialisieren( RITUALKENNTNIS, SKT.E, KL, null );
        RITUALKENNTNIS.einfuegen( Voraussetzung.NICHT_FREI_WAEHLBAR );

        Collections.sort( TALENTE );
    }

    /** @return eine alphabetisch sortierte Liste aller Basis-Talente. */
    public static final List< Talent > getBasisTalente() {
        return new ArrayList< Talent >( BASIS_TALENTE );
    }

    /**
     * @return eine alphabetisch sortierte Liste aller Fernkampf-Techniken.
     *      Die Liste ist leer, so lange die Methode {@link #initialisieren()}
     *      nicht ausgefuehrt wurde.
     */
    public static final List< Kampftechnik > getFernkampfTechniken() {
        return new ArrayList< Kampftechnik >( FERNKAMPF_TALENTE );
    }

    /**
     * @return eine alphabetisch sortierte Liste aller Talente, ausser den
     *      Gaben. Die Liste ist leer, so lange die Methode
     *      {@link #initialisieren()} nicht ausgefuehrt wurde.
     */
    public static final List< Talent > getTalente() {
        return new ArrayList< Talent >( TALENTE );
    }

    /**
     * Initialisiert alle Talente ausser Sprachen, Schriften, Kampftechniken und
     * Wissens-Talenten.
     *
     * @param talent
     *            das initialisiert werden soll.
     * @param steigerungsfaktor
     *            des Talents.
     * @param eigenschaft1
     * 			  die erste Probeneigenschaft
     * @param eigenschaft2
     * 			  die dritte Probeneigenschaft.
     * @param eigenschaften
     *            die dritte Probeneigenschaft oder eine Liste von
     *            Eigenschaften, wenn die Probe nicht immer gleich
     *            gewuerfelt wird.
     */
    private static final void initialisieren(
            final Talent talent,
            final int steigerungsfaktor,
            final Eigenschaft eigenschaft1,
            final Eigenschaft eigenschaft2,
            final Eigenschaft ... eigenschaften ) {

        initialisieren(
                talent,
                Kosten.TALENT,
                steigerungsfaktor,
                Stufen.TALENT,
                eigenschaft1,
                eigenschaft2,
                eigenschaften );
    }

    /**
     * Initialisiert ein Talent mit den uebergebenen Werten. Diese Methode muss
     * nicht direkt aufgerufen werden, es gibt noch andere
     * initialisiere-Methoden die sich dieser hier bedienen. Jede andere
     * initialisieren Methode ruft diese hier auf.
     *
     * @see #initialisieren(Talent, int,
     *      de.bernhardjung.dsaprogramm.talente.Talent.Typ, Eigenschaft[])
     * @see #initialisierenWissen(Talent, Eigenschaft[])
     * @see #initialisiereKampftechnik(Kampftechnik, int,
     *      de.bernhardjung.dsaprogramm.talente.Talent.Typ, Art)
     * @see #initialisiereSchrift(Schrift, int, int)
     * @see #initialisiereSprache(Sprache, int, int, Sprachfamilie, Schrift[])
     * @param talent
     *            das initialisiert werden soll.
     * @param kosten
     *            fuer das Talent
     * @param steigerungsfaktor
     *            fuer das Talent.
     * @param stufen
     *            fuer das Talent.
     * @param eigenschaft1
     * 			  die erste Probeneigenschaft
     * @param eigenschaft2
     * 			  die zweite Probeneigenschaft
     * @param eigenschaften
     * 			  die dritte Probeneigenschaft oder eine Liste von
     * 			  Eigenschaften, wenn die Probe nicht immer gleich
     * 			  gewuerfelt wird.
     */
    private static final void initialisieren(
            final Talent talent,
            final Kosten kosten,
            final int steigerungsfaktor,
            final Stufen stufen,
            final Eigenschaft eigenschaft1,
            final Eigenschaft eigenschaft2,
            final Eigenschaft ... eigenschaften  ) {

        talent.setKosten( kosten );
        talent.setSteigerungsspalte( steigerungsfaktor );
        talent.setStufen( stufen );
        talent.setProbe( eigenschaft1, eigenschaft2, eigenschaften );
        talent.einfuegen( Voraussetzung.STANDARD );

        int index = Collections.binarySearch( TALENTE, talent );

        assert ( index < 0 ) : String.format(
                "Das Talent %1$s wurde zum zweiten mal in die Liste der " +
                "Talente eingefuegt.",
                talent.getName() );

        index = -( index + 1 );

        if ( ! talent.istGabe() ) {
            TALENTE.add( index, talent );
        }

        TALENTE_MAP.put( talent.getId(), talent );
    }

    /**
     * Initialisiert die Wissens-Talente.
     *
     * @param talent
     * @param typ
     */
    private static final void initialisierenWissen(
            final Talent talent,
            final Eigenschaft eigenschaft1,
            final Eigenschaft eigenschaft2,
            final Eigenschaft ... eigenschaften ) {

        initialisieren(
                talent,
                Kosten.SCHRIFT_WISSEN,
                SKT.B,
                Stufen.TALENT,
                eigenschaft1,
                eigenschaft2,
                eigenschaften );
    }

    /**
     * Initialisiert eine Kampftechnik.
     *
     * @param kampftechnik
     *            die initialisiert werden soll.
     * @param steigerungsfaktor
     *            der Kampftechnik
     * @param eBe
     *            die Zahl, die fuer die effektive Behinderung von der
     *            Behinderung abgezogen wird.
     */
    private static final void initialisiereKampftechnik(
            final Kampftechnik kampftechnik,
            final int steigerungsfaktor,
            final int eBe ) {

        kampftechnik.setEBe( eBe );

        if ( kampftechnik.istNahkampfTechnik() ) {

            if ( kampftechnik != LANZENREITEN ) {

                kampftechnik.einfuegen( Voraussetzung.KAMPFTECHNIK );

            }
        } else {

        	int index = Collections.binarySearch(
                    FERNKAMPF_TALENTE, kampftechnik );

            index = -( index + 1 );

            FERNKAMPF_TALENTE.add( index, kampftechnik );
        }

        if ( kampftechnik.istNahkampfTechnik() ) {

        	initialisieren(
        			kampftechnik,
        			Kosten.KAMPFTECHNIK,
        			steigerungsfaktor,
        			Stufen.TALENT,
        			GE, GE, KK );

        } else {

        	initialisieren(
                    kampftechnik,
                    Kosten.KAMPFTECHNIK,
                    steigerungsfaktor,
                    Stufen.TALENT,
                    GE, FF, KK );

        }
    }

    private static final void initialisiereSchrift(
            final Schrift schrift,
            final int steigerungsfaktor,
            final int komplexitaet ) {

        initialisieren(
                schrift,
                Kosten.SCHRIFT_WISSEN,
                steigerungsfaktor,
                Stufen.SCHRIFT,
                KL, KL, FF );

        schrift.setKomplexitaet( komplexitaet );
    }
    /**
     * Initialisiert eine Sprache.
     *
     * @param sprache die initialisiert werden soll.
     * @param steigerungsfaktor der Sprache.
     * @param komplexitaet bis zu der die Sprache gesteigert werden darf.
     * @param sprachfamilie zu der die Sprache gehoert.
     * @param schriften in der die Sprache geschrieben wird.
     */
    private static final void initialisiereSprache(
            final Sprache sprache,
            final int steigerungsfaktor,
            final int komplexitaet,
            final Sprachfamilie sprachfamilie,
            final Schrift ... schriften ) {

        initialisieren(
                sprache,
                Kosten.SPRACHE,
                steigerungsfaktor,
                Stufen.SPRACHE,
                KL, IN, CH );

        sprache.setKomplexitaet( komplexitaet );
        sprache.setSprachfamilie( sprachfamilie );
        for ( final Schrift schrift : schriften ) {
            sprache.einfuegen( schrift );
        }
    }

    /** Die Id zum Identifizieren des Talents. */
    private final String id;

    /**
     * Das Objekt, das sich um die Berechnung der Steigerungskosten des
     * Talents kuemmert.
     */
    private Kosten kosten;

    /** Der Name des Talents. */
    private final String name;

    /** Die Probe die auf das Talent abgelegt wird. */
    private Eigenschaftsprobe probe;

    /** Die moeglichen Talentspezialisierungen. */
    private final List< String > spezialisierungen;

    /** Die Steigerungsspalte des Talents. */
    private int steigerungsspalte;

    /**
     * Ein Objekt, das sich um die Berechnung der Stufe und der
     * Maximal-/Minimal-Stufe kuemmert.
     */
    private Stufen stufen;

    /** Die Talengruppe zu der das Talent gehoert. */
    private final Talentgruppe talentgruppe;

    /**
     * Der Typ des Talents. Bestimmt ob es ein Basis-Talent, Spezial-Talent
     * oder eine Gabe ist.
     */
    private final Typ typ;

    /**
     * Die Voraussetzungen die fuer das Talent gelten.
     */
    private final Collection< Voraussetzung > voraussetzungen;

    /**
     * @param dieId des Talents.
     * @param dieTalentgruppe zu der das Talent gehoert.
     * @throws AssertionError wenn {@code derName} oder {@code dieTalentgruppe}
     *      {@code null} ist.
     */
    @SuppressWarnings("unchecked")
    Talent( final String dieId,
            final Talentgruppe dieTalentgruppe,
            final Typ derTyp ) {

        assert( null != dieId ) :
            "dieId darf nicht null sein.";

        assert( ! dieId.equals( "" ) ) :
            "derId darf kein leerer String sein.";

        assert( null != dieTalentgruppe ) : String.format(
            "Als Talentgruppe fuer das Talent %1$s wurde null uebergeben.",
            dieId );

        id = dieId;

        name = I18n.i18n( id + ".Name" );

        spezialisierungen = new ArrayList< String >();

        steigerungsspalte = -1;

        talentgruppe = dieTalentgruppe;

        talentgruppe.einfuegen( this );

        typ = derTyp;

        voraussetzungen = new ArrayList< Voraussetzung >();

        if ( Typ.BASIS == typ ) {

            int index = Collections.binarySearch( BASIS_TALENTE, this );

            index = -( index + 1 );

            BASIS_TALENTE.add( index, this );
        }
    }

    /**
     * Sortiert die Talente alphabetisch nach Name.
     */
    public int compareTo( final Talent o ) {
        return COLLATOR.compare( getName(), o.getName() );
    }

    /**
     * Fuegt dem Talent eine weitere Voraussetzung hinzu.
     *
     * @param voraussetzung fuer das Talent.
     */
    final void einfuegen( final Voraussetzung voraussetzung ) {
        voraussetzungen.add( voraussetzung );
    }

    /**
     * @param spezialisierung fuer das Talent.
     */
    public final void einfuegenSpezialisierung( final String spezialisierung ) {

        int index = Collections.binarySearch(
                spezialisierungen,
                spezialisierung,
                COLLATOR );

        if ( index < 0 ) {

            index = - ( index + 1 );

            spezialisierungen.add( index, spezialisierung );
        }
    }

    /**
     * @param spezialisierung
     *            die aus der Liste der Spezialisierungen entfernt werden soll.
     */
    public final void entferneSpezialisierung( final String spezialisierung ) {

        spezialisierungen.remove( spezialisierung );
    }

    /**
     * @return die Regelelemente von denen das Talent in irgendeiner Weise
     *         abhaengig ist.
     */
    public final Collection< Object > getAbhaengigkeiten() {

        final Collection< Object > abhaengigkeiten = new HashSet< Object >();

        abhaengigkeiten.add( this );

        abhaengigkeiten.addAll( probe.getEigenschaften() );

        abhaengigkeiten.addAll( kosten.getAbhaengigkeiten() );

        abhaengigkeiten.addAll( stufen.getAbhaengigkeiten() );

        for ( final Voraussetzung voraussetzung : voraussetzungen ) {

            abhaengigkeiten.addAll( voraussetzung.getAbhaengigkeiten() );
        }

        return abhaengigkeiten;
    }

    /**
     * @param held
     *            der das Talent aktivieren will.
     * @return die AP die der Held zahlen muss um das Talent zu aktivieren.
     */
    public final int getAktivierungskosten( final Held held ) {
        return getAktivierungskosten( held, Lernmethode.KEINE );
    }

    /**
     * @param held
     *            der das Talent aktivieren will.
     * @param lernmethode
     *            die zum Aktivieren verwendet werden soll.
     * @return die AP die der Held zahlen muss um das Talent zu aktivieren.
     */
    public final int getAktivierungskosten(
            final Held held,
            final Lernmethode lernmethode ) {
        return kosten.getAktivierungskosten( held, this, lernmethode );
    }

    /**
     * @param held fuer den die Hoechsstufe berechnet werden soll.
     * @return die hoechste noch erlaubte Stufe fuer den Helden in diesem
     *      Talent.
     */
    public final int getHoechststufe( final Held held ) {
        return stufen.getHoechststufe( held, this );
    }

    /** @return die Id des Talents. */
    public final String getId() {
        return id;
    }

    /** @return den Namen des Talents. */
    public final String getName() {
        return name;
    }

    /**
     * @return die Proben die auf diese Eigenschaft gewuerfelt werden.
     */
    public final Eigenschaftsprobe getProbe() {
        return probe;
    }

    /**
     * @return die Eigenschaften auf die bei einer Talentprobe gewuerfelt
     *      werden muss. Die Collection enthaelt jede Eigenschaft nur einmal,
     *      auch wenn mehrmals auf sie gewuerfelt wird.
     *      Im Fall der Kampftechniken, auf die keine Probe abgelegt wird,
     *      wird bei den Nahkampftechniken GE und KK geliefert, bei
     *      Fernkampftechniken FF, GE und KK.
     */
    public final Collection< Eigenschaft > getProbeneigenschaften() {

        return new HashSet< Eigenschaft >( probe.getEigenschaften() );
    }

    /**
     * @return die moeglichen Spezialisierungen des Talents fuer die
     *         Sonderfertigkeit Talentspezialisierung.
     */
    public final List< String > getSpezialisierungen() {
        return new ArrayList< String >( spezialisierungen );
    }

    /**
     * @see #getSteigerungsspalte(Held,int)
     * @return die Standardsteigerungsspalte des Talents. Die Steigerunsspalte
     *      der beim steigern eines Talentes gilt ist noch vom Helden und der
     *      Stufe des Talents abhaengig
     *      ({@link #getSteigerungsspalte(Held,int)}.
     */
    public final int getSteigerungsspalte() {
        return steigerungsspalte;
    }

    /**
     * Liefert die Steigerungsspalte fuer einen bestimmten Helden, in einem
     * Talent auf eine bestimmten Stufe gilt.
     *
     * @param held der das Talent steigern will.
     * @param stufe des Talents, das gesteigert werden soll.
     * @return die Steigerungsspalte des Talents fuer den Helden.
     */
    public final int getSteigerungsspalte( final Held held, final int stufe ) {

        return kosten.getSteigerungsspalte(
                held, this, stufe, Lernmethode.KEINE );
    }

    /**
     * Liefert die Steigerungsspalte fuer einen bestimmten Helden, in einem
     * Talent auf eine bestimmten Stufe gilt.
     *
     * @param held der das Talent steigern will.
     * @param stufe des Talents, das gesteigert werden soll.
     * @param lernmethode die zum Steigern verwendet werden soll.
     * @return den Steigerungsspalte des Talents fuer den Helden.
     */
    public final int getSteigerungsspalte(
            final Held held,
            final int stufe,
            final Lernmethode lernmethode ) {

        return kosten.getSteigerungsspalte( held, this, stufe, lernmethode );
    }

    /**
     * @return den Steigerungsfaktor des Talents.
     */
    public int getSteigerungsfaktor() {
        return SKT.getSteigerungsfaktor( steigerungsspalte );
    }

    /**
     * Liefert die AP-Kosten die zu bezahlen sind, um das Talent um einen Punkt,
     * zu steigern.
     *
     * @param held
     *            der das Talent steigern will.
     * @param stufe
     *            die das Talent aktuell besitzt.
     * @return die AP-Kosten um das Talent um einen Punkt zu steigern.
     */
    public final int getSteigerungskosten( final Held held, final int stufe ) {
        return getSteigerungskosten( held, stufe, Lernmethode.KEINE );
    }

    /**
     * Liefert die AP-Kosten die zu bezahlen sind, um das Talent um einen Punkt,
     * zu steigern.
     *
     * @param held
     *            der das Talent steigern will.
     * @param stufe
     *            die das Talent aktuell besitzt.
     * @param lernmethode
     *            die zum Steigern verwendet werden soll.
     * @return die AP-Kosten um das Talent um einen Punkt zu steigern.
     */
    public final int getSteigerungskosten(
            final Held held,
            final int stufe,
            final Lernmethode lernmethode ) {
        return kosten.getSteigerungskosten( held, this, stufe, lernmethode );
    }

    /**
     * Liefert die Modifikatin die ein Held auf ein Talent erhaelt.
     *
     * @param held der das Talent besitzt.
     * @return die Modifikation des Helden in dem Talent.
     */
    public final int getModifikation( final Held held ) {
        return stufen.getModifikation( held, this );
    }

    /** @return die Talentgruppe des Talents. */
    public final Talentgruppe getTalentgruppe() {
        return talentgruppe;
    }

    /**
     * @return {@code true} wenn es sich bei dem Talent um ein Basis-Talent
     *      handelt, {@code false} wenn es ein Spezial-Talent oder eine Gabe
     *      ist.
     */
    public final boolean istBasisTalent() {
        return Typ.BASIS == typ;
    }

    /**
     * @return {@code true} wenn es sich bei dem Talent um einr Gabe
     *      handelt, {@code false} wenn es ein normales Talent.
     */
    public final boolean istGabe() {
        return Typ.GABE == typ;
    }

    /**
     * Prueft ob der Held alle Voraussetzungen fuer dieses Talent einhaelt.
     *
     * @param held der das Talent waehlen will oder schon besitzt.
     * @return eine Beschreibung der aufgetretenen Probleme, wenn der Held
     *      nicht alle Voraussetzungen fuer das Talent erfuellt.
     */
    public final KonfliktBeschreibung pruefen( final Held held ) {

        final KonfliktBeschreibung beschreibung = new KonfliktBeschreibung();

        for ( final Voraussetzung voraussetzung : voraussetzungen ) {
            beschreibung.einfuegen( voraussetzung.pruefen( held, this ) );
        }

        return beschreibung;
    }

    /**
     * Bestimmt die Kosten des Talents. Die Methode sollte nur einmal
     * aufgerufen werden um das Talent zu initialisieren.
     *
     * @param dieKosten Das Objekt das sich um die Berechnung der Kosten des
     *      Talents kuemmert.
     * @throws AssertionError wenn die Kosten schon gesetzt wurden.
     */
    private final void setKosten( final Kosten dieKosten ) {

        assert( null == kosten ) : String.format(
                "Die Kosten von Talent %1$s sollten ein zweites mal gesetzt " +
                "werden.", getName() );

        kosten = dieKosten;
    }

    /**
     * Fuegt dem Talent eine weitere Eigenschaftsprobe hinzu.
     *
     * @param eigenschaft1 Die erste Eigenschaft auf die bei der Probe
     *      gewuerfelt wird.
     * @param eigenschaft2 Die zweite Eigenschaft auf die bei der Probe
     *      gewuerfelt wird.
     * @param eigenschaften Die dritte Eigenschaft auf die bei der Probe
     *      gewuerfelt wird, oder eine Reihe von Eigenschaften, wenn die
     *      Probe nicht immer gleich gewuerfelt wird.
     */
    private final void setProbe(
            final Eigenschaft eigenschaft1,
            final Eigenschaft eigenschaft2,
            final Eigenschaft ... eigenschaften ) {

        probe = new Eigenschaftsprobe(
                eigenschaft1,
                eigenschaft2,
                eigenschaften );
    }

    /**
     * Bestimmt die moeglichen Spezialisierungen fuer die Sonderfertigkeit
     * Talentspezialisierung.
     *
     * @param strings
     *            die moeglichen Talentspezialisierungen.
     */
    final void setSpezialisierungen( final String ... strings ) {

        spezialisierungen.clear();

        for ( final String spezialisierung : strings ) {

            // Ich hab nicht fuer alle Worte eine Uebersetzung gefunden.
            // Die Worte ohne Uebersetzung sind leere String, die aussortiert
            // werden muessen.
            if ( ! spezialisierung.equals( "" ) ) {

                spezialisierungen.add( spezialisierung );
            }
        }

        Collections.sort( spezialisierungen, COLLATOR );
    }

    /**
     * Bestimmt den Standard-Steigerungsfaktor des Talents.
     * Die Methode sollte nur einmal aufgerufen werden um das Talent zu
     * initialisieren.
     *
     * @param derSteigerungsspalte fuer das Talent.
     * @throws AssertionError wenn der Steigerungsfaktor schon festgelegt
     *      wurde.
     */
    private final void setSteigerungsspalte( final int derSteigerungsspalte ) {

        assert( -1 == steigerungsspalte ) : String.format(
                "Der Steigerungsfaktor von Talent %1$s sollte ein zweites " +
                "mal gesetzt werden.", getName() );

        steigerungsspalte = derSteigerungsspalte;
    }

    /**
     * Bestimmt das Objekt das sich um die Berechnung der Stufe des Talents
     * kuemmert.
     * Die Methode sollte nur einmal aufgerufen werden um das Talent zu
     * initialisieren.
     *
     * @param dieStufen Das Objekt das sich um die Berechnung der Stufe fuer
     *      das Talent kuemmert.
     * @throws AssertionError wenn die Stufe schon festgelegt wurde.
     */
    private final void setStufen( final Stufen dieStufen ) {

        assert( null == stufen ) : String.format(
                "Das Stufen-Objekt von Talent %1$s sollte ein zweites mal " +
                "gesetzt werden.", getName() );

        stufen = dieStufen;
    }

    /**
     * @return {@link #getName()}
     */
    @Override
    public String toString() {
        return getName();
    }
}
