/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.talente.voraussetzungen;

import java.util.Arrays;
import java.util.Collection;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.talente.Talent;

/**
 * Eine Voraussetzung die zur Aktivierung eines Talents erfuellt werden muss.
 * Diese Voraussetzung sieht so aus, dass man ein anderes Talent mit einer
 * Stufe von >= 4 beherrschen muss.
 *
 * @author Bernhard Jung
 */
class VoraussetzungAktivierung extends Voraussetzung {

    /**
     * Die Meldung die ausgegeben wird, wenn die Voraussetzung nicht erfuellt
     * wird.
     */
    private final String beschreibungString;

    /** Das Talent das mit einer Stufe von 4 voraussgestzt ist. */
    private final Talent vorausgesetztesTalent;

    /**
     * @param dasTalent das der Held mit einer Stufe von 4 besitzen muss.
     */
    VoraussetzungAktivierung( final Talent dasTalent ) {

        vorausgesetztesTalent = dasTalent;

        beschreibungString = I18n.i18n( "VoraussetzungAktivierung" );
    }

    /**
     * @return das vorausgesetzte Talent.
     */
    @Override
    public Collection<Object> getAbhaengigkeiten() {
        return Arrays.asList( (Object) vorausgesetztesTalent );
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public KonfliktBeschreibung pruefen(
            final Held held,
            final Talent talent ) {

        final KonfliktBeschreibung beschreibung = new KonfliktBeschreibung();

        // Die Voraussetzung soll nur ueberprueft werden wenn der Held das
        // auch aktivert hat.
        // Die Voraussetzung soll auch ueberprueft werden, wenn der Held das
        // Talent noch nicht besitzt, damit der Benutzer vor dem Aktivieren
        // schon sieht welche Voraussetzung zu erfuellen ist.
        // Der letzte Fall der bleibt ist, dass der Held das Talent mit einer
        // negativen Stufe besitzt.In diesem Fall soll die Voraussetzung nicht
        // ueberprueft werden, das sie dann nicht zu erfuellen ist.
        if ( ( ! held.hat( talent ) )
                || held.getWert( talent ).istAktiviert() ) {

            if ( ( ! held.hat( vorausgesetztesTalent ) )
                    || held.getStufe( vorausgesetztesTalent ) < 4 ) {

                beschreibung.einfuegen( String.format(
                        beschreibungString,
                        vorausgesetztesTalent ) );
            }
        }
        return beschreibung;
    }

}
