/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.zauber.voraussetzungen;

import java.util.Collection;
import java.util.Collections;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.zauber.ZauberWert;

/**
 * Prueft die Voraussetzungen die fuer jeden Zauber erfuellt sein muessen.
 * <ul>
 * <li>
 *     Ist der Zauber vorgegeben muss der Held ihn besitzen.
 * </li>
 * <li>
 *     Die Stufe des Zaubers muss zwischen Mindest- und Hoechststufe liegen.
 * </li>
 * </ul>
 *
 * @author Bernhard Jung
 *
 */
class VoraussetzungStandard implements Voraussetzung {

    /**
     * Beschreibt das die Stufe eines Zaubers zwischen zwei Grenzwerten liegen
     * muss. Der String wird mit {@link String#format(String, Object[])}
     * verarbeitet.<br>
     * Beispiel: 'Voraussetzung %1$s &le; TaW &le; %2$s'
     */
    private static final String GRENZEN =
        I18n.i18n( "VoraussetzungStandard.Zauber.Grenzen" );

    /**
     * Beschreibt das der Held den Zauber besitzen muss, weil er vorgegeben ist.
     */
    private static final String VORGEGEBEN =
        I18n.i18n( "VoraussetzungStandard.Zauber.Vorgegeben" );

    /**
     * @return leere Liste.
     */
    @SuppressWarnings("unchecked")
    public Collection<Object> getAbhaengigkeiten() {
        return Collections.EMPTY_LIST;
    }

    /** {@inheritDoc} */
    public KonfliktBeschreibung pruefen(
            final Held held,
            final ZauberWert wert ) {

        final KonfliktBeschreibung beschreibung = new KonfliktBeschreibung();

        if ( held.hat( wert ) ) {

            // Grenzwerte testen
            final int mindeststufe;

            if ( held.hatVorgegeben( wert ) ) {

                mindeststufe = held.getMindeststufe( wert );

            } else {

                mindeststufe = 0;

            }

            final int hoechststufe = wert.getZauber().getHoechststufe( held );

            final int stufe = wert.getStufe();

            if ( ( stufe < mindeststufe ) || ( hoechststufe < stufe ) ) {

                beschreibung.einfuegen(
                        String.format( GRENZEN, mindeststufe, hoechststufe ) );
            }


        } else {

            // Der Held besitzt den Zauber nicht. Deswegen sind wir in diesem
            // else Teil gelandet ...
            if ( held.hatVorgegeben( wert ) ) {
                // ... er muss ihn aber besitzen
                beschreibung.einfuegen( VORGEGEBEN );
            }
        }

        return beschreibung;
    }

}
